/// <reference path="modules/shared/material.module.ts" />
import { BrowserModule } from '@angular/platform-browser';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { HttpClientModule, HTTP_INTERCEPTORS } from '@angular/common/http';
import { NgModule, CUSTOM_ELEMENTS_SCHEMA } from '@angular/core';
import { ReactiveFormsModule, FormsModule } from '@angular/forms';
import { AppRoutingModule } from './app-routing.module';
import { AuthLoggedIn, AuthGuard, AuthLoad, AuthGuardWithRefresh } from './core/auth-guard.service';
import { Globals } from './core/global';
import { HttpService } from './core/http.service';
import { JwtModule, JwtModuleOptions } from '@auth0/angular-jwt';
import { ModalService } from './core/modal.service';
import { SharedComponentModule } from './modules/shared/shared-component.module';
import { AppComponent } from './app.component';
import { LoginComponent } from './pages/login/login.component';
import { MainComponent } from './pages/main/main.component';
// import { MainAComponent } from './pages/main-a/main-a.component';
import { MainIComponent } from './pages/main-i/main-i.component';
// import { MainJComponent } from './pages/main-j/main-j.component';
// import { MainKComponent } from './pages/main-k/main-k.component';
// import { MainLComponent } from './pages/main-l/main-l.component';
// import { MainMComponent } from './pages/main-m/main-m.component';
// import { MainNComponent } from './pages/main-n/main-n.component';
import { MainOComponent } from './pages/main-o/main-o.component';
import { ReportPopupComponent } from './pages/report-popup/report-popup.component';

// import { DealerComponent } from './pages/dealer/dealer.component';
// import { DealerIComponent } from './pages/dealer-i/dealer-i.component';
// import { DealerOComponent } from './pages/dealer-o/dealer-o.component';

// import { VdmComponent } from './pages/vdm/vdm.component';
// import { PerformanceComponent } from './pages/performance/performance.component';
// import { ManagementComponent } from './pages/management/management.component';
import { PortalComponent } from './pages/portal/portal.component';
import { RoleListComponent } from './pages/portal/role-list.component';
import { MatSidenavModule } from '@angular/material/sidenav';
import { MaterialImportModule } from './modules/shared/material.module';
import { ChangePasswordComponent } from './pages/changepwd/changepwd.component';
// import { QVocComponent } from './pages/qvoc/qvoc.component';
// import { SyscusCommentComponent } from './pages/sys-cus-comment/sys-cus-comment.component';
// import { BPCSILocalSurvayComponent } from './pages/BP-CSI-local-survay/BP-CSI-local-survay.component';
// import { MenuDemoComponent } from './pages/demo/menu-demo/menu-demo.component';


// import { MatAutocompleteModule } from '@angular/material/autocomplete';
// import { MatFormFieldModule } from '@angular/material/form-field';
// import { MatInputModule } from '@angular/material/input'; 


export function jwtTokenGetter() {
  return localStorage.getItem("jwtToken") || '';
}

const JWT_Module_Options: JwtModuleOptions = {
  config: {
    tokenGetter: jwtTokenGetter,
    whitelistedDomains: ['']
  }
};

@NgModule({
  declarations: [
    AppComponent,
    LoginComponent,
    ChangePasswordComponent,
    MainComponent,
    // MainAComponent,
    MainIComponent,
    // MainJComponent,
    // MainKComponent,
    // MainLComponent,
    // MainMComponent,
    // MainNComponent,
    ReportPopupComponent,
    // VdmComponent,
    // PerformanceComponent,
    // ManagementComponent,
    PortalComponent,
    RoleListComponent,
    // MenuDemoComponent,
    // QVocComponent,
    // SyscusCommentComponent,
    MainOComponent,
    // DealerComponent,
    // DealerIComponent,
    // DealerOComponent,
    // BPCSILocalSurvayComponent
  ],
  imports: [
    BrowserModule,
    HttpClientModule,
    ReactiveFormsModule,
    BrowserAnimationsModule,
    MatSidenavModule,
    AppRoutingModule,
    SharedComponentModule,
    MaterialImportModule,

    FormsModule,
    JwtModule.forRoot(JWT_Module_Options),
  ],
  providers: [
    HttpService,
    AuthLoggedIn,
    AuthGuard,
    AuthGuardWithRefresh,
    AuthLoad,
    Globals,
    ModalService,
  ],
  bootstrap: [AppComponent],
  schemas: [CUSTOM_ELEMENTS_SCHEMA]
})
export class AppModule { }
