import { Component, OnInit, Input, Output, EventEmitter, OnChanges, Inject } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { FormControl } from '@angular/forms'
import { Observable } from 'rxjs';
import { map, startWith, debounceTime } from 'rxjs/operators';
import { MatDialogRef, MAT_DIALOG_DATA, MatDialog } from '@angular/material/dialog';


/**
 * How to use AutoCompleteQuerySearch
 * 1.create new tag name
 *    <auto-complete-query-search></auto-complete-query-search>
 * 2.add properties
 *    - config [Required][Input]
 *    - datacallback [Not-required][Output]
 * 
 * ***config property's model
 *  1. ColName: Key of object to show in screen.
 *  2. ColValue: Value for returning to callback
 *  3. PlaceholderText: A name that user understand what to do with this auto-complete
 *  4. SourceController
 *  5. SourceMethodName
 *  6. NoOfShowRecords
 */


@Component({
  selector: 'auto-complete-query-search',
  template: '<button mat-icon-button (click)="OpenDialog()"><i class="material-icons">arrow_drop_down</i></button>',
  styles: [``]
})

export class AutoCompleteQuerySearchComponent {

  constructor(public dialog: MatDialog) { }

  @Input() config: string;
  @Input() textInput: string = "";
  @Output() datacallback = new EventEmitter<string>();

  OpenDialog() {
    const dialogRef = this.dialog.open(AutoCompleteQuerySearchDialog, {
      width: '40%',
      data: {
        config: this.config,
        textInput: this.textInput,
        datacallback: this.datacallback
      }
    })

    dialogRef.afterClosed().subscribe(result => { })
  }

}

@Component({
  selector: 'auto-complete-query-search-dialog',
  templateUrl: './auto-complete-query-search.component.html',
  styleUrls: ['./auto-complete-query-search.component.css']
})
export class AutoCompleteQuerySearchDialog {

  public setting: any;
  private textInput: string = "";
  public selectedValue: any;

  myControl = new FormControl();
  filteredOptions: any = [];

  constructor(
    public dialogRef: MatDialogRef<AutoCompleteQuerySearchDialog>,
    @Inject(MAT_DIALOG_DATA) public dialogData: any,
    private http: HttpService
  ) { }

  ngOnInit() {
    this.setting = this.dialogData.config;
    this.textInput = this.dialogData.textInput || "";
    this.myControl.setValue(this.textInput);
  }

  GetDataByValue(value) {
    this.filteredOptions = [];
    this.http.get(this.setting.SourceController, this.setting.SourceMethodName, value, this.setting.NoOfShowRecords || 20)
      .subscribe(result => {
        this.filteredOptions = result.Table;
      })
  }

    ClickCallback() {
    this.dialogData.datacallback.emit(this.selectedValue);
    this.dialogRef.close();
  }

  OnCancelClick() {
    this.dialogRef.close();
  }

  SetValue(evt: any) {
    this.myControl.setValue(evt.option.value[this.setting.ColName]);
    this.selectedValue = evt.option.value[this.setting.ColValue];
  }
}
