import { Component, OnInit, Input, OnChanges, ContentChild, TemplateRef, Output, EventEmitter, ViewChild } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { MatTableDataSource } from '@angular/material/table';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import { CdkDragDrop, moveItemInArray } from '@angular/cdk/drag-drop';

/**
 * How to use the table?
 * you can just add element into yout html
 * @param {string} tableName Field TABLE_NAME.
 * @param {Array<any>} tableData Detail row list in table.
 * @event editClick($event) Send function name to this for Edit button action.
 * 
 * you can custom condition to show another value inside the column
 * @var {object} header Current header row
 * @var {object} data Current data row
 * @description<ng-template let-header="header" let-data="data">
 * @description--> insert your condition
 * @description</ng-template>
 */

@Component({
  selector: 'auto-table',
  templateUrl: './auto-table.component.html',
  styleUrls: ['./auto-table.component.css']
})
export class AutoTableComponent {
  public headerList: Array<any> = [];
  public dataList: any;
  public displayedColumns: string[] = [];
  public isCustomContent: boolean = false;

  @Input("tableName") tableName: string;
  @Input("tableData") tableData: Array<any> = [];
  @Output("editClick") editClick = new EventEmitter<any>()

  @ContentChild(TemplateRef, {static: false}) templateRef;
  @ViewChild(MatPaginator, {static: true}) paginator: MatPaginator;
  @ViewChild(MatSort, {static: true}) sort: MatSort;

  constructor(
    private http: HttpService
  ) { }

  ngOnInit(): void {
    this.getTableHeader();
  }

  ngOnChanges(): void {
    this.dataList = new MatTableDataSource<any>(this.tableData);
    this.dataList.paginator = this.paginator;
    this.dataList.sort = this.sort;
  }

  ngAfterContentInit(): void {
    if(this.templateRef) {
      this.isCustomContent = true;
    }
  }

  getTableHeader () {
    this.http.get("System", "GetHeaderGridView", this.tableName)
      .subscribe(result => {
        this.headerList = result.filter(v => v.VISIBILITY === true);

        this.displayedColumns = this.headerList.map(v => v.FIELD_NAME);
      })
  }

  editBtnClick(item: any) {
    this.editClick.emit(item);
  }
}
