import { Component, OnInit, Input, Output, EventEmitter, OnChanges, Inject } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { FormGroup, FormControl } from '@angular/forms'
import { Router } from '@angular/router';
import { Observable, config } from 'rxjs';
import { map, startWith, debounceTime } from 'rxjs/operators';
import { MatDialogRef, MAT_DIALOG_DATA, MatDialog } from '@angular/material/dialog';


/**
 * How to use AutoComplete
 * 1.create new tag name
 *    <auto-complete></auto-complete>
 * 2.add properties
 *    - config [Required]
 *    - data [Required]
 *    - datacallback [Required]
 * 
 * ***config property's model
 *  1. ColName: Key of object to show in screen.
 *  2. ColValue: Value for returning to callback (if this is not defined, return value will be a selected object.)
 *  3. PlaceholderText: A name that user understand what to do with this auto-complete
 *  4. MinLengthSearch
 */


@Component({
    selector: 'auto-complete',
    template: '<button mat-icon-button (click)="OpenDialog()"><i class="material-icons">arrow_drop_down</i></button>',
})

export class AutoCompleteComponent {

    constructor(public dialog: MatDialog) { }

    @Input() config: string;
    @Input() data: any;
    @Input() sWidth: string;
    @Output() datacallback = new EventEmitter<string>();



    OpenDialog() {
        const dialogRef = this.dialog.open(AutoCompleteDialog, {
            width: this.sWidth,
            data: {
                config: this.config,
                data: this.data,
                datacallback: this.datacallback
            }
        })

        dialogRef.afterClosed().subscribe(result => { })
    }

}

@Component({
    selector: 'auto-complete-dialog',
    templateUrl: './auto-complete.component.html',
    styleUrls: ['./auto-complete.component.css']
})
export class AutoCompleteDialog {

    public states: any;
    public setting: any;
    public selectedValue: any;
  public defaultMinLengthSearch: number = 2;

    myControl = new FormControl();
  filteredOptions: Observable<any>;

    constructor(
        public dialogRef: MatDialogRef<AutoCompleteDialog>,
        @Inject(MAT_DIALOG_DATA) public dialogData: any
    ) { }

    ngOnInit() {
        this.setting = this.dialogData.config;

        this.states = this.dialogData.data
        //console.log('this.states ', this.states);
        this.filteredOptions = this.myControl.valueChanges
            .pipe(
                startWith(''),
        debounceTime(500),
        map(value => value.length >= (this.setting.MinLengthSearch || this.defaultMinLengthSearch) && this._filter(value))
            );
    }

    _filter(value: string): string[] {
        const filterValue = value.toLowerCase();
        let prop = this.setting.ColName;
        return this.states.filter(option => option[prop].toLowerCase().includes(filterValue));
    }

    ClickCallback() {
        this.dialogData.datacallback.emit(this.selectedValue);
        this.dialogRef.close();
    }

    OnCancelClick() {
        this.dialogRef.close();
    }

    SetValue(evt: any) {
        this.myControl.setValue(evt.option.value[this.setting.ColName]);
    this.selectedValue = (this.setting.ColValue) ? evt.option.value[this.setting.ColValue] : evt.option.value;
    }
}
