import { Component, OnInit } from '@angular/core';
import * as d3 from 'd3';

@Component({
    selector: 'graphchartthree',
    templateUrl: './graph-chart-three.component.html',
    styleUrls: ['./graph-chart-three.component.css']
})
export class GraphChartThreeComponent implements OnInit {


    constructor() { }
    private labelxScalemonth12 = ["Apr", "May", "Jun"];
    private dataInput: dataJsonMonthOfYear[] = [
        {
            monthOf: 'Apr',
            value: 81.2,
            taget: 10,
        },
        {
            monthOf: 'May', 
            value: 85.1,
            taget: 10,
        },
        {
            monthOf: 'Jun',
            value: 89.4,
            taget: 10,
        }
    ]; 

    ngOnInit() {
        this.createBarChartV1(this.dataInput);
    }

    private createBarChartV1(sample: dataJsonMonthOfYear[]): void {
        //Set Text of x scale, format is 'Jan','Feb' that it must math "string" of sample.monthOf from dataJsonMonthOfYear
        const month12 = this.labelxScalemonth12;

        const svg = d3.select('#chartBart04');

        const MARGIN = 80;
        const WIDTH = 1000 - 2 * MARGIN;
        const HEIGHT = 600 - 2 * MARGIN;

        const chart = svg.append('g')
            .attr('transform', `translate(${MARGIN}, ${MARGIN})`);

        const xScale = d3.scaleBand()
            .range([0, WIDTH])
            .domain(month12.map((s) => s))
            .padding(0.4)


        //set dynamic range of yScale for optimize chart to cool.
        let idomainMax = d3.max(sample, d => d.value);
        idomainMax = idomainMax + ((idomainMax * 5) / 100);//+5%
        let idomainMin = d3.min(sample, d => d.value);
        idomainMin = idomainMin - ((idomainMin * 5) / 100);//+5%    
        let yScale = d3.scaleLinear()
            .range([HEIGHT, 0])
            //.domain([idomainMin, idomainMax]);
            .domain([60, 100]);

        chart.append('g')
            .attr('transform', `translate(0, ${HEIGHT})`)
            .call(d3.axisBottom(xScale));

        //กำหนดจำนวนบาร์สเกล
        chart.append('g')
            .call(d3.axisLeft(yScale).ticks(5));

        //get range of grid line from yScale
        const makeYLines = () => d3.axisLeft(yScale);

        //create grid line y position allow range that get.
        chart.append('g')
            .attr('class', 'grid')
            .call(makeYLines().tickSize(-WIDTH).tickFormat((d) => ""));

        //create bar by draw rect at chart.
        const barGroups = chart.selectAll()
            .data(sample)
            .enter()
            .append('g')
        barGroups
            .append('rect')
            .attr('class', 'bar')
            .attr('x', (g) => xScale(g.monthOf))
            .attr('y', (g) => yScale(g.value))
            .attr('height', (g) => HEIGHT - yScale(g.value))
            .attr('width', xScale.bandwidth())
            .on('mouseenter', function (actual, i) {
                d3.selectAll('.value')
                const y = yScale(actual.value)
                var line = chart.append('rect')
                    .transition()
                    .duration(500)
                    .attr('id', 'showzone')
                    .attr('x', 0)
                    .attr('y', 0)
                    .attr('width', WIDTH)
                    .attr('height', y)
                    .attr("fill", "blue")
                    .attr('opacity', 0.1)
                    ;

            })
            .on('mouseleave', function () {
                d3.selectAll('.value')
                    .attr('opacity', 1)

                d3.select(this)
                    .transition()
                    .duration(300)
                    .attr('opacity', 1)
                    .attr('x', (a: dataJsonMonthOfYear) => xScale(a.monthOf))
                    .attr('width', xScale.bandwidth())
                  
                chart.selectAll('#showzone').remove()
            })
        

        //Text of value data show at on bar.  //ตั้งค่าการโชว์แบบข้อความ
        barGroups.append('text')
            //.attr('class', 'value')
            .attr('class', 'txtValue')
            .attr('x', (a) => xScale(a.monthOf) + xScale.bandwidth() / 2)
            .attr('y', (a) => yScale(a.value) - 5)
            .attr('text-anchor', 'middle')
          //.text((a) => `${a.value}`)
            .text((a) => `${a.value}%`)

        
            
     
        let target1: number;
        target1 = 90;
        var line = svg.append('line')
              .attr('id', 'limit')
              .attr('x1', MARGIN + 610)
              .attr('y1', (d) => yScale(target1) + MARGIN)
              .attr('x2', WIDTH - 30 )
              .attr('y2', (d) => yScale(target1) + MARGIN)          
        svg.append("text")
            .attr('class', 'linetext')
            .attr("transform", "translate(" + (WIDTH - 20) + "," + (yScale(target1) + MARGIN) + ")")
            .attr("dy", ".35em")
            .attr("text-anchor", "start")
            .style("fill", "red")
            .text(target1 + "%");

        let target2: number;
        target2 = 86;
        var line = svg.append('line')
            .attr('id', 'limit')
            .attr('x1', MARGIN + 360)
            .attr('y1', (d) => yScale(target2) + MARGIN)
            .attr('x2', WIDTH - 280)
            .attr('y2', (d) => yScale(target2) + MARGIN)
        svg.append("text")
            .attr('class', 'linetext')
            .attr("transform", "translate(" + (WIDTH - 270) + "," + (yScale(target2) + MARGIN) + ")")
            .attr("dy", ".35em")
            .attr("text-anchor", "start")
            .style("fill", "red")
            .text(target2 + "%");

        let target3: number;
        target3 = 82;
        var line = svg.append('line')
            .attr('id', 'limit')
            .attr('x1', MARGIN + 120)
            .attr('y1', (d) => yScale(target3) + MARGIN)
            .attr('x2', WIDTH - 530)
            .attr('y2', (d) => yScale(target3) + MARGIN)
        svg.append("text")
            .attr('class', 'linetext')
            .attr("transform", "translate(" + (WIDTH - 520) + "," + (yScale(target3) + MARGIN) + ")")
            .attr("dy", ".35em")
            .attr("text-anchor", "start")
            .style("fill", "red")
            .text(target3 + "%");

    }

}

export interface dataJsonMonthOfYear {
    monthOf: string,
    value: number,
    taget: number
}
export interface LegenModel {
    legend: string,
    color: string
}
