import { Component, OnInit, OnChanges, Input, SimpleChanges } from "@angular/core";
import * as D3 from 'd3';

@Component({
    selector:'bar-chart-3d',
    templateUrl:'./bar-chart-3d.component.html',
    styleUrls:['./bar-chart-3d.component.css']
})

export class BarChart3DComponent implements OnInit,OnChanges{

    constructor(){

    }

    @Input("colors") colors : any = ["#364c6e","#223046"] ;
    @Input("title") title : string = "Demo BarChart 3D";
    @Input("WidthSizeChart") WidthSizeChart: any;
    @Input("HeigthSizeChart") HeigthSizeChart: any;
    @Input("data") data : any = [];
    @Input("ID") ID : string ;

    private d3 : any = D3;
    private svg: any;
    private yScale: any;
    private xScale: any;
    private xAxis: any;
    private yAxis: any;
    private chart: any;
    private barGroups: any;
    private margin = { top: 30, right: 20, bottom: 40, left: 55 }
    private width = 600 - this.margin.left - this.margin.right
    private height = 350 - this.margin.top - this.margin.bottom;

    ngOnInit(){

      if (this.WidthSizeChart) {
        this.width = this.WidthSizeChart - this.margin.left - this.margin.right
      }
      if (this.HeigthSizeChart) {
        this.height = this.HeigthSizeChart - this.margin.top - this.margin.bottom
      }

      if(this.data.length == 0 ){
        this.data = [{ catagory: 'sss', Value: 9.5 }, 
                    { catagory: 'ggg', Value: 9.6 },
                    { catagory: 'XXX', Value: 9.9 }, 
                    { catagory: 'ZZZ', Value: 8.3 },
                    { catagory: 'ccc', Value: 8 }, 
                    { catagory: 'bbb', Value: 0 },
                    { catagory: 'www', Value: 9 }, 
                    { catagory: 'rrr', Value: 7.9 },
                    { catagory: 'ttt', Value: 8.9 }, 
                    { catagory: 'lll', Value: 9.2 },
                    { catagory: 'vvv', Value: 8.8 }, 
                    { catagory: 'uuu', Value: 9.5 }]
      }

      //this.colors = ["#364c6e","#223046"]
      //this.title = "Demo BarChart 3D"
      //this.createchart()
      
    }

   
  ngOnChanges(changes: SimpleChanges): void {

    //if (!this.data || this.data.length == 0) { return; }
    //this.createChart(this.data)

    if (!changes["data"].isFirstChange()) {
      this.d3.select("#" + 'chart3d' + this.ID).remove();
      this.createchart(this.ID);
    }

  }

    ngAfterViewInit() {
      this.createchart(this.ID)
      
    }

    createchart(id){


      this.svg = this.d3.select('#' + id).append('svg')
      .attr("id", 'chart3d' + id)
      .attr('width', this.width + this.margin.left + this.margin.right)
      .attr('height', this.height + this.margin.top + this.margin.bottom)
      .style("background", "rgb(255, 255, 255");

      this.yScale = this.d3.scaleLinear()
          .range([this.height - 20, 0])

        this.xScale = this.d3.scaleBand()
          .range([0, this.width])
          .padding(0.2)

        this.xAxis = this.d3.axisBottom(this.xScale);

        this.yAxis = this.d3.axisLeft(this.yScale).ticks(6).tickFormat(d => d);

        this.chart = this.svg.append('g')
          .attr('transform', `translate(${this.margin.left}, ${this.margin.top + 10})`);

        this.chart.append('g')
          .attr("class", "y axis")
          .attr('transform', `translate(0, ${this.height - this.height + 20})`)
          .call(this.yAxis)

        this.chart.append('g')
          .attr("class", "xAxis")
          .attr('transform', `translate(0, ${this.height})`)
          .call(this.xAxis)
          .selectAll("text")
          .style("text-anchor", "end")

        let min = this.generateYminDomain(this.data)
        let max = this.generateYmaxDomain(this.data)

        this.yScale.domain([min, max])
        this.xScale.domain(this.data.map((d) => d.catagory))

        this.chart
          .select(".y")
          .call(this.yAxis)

        this.chart
          .select(".xAxis")
          .attr('transform', `translate(0, ${this.height})`)
          .call(this.xAxis)
          .selectAll("text")
          .style("text-anchor", "middle")


          this.barGroups = this.chart.selectAll(".bar")
          .remove()
          .exit()
          .data(this.data)


          //create range of grid line from yScale
        this.svg.select('.grid').remove();//remove then create new because each data had scale of it.
        var makeYLines = () => this.d3.axisLeft(this.yScale);
        this.chart.append('g')
          .attr('class', 'grid')
          .attr('transform', `translate(0, ${this.height - this.height + 20})`)
          .call(makeYLines().ticks(5).tickSize(-this.width).tickFormat((d) => ""));


        var rh = 100, rw = 20, ang=45;

        let h = this.height
        let y = this.yScale
        let x = this.xScale

          //forward
          this.barGroups.enter().append("rect")
          .attr('x', (d) => this.xScale(d.catagory) + 0.5 )
          .attr('y', (d) => this.yScale(d.Value) + 26.5)
          //.attr('height',function (d) { return (h - 20) - y(d.Value)  })
          .attr('height', (d) => d.Value == 0 ? 0 : (this.height - 26.5) - this.yScale(d.Value) )
          .attr('width',(d) => d.Value == 0 ? 0 : this.xScale.bandwidth() / 2)
          .attr("fill", this.colors[0])
          .on("mouseover", (d) => this.handleMouseOver(d, 'Value'))
          .on("mouseout", (d) => this.handleMouseOut(d));
          
          //top
          this.barGroups.enter().append("rect")          
          // .attr("x", (d) => this.xScale(d.catagory) )
          // .attr("y", 10)
          .attr("fill", this.colors[1])
          .attr("width",(d) => d.Value == 0 ? 0 : this.xScale.bandwidth() / 2)
          .attr("height",(d) => d.Value == 0 ? 0 : 6.5)
          .attr ("transform",(d) => "translate ("+(this.xScale(d.catagory) + 10.1)+","+((this.yScale(d.Value)+20.2))+") skewX("+-55+")");
          
          //side
          this.barGroups.enter().append("rect")         
          // .attr("x", (d) => this.xScale(d.catagory))
          // .attr("y", 0)
          .attr("fill", this.colors[1])
          .attr("width",(d) => d.Value == 0 ? 0 : 10)
          .attr('height', (d) => d.Value == 0 ? 0 : (this.height - 25) - this.yScale(d.Value))
          .attr ("transform",(d) => "translate ("+(this.xScale(d.catagory) + (this.xScale.bandwidth() / 2))+","+(this.yScale(d.Value) + (this.margin.top - 3.8))+") skewY("+-30+")");
          
          this.barGroups = this.chart.selectAll(".textvalue")
          .remove()
          .exit()
          .data(this.data)

        this.barGroups
          .enter()
          .append('text')
          .attr('class', 'textvalue')
          .attr('x', (d) => this.xScale(d.catagory) + this.xScale.bandwidth() / 3)
          .attr('y', (d) => this.yScale(d.Value) + 5)
          .attr('text-anchor', 'middle')
          .attr("font-size", "12px")
          .attr("fill", "#000000")
        /*.text((d) => d.Value > 0 ? d.Value : '' )*/

        this.svg.select('.textH').remove()

        this.chart.append('text')
          .attr('class', 'textH')
          .attr('x', this.width / 2)
          .attr('y', -15)
          .attr("font-size", "20")
          .style("font-weight", 'bold')
          .style("fill", "#595959")
          .attr('text-anchor', 'middle')
          .text(this.title)

      let chart = this.chart
  }

  handleMouseOver = (d, Value) => {
    this.chart.append("text")
      .attr('class', 'textHover')
      .attr('x', this.xScale(d.catagory) + this.xScale.bandwidth() / 3)
      .attr('y', this.yScale(d.Value))
      .attr("font-size", "7pt")
      .style("font-weight", 'bold')
      .style("fill", "#000000")
      .text(d[Value] + '%')
  }

  handleMouseOut = (d) => {
    this.chart.select('.textHover').remove()
  }

    generateYmaxDomain(data: any) {

      var yMaxDomain: number = 0;
      yMaxDomain = this.d3.max(data, d => Number(d.Value))
      yMaxDomain = yMaxDomain + ((yMaxDomain * 5) / 100);
      return yMaxDomain
    }

    generateYminDomain(data: any) {
      var yMinDomain: number = 0;
      // yMinDomain = this.d3.min(data, d => Number(d.Value))

      // if(yMinDomain <= 2){
      //   yMinDomain = 0
      // }else{
      //   yMinDomain = yMinDomain - ((yMinDomain * 5) / 100)
      // }
      return yMinDomain
    }
  





}
