import { Component, OnInit, OnChanges, ElementRef, Input, ViewChild, ViewEncapsulation, SimpleChanges, Output, EventEmitter } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import * as D3 from 'd3';
//import { stack } from 'd3';

@Component({
  selector: 'bar-chart-line-value-i',
  templateUrl: './bar-chart-line-value-i.component.html',
  styleUrls: ['./bar-chart-line-value-i.component.css']
})
export class BarChartLineValue_IComponent implements OnInit, OnChanges {

  constructor(
    private http: HttpService
  ) { }

  @Input() header: string = 'Demo';
  @Input("ID") ID: string;
  @Input() showLine: boolean = true;
  @Input() showLegen: boolean = true;
  @Input() Width: any;
  @Input() Height: any;
  @Input() showPercent: string = '';
  @Input() Data: any = [];
  @Input() LegenList: any = [{ legend: "aaaaaaaaaaaaaaa", color: '#248623' }, { legend: "bbbbbbbbbbbbbbbb", color: '#6395ec' }]
  @ViewChild('chart', { static: true }) chartContainer: ElementRef;
  @Output("barClick") barClick: EventEmitter<any> = new EventEmitter<any>();

  private svg: any;
  private yScale: any;
  private yScale1: any;
  private xScale: any;
  private xAxis: any;
  private yAxis: any;
  private yAxis1: any;
  private chart: any;
  private barGroups: any;
  private margin = { top: 30, right: 20, bottom: 20, left: 35 }
  private width = 600 - this.margin.left - this.margin.right
  private height = 350 - this.margin.top - this.margin.bottom;

  private d3: any = D3;

  ngOnInit() {
    //this.setUpChart(this.ID)
    // this.showLine = true;
    // this.showLegen = true

    if (this.Width) {
      this.width = this.Width - this.margin.left - this.margin.right
    }

    if (this.Height) {
      this.height = this.Height - this.margin.top - this.margin.bottom
    }

    if (this.Data.length == 0) {
      this.Data = [];
    }
  }

  ngOnChanges(changes: SimpleChanges): void {

    if (!changes["Data"].isFirstChange()) {
      this.d3.select("#" + 'barchart' + this.ID).remove();
      this.setUpChart(this.ID)
    }

  }

  ngAfterViewInit() {
    this.setUpChart(this.ID)
    //console.log('ngAfterViewInit')
  }



  setUpChart(id) {


    this.svg = this.d3.select("#" + id).append('svg')
      .attr("id", 'barchart' + id)
      .attr('width', this.width + this.margin.left + this.margin.right)
      .attr('height', this.height + this.margin.top + this.margin.bottom + 20)
      .style("background", "rgb(255, 255, 255");


    this.xScale = this.d3.scaleBand()
      .range([this.margin.left, this.width - this.margin.right])
      .padding(0.3)

    this.yScale = this.d3.scaleLinear()
      .rangeRound([this.height - this.margin.bottom, this.margin.top])

    this.yScale1 = this.d3.scaleLinear()
      .range([this.height - this.margin.bottom, this.margin.top])
    //.domain([0, 120])

    this.xAxis = this.d3.axisBottom(this.xScale);

    this.yAxis = this.d3.axisLeft(this.yScale);

    //this.yAxis1 = this.d3.axisRight(this.yScale1);

    this.chart = this.svg.append('g')
      .attr('transform', `translate(${this.margin.left / 2}, ${this.margin.top})`);

    this.chart.append('g')
      .attr("class", "x-axis")
      .attr("transform", `translate(0,${this.height - this.margin.bottom})`)
      .call(this.xAxis)

    this.chart.append('g')
      .attr("class", "y-axis")
      .attr("transform", `translate(${this.margin.left},0)`)
      .call(this.yAxis)

    //this.chart.append('g')
    //.attr("class", "y-axis1")
    //.attr("transform", `translate(${this.width - this.margin.right},0)`)
    //.call(this.yAxis1);

    this.createChart(this.Data)
  }

  createChart(data) {

    var yMinDomain = this.generateYminDomain(data)
    var yMaxDomain = this.generateYmaxDomain(data)

    this.yScale.domain([yMinDomain, yMaxDomain])
    this.xScale.domain(data.map((d) => d.catagory))
    this.yScale1.domain([0, 110])

    this.chart.selectAll(".y-axis").transition().duration(5)
      .call(this.d3.axisLeft(this.yScale).ticks(5))

    this.chart.selectAll(".x-axis").transition().duration(5)
      .call(this.d3.axisBottom(this.xScale).tickSizeOuter(0))



    var defs = this.svg.append("defs");

    var filter = defs.append("filter")
      .attr("id", "drop-shadow")
      .attr("height", "150%");

    filter.append("feGaussianBlur")
      .attr("in", "SourceAlpha")
      .attr("stdDeviation", 5)
      .attr("result", "blur");

    filter.append("feOffset")
      .attr("in", "blur")
      .attr("dx", 5)
      .attr("dy", 5)
      .attr("result", "offsetBlur");

    var feMerge = filter.append("feMerge");

    feMerge.append("feMergeNode")
      .attr("in", "offsetBlur")
    feMerge.append("feMergeNode")
      .attr("in", "SourceGraphic");

    this.barGroups = this.chart.selectAll(".bar")
      .remove()
      .exit()
      .data(data)

    this.barGroups
      .enter()
      .append("rect")
      .merge(this.barGroups)
      .attr("class", "bar")
      .transition().duration(150)
      .attr('x', (d) => this.xScale(d.catagory))
      .attr('y', (d) => this.yScale(d.Value1))
      .attr('height', (s) => (this.height - 20) - this.yScale(s.Value1))
      .attr('width', this.xScale.bandwidth())
      .attr("fill", (d) => this.LegenList[1].color)
    //.style("filter", "url(#drop-shadow)")

    this.barGroups = this.chart.selectAll(".textvalue")
      .remove()
      .exit()
      .data(data)

    this.barGroups
      .enter()
      .append('text')
      .attr('class', 'textvalue')
      .attr('x', (d) => this.xScale(d.catagory) + this.xScale.bandwidth() / 2)
      .attr('y', (d) => this.yScale(d.Value1) + 10)
      .attr('text-anchor', 'middle')
      .attr("font-size", "8pt")
      .attr("fill", "#000000")
      .text((d) => d.Value1 > 0 ? d.Value1 : '')

    this.barGroups = this.chart.selectAll(".textValue2")
      .remove()
      .exit()
      .data(data)

    this.barGroups
      .enter()
      .append("text")
      .attr("class", "textValue2")
      .attr("text-anchor", "middle")
      .attr("font-size", "8pt")
      .transition().duration(5)
      .attr("x", d => (this.xScale(d.catagory) + this.xScale.bandwidth() / 2))
      .attr("y", d => (this.yScale(d.Value2) - 9))
      .text(d => d.Value2 > 0 ? d.Value2 + ' ' + this.showPercent : '')


    if (this.showLine) {
      this.CreateLine(data)
    }

    if (this.showLegen) {
      this.CreateLegen(data)
    }

    this.svg.select('.textH').remove()
    this.chart.append('text')
      .attr('class', 'textH')
      .attr('x', this.width / 2)
      .attr('y', 0)
      .attr("font-size", "12pt")
      .attr('text-anchor', 'middle')
      .style("font-weight", 'bold')
      .style("fill", "#595959")
      .text(this.header)


    //Add event
    if (this.barClick) {
      // var bars = this.d3.selectAll(".textValue2")
      var circles = this.d3.selectAll(".circle");

      circles.style("cursor", "pointer")
        .on("click", (d) => {
          this.barClick.emit(d);
        })

      circles.on("mouseover", (d, i) => {
        let thisNode = d3.select(circles.nodes()[i]);
        thisNode.attr("r", 8)
      })
      circles.on("mouseout", (d, i) => {
        let thisNode = d3.select(circles.nodes()[i]);
        thisNode.attr("r", 4)
      })

    }

  }

  CreateLine(data) {

    var line = this.d3.line()
      .x((d, i) => this.xScale(d.catagory) + this.xScale.bandwidth() / 2)
      .y((d) => this.yScale(d.Value2))

    this.chart.append("path")
      .attr("fill", "none")
      .attr("stroke-width", 1.5)
      .attr("stroke", this.LegenList[0].color)
      .attr("d", line(data));

    this.chart.selectAll("dot")
      .data(data)
      .enter()
      .append("circle")
      .style("fill", this.LegenList[0].color)
      .attr("class", "circle")
      .attr("r", 4)
      .attr("cx", (d) => (this.xScale(d.catagory) + this.xScale.bandwidth() / 2))
      .attr("cy", (d) => this.yScale(d.Value2))

  }


  CreateLegen(data) {
    let legendList = this.LegenList
    let height = this.height;
    let width = this.width;

    var nodeWidth = (d) => d.getBBox().width;
    //const cScale = this.d3.scaleOrdinal();

    const legend = this.svg.append('g')
      .attr('class', 'legend')
      .attr('transform', 'translate(0,0)');

    const lg = legend.selectAll('g')
      .data(legendList)
      .enter()
      .append('g')
      .attr('transform', (d, i) => `translate(${i * 100},${height + 15})`);

    lg.append('rect')
      .style('fill', d => d.color)
      .attr('x', 35)
      .attr('y', (this.height + this.margin.top + this.margin.bottom + this.margin.left))
      .attr('width', 50)
      .attr('height', 10);

    lg.append('text')
      //.style('font-family', 'Georgia')
      .style('font-size', '8pt')
      .attr('x', 95)
      .attr('y', (this.height + this.margin.top + this.margin.bottom + this.margin.left + 10))
      .text(d => d.legend);

    let offset = 0;
    lg.attr('transform', function (d, i) {
      let x = offset;
      offset += nodeWidth(this) + 10;
      return `translate(${x},${-40})`;
    });

    legend.attr('transform', function () {
      return `translate(${(width - nodeWidth(this)) / 2},${0})`
    });
  }

  generateYmaxDomain(data: any) {

    var yMaxDomain: number = 0;
    //yMaxDomain = 100
    yMaxDomain = this.d3.max(data, d => Number(d.Value1) > Number(d.Value2) ? Number(d.Value1) : Number(d.Value2))
    yMaxDomain = yMaxDomain + ((yMaxDomain * 5) / 100);


    return yMaxDomain
  }

  generateYminDomain(data: any) {
    var yMinDomain: number = 0;
    yMinDomain = this.d3.min(data, d => Number(d.Value1) < Number(d.Value2) ? Number(d.Value1) : Number(d.Value2))
    yMinDomain = yMinDomain - ((yMinDomain * 5) / 100)
    return yMinDomain
  }



}





