import { Component, OnInit, OnChanges, ElementRef, Input, ViewChild, ViewEncapsulation, SimpleChanges } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import * as d3 from 'd3';
//import { stack } from 'd3';

@Component({
  selector: 'bar-chart',
  templateUrl: './bar-chart.component.html',
  styleUrls: ['./bar-chart.component.css']
})
export class BarChartComponent implements OnInit, OnChanges {

  constructor(
    private http: HttpService
  ) { }

  @Input() Showtable: boolean = false;
  @Input() TypeData: string;
  @Input() TypeChart: string;
  @Input() serviceName: string;
  @Input() header: string;
  @Input("ID") ID: string;
  @Input() showLine: boolean = false;
  //add by BF.24/09/2019
  @Input() yAxisMax: number;
  @Input() yAxisMin: number = 0;
  @Input() FY: string;
  @Input() QTR: string;
  @Input("xScaleSizeChartPayment") xScaleSizeChartPayment: any;
  @Input("yScaleSizeChartPayment") yScaleSizeChartPayment: any;
  //end add BF.24/09/2019
  @ViewChild('chart', { static: true }) chartContainer: ElementRef;

  public Columns: string[] = [];
  public headerList: Array<any> = [];
  public dataList: any;

  private svg: any;
  private yScale: any;
  private xScale: any;
  private xAxis: any;
  private yAxis: any;
  private chart: any;
  private barGroups: any;
  private data: any;
  private margin = { top: 30, right: 20, bottom: 40, left: 55 }
  private width = 600 - this.margin.left - this.margin.right
  private height = 350 - this.margin.top - this.margin.bottom;

  ngOnInit() {
    this.setUpChart(this.ID)

    this.width = this.xScaleSizeChartPayment - this.margin.left - this.margin.right;
    if (this.width == null) {
      this.width = 600 - this.margin.left - this.margin.right
    }
    this.height = this.yScaleSizeChartPayment - this.margin.top - this.margin.bottom;
    if (this.height == null) {
      this.height = 350 - this.margin.top - this.margin.bottom
    }

  }

  ngOnChanges(changes: SimpleChanges): void {


    if (!changes["serviceName"].isFirstChange() || !changes["QTR"].isFirstChange() || !changes["FY"].isFirstChange()) {
      d3.select("#" + 'barchart' + this.ID).remove();
      this.setUpChart(this.ID)
    }

  }

  ngAfterViewInit() {
    //this.setUpChart(this.ID)
    //console.log('ngAfterViewInit')
  }

  getData() {

    if (this.QTR != '') {
      this.http.get("System", this.serviceName, this.FY, this.QTR)
        .subscribe(result => {
          this.data = result.Table;
          this.createChart(this.data)
          this.dataList = this.data
          console.log("DATAAAAAAA", this.data)
        })
    }
    if (this.QTR == '') {
      let FirstCol = '%PASS'
      this.http.get("System", this.serviceName, this.FY)
        .subscribe(result => {
          if (this.TypeData == 'SSI') {
            this.data = result.Table;
            this.createChart(this.data)
            this.dataList = this.data
            if (this.Showtable) {
              this.headerList = this.generateHearderList(this.dataList);
              this.Columns = this.dataList.map(v => v.catagory.substring(0, 3));
              this.Columns.unshift(FirstCol)
            }

          }
          else {
            this.data = result.Table1;
            this.createChart(this.data)
            this.dataList = this.data
            if (this.Showtable) {
              this.headerList = this.generateHearderList(this.dataList);
              this.Columns = this.dataList.map(v => v.catagory.substring(0, 3));
              this.Columns.unshift(FirstCol)
            }
          }
        })
    }

  }

  generateData(data) {
    let newData = [];
    for (var key in data) {
      if (key != 'DATA_MONTH' && key != 'OUTLET_ID') {
        newData.push({ catagory: key, Value: data[key] })
      }
    }
    return newData
  }

  generateHearder(text) {
    let month = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
    let newtext = text.substring(4)
    return month[Number(newtext) - 1]

  }


  setUpChart(id) {


    let data: valueModel[];

    data = [{ catagory: 'XXX', Value: 50 }, { catagory: 'ZZZ', Value: 90 }]


    this.svg = d3.select('#' + id).append('svg')
      .attr("id", 'barchart' + id)
      .attr('width', this.width + this.margin.left + this.margin.right)
      .attr('height', this.height + this.margin.top + this.margin.bottom)
      .style("background", "rgb(255, 255, 255");

    this.yScale = d3.scaleLinear()
      .range([this.height - 20, 0])

    this.xScale = d3.scaleBand()
      .range([0, this.width])
      .padding(0.2)

    this.xAxis = d3.axisBottom(this.xScale);

    this.yAxis = d3.axisLeft(this.yScale).ticks(5).tickFormat(d => this.TypeChart == 'Value' ? d + '' : d + '%');;

    this.chart = this.svg.append('g')
      .attr('transform', `translate(${this.margin.left}, ${this.margin.top + 10})`);

    this.chart.append('g')
      .attr("class", "y axis")
      .attr('transform', `translate(0, ${this.height - this.height + 20})`)
      .call(this.yAxis)

    this.chart.append('g')
      .attr("class", "xAxis")
      .attr('transform', `translate(0, ${this.height})`)
      .call(this.xAxis)
      .selectAll("text")
      .style("text-anchor", "end")


    this.getData();
  }

  createChart(gData) {



    let data: any[];

    data = gData

    var yMinDomain = this.generateYminDomain(data)
    var yMaxDomain = this.generateYmaxDomain(data)

    this.yScale.domain([yMinDomain, yMaxDomain + 10])
    this.xScale.domain(data.map((d) => d.catagory))

    this.chart
      .select(".y")
      .call(this.yAxis)

    this.chart
      .select(".xAxis")
      .attr('transform', `translate(0, ${this.height})`)
      .call(this.xAxis)
      .selectAll("text")
      .style("text-anchor", "middle")
    //.attr("dx", "-.8em")
    //.attr("dy", ".15em")
    //.attr("transform", (d) => "rotate(-65)");

    //create range of grid line from yScale
    this.svg.select('.grid').remove();//remove then create new because each data had scale of it.
    var makeYLines = () => d3.axisLeft(this.yScale);
    this.chart.append('g')
      .attr('class', 'grid')
      .attr('transform', `translate(0, ${this.height - this.height + 20})`)
      .call(makeYLines().ticks(5).tickSize(-this.width).tickFormat((d) => ""));

    this.barGroups = this.chart.selectAll(".bar")
      .remove()
      .exit()
      .data(data)

    this.barGroups
      .enter()
      .append("rect")
      .merge(this.barGroups)
      .attr("class", "bar")
      .transition().duration(150)
      .attr('x', (d) => this.xScale(d.catagory))
      .attr('y', (d) => this.yScale(d.Value) + 20)
      .attr('height', (s) => (this.height - 20) - this.yScale(s.Value))
      .attr('width', this.xScale.bandwidth())
      .attr("fill", function (d) {

        return "#538ed5";

      })

    this.barGroups = this.chart.selectAll(".textvalue")
      .remove()
      .exit()
      .data(data)

    this.barGroups
      .enter()
      .append('text')
      .attr('class', 'textvalue')
      .attr('x', (d) => this.xScale(d.catagory) + this.xScale.bandwidth() / 2)
      .attr('y', (d) => this.yScale(d.Value) + 15)
      .attr('text-anchor', 'middle')
      .attr("font-size", "12px")
      .attr("fill", "#000000")
      .text((d) => this.TypeChart == 'Value' ? d.Value : d.Value + '%')


    if (this.showLine) {
      this.CreateLine(data)
    }
    this.svg.select('.textH').remove()

    this.chart.append('text')
      .attr('class', 'textH')
      .attr('x', this.width / 2)
      .attr('y', -20)
      .attr("font-size", "20px")
      .attr('text-anchor', 'middle')
      .text(this.header)

  }

  CreateLine(data) {

    let target: number;
    target = data[0].TARGET
    this.chart.selectAll('#limit').remove()

    var line = this.chart.append('line')
      .attr('id', 'limit')
      .attr('x1', 20)
      //.attr('y1', (d) => height - this.yScale(yMaxDomain) - target)
      .attr('y1', (d) => this.yScale(target) + 20)
      .attr('x2', this.width - 30)
      //.attr('y2', (d) => height - this.yScale(yMaxDomain) - target)
      .attr('y2', (d) => this.yScale(target) + 20)

    this.svg.select('.linetext').remove()

    this.chart.append("text")
      .attr('class', 'linetext')
      .attr("transform", "translate(" + (this.width - 20) + "," + (this.yScale(target) + 19) + ")")
      .attr("dy", ".35em")
      .attr("text-anchor", "start")
      .style("fill", "red")
      .text(target);

  }

  generateYmaxDomain(data: valueModel[]) {

    var yMaxDomain: number;

    if (this.TypeChart == 'Value') {
      //yMaxDomain = 100
      yMaxDomain = d3.max(data, d => Number(d.Value))
      //yMaxDomain = yMaxDomain + ((yMaxDomain * 5) / 100);
    } else {

      yMaxDomain = 100;
    }

    return yMaxDomain
  }

  generateYminDomain(data: valueModel[]) {
    var yMinDomain: number = 0;
    //var yMinDomain = d3.min(data, d => Number(d.Value))
    //yMinDomain = yMinDomain - ((yMinDomain * 5) / 100)
    return yMinDomain
  }

  generateHearderList(data) {
    let newData = [];
    for (var key in data[0]) {
      if (key != 'FY' && key != 'catagory' && key != 'Value') {
        newData.push(key)
      }

    }
    return newData
  }


}


export interface ChartData {
  Values: Array<valueModel>;
  Legend: Array<legendModel>
}


export interface valueModel {
  catagory: string;
  Value: number;
  //Value2: number;kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk
  //Value3: number;
}

export interface legendModel {
  legend: string;
  color: string;

}



