import { Component, OnInit, OnChanges, ElementRef, Input, ViewChild, ViewEncapsulation, SimpleChanges } from '@angular/core';
import * as D3 from 'd3';
//import { daLocale } from 'ngx-bootstrap/chronos/i18n/da';
import { HttpService } from 'src/app/core/http.service';


@Component({
  selector: 'bar-with-line-single-i',
  templateUrl: './bar-with-line-single-i.component.html',
  styleUrls: ['./bar-with-line-single-i.component.css']
})
export class BarWithLineSingleIComponent implements OnInit, OnChanges {
  constructor(
    private http: HttpService
  ) { }

  @Input() Data: any = [];
  @Input() chartTitle: string = 'Simple';
  
  @Input() ID: string;
  @Input() TypeChart: string
  @Input() Width: number;
  @Input() Heigth: number;

  @ViewChild('chart', { static: true }) chartContainer: ElementRef;
  //private chartContainer: ElementRef;

  private svg: any;
  private yScale: any;
  private xScale: any;
  private xAxis: any;
  private yAxis: any;
  private chart1: any;
  private barGroups: any;
  public ChartID1: string;
  private Delay : number = 0
  private d3: any = D3;
  private check: number = 0;
  private margin = { top: 20, right: 20, bottom: 40, left: 40 }
  private width = 580 - this.margin.left - this.margin.right
  private height = 300 - this.margin.top - this.margin.bottom;

  ngOnInit() {
   
    this.ChartID1 = 'c1' + this.ID

    if (this.Width) {
      this.width = this.Width - this.margin.left - this.margin.right
    }
    if (this.Heigth) {
      this.height = this.Heigth - this.margin.top - this.margin.bottom
    }
  }


  ngOnChanges(changes: SimpleChanges): void {
   
    if (!changes["Data"].isFirstChange()) {
      this.check++
      //this.d3.select("#" + 'c1' + this.ID).remove();
      //this.SetUpchart(this.ChartID1);
      if (this.check > 1) {
        this.Delay = 3000;
      }
     
      this.CreateChart(this.Data)
    }
  }

  ngAfterViewInit() {
    this.SetUpchart(this.ChartID1);
  }


  SetUpchart(id1) {


    this.svg = this.d3.select('#' + id1).append('svg')
      .attr("id", 'barchart1' + id1)
      .attr('width', this.width + this.margin.left + this.margin.right)
      .attr('height', this.height + this.margin.top + this.margin.bottom)
      .style("background", "rgb(255, 255, 255");


    this.yScale = this.d3.scaleLinear()
      .range([this.height, 50])


    this.xScale = this.d3.scaleBand()
      .range([0, this.width - 20])
      //.domain(data.map((d) => d.catagory))
      .padding(0.4)

    this.xAxis = this.d3.axisBottom(this.xScale);

    this.yAxis = this.d3.axisLeft(this.yScale).ticks(5).tickFormat(d => this.TypeChart == 'Value' ? d + '' : d + '%');;

    this.chart1 = this.svg.append('g')
      .attr('transform', `translate(${this.margin.left}, ${this.margin.top})`);

    this.chart1.append('g')
      .attr("class", "y axis")
      .call(this.yAxis);

    this.chart1.append('g')
      .attr("class", "xAxis")
      .attr('transform', `translate(0, ${this.height})`)
      .call(this.xAxis)
      .selectAll("text")
      .style("text-anchor", "end")

    if (this.Data.length > 0) {
      this.CreateChart(this.Data)
    }
  }




  CreateChart(gData) {
   

    let data: valueModel[];
    let hearder: string;
    let target: number;

    data = gData
    target = gData[0].TARGET

    this.yScale.domain([this.GenerateYminDomain(data, target), this.GenerateYmaxDomain(data, target)]);

    this.xScale.domain(data.map((d) => d.catagory))

    this.chart1
      .select(".y")
      .transition()
      .duration(this.Delay)
      .call(this.yAxis);

    this.chart1
      .select(".xAxis")
      .transition()
      .duration(this.Delay)
      .attr('transform', `translate(0, ${this.height})`)
      .call(this.xAxis)
      .selectAll("text")

    this.svg.select('.grid').remove();//remove then create new because each data had scale of it.
    var makeYLines = () => this.d3.axisLeft(this.yScale);
    this.chart1.append('g')
      .attr('class', 'grid')
      .call(makeYLines().ticks(5).tickSize(-this.width).tickFormat((d) => ""));

    let xScaleOfBar = this.xScale;
    let yScaleOfBar = this.yScale;

    let chart = this.chart1;

    this.barGroups = this.chart1.selectAll(".bar1")
      .data(data)
    
  var xx =  this.barGroups
      .enter()
      .append("rect")
      .merge(this.barGroups)
      .attr("class", "bar1")
      //.transition()
      //.duration(this.Delay)
      .attr('x', (d) => this.xScale(d.catagory))
      .attr('y', (d) => this.yScale(d.Value > 0 ? d.Value : 0))

      .attr('height', (s) => {
        let iHeightBar = this.height - this.yScale(s.Value);
        if (iHeightBar <= 0) { return 0; }
        else { return iHeightBar; }
      })//end add
      .attr('width', this.xScale.bandwidth())
      .attr("fill", function (d) {
        return "#85C1E9";
      })
    xx.transition().duration(this.Delay)
    xx.on("mouseover", (d) => this.handleMouseOver(d, 'Value'))
      .on("mouseout", (d) => this.handleMouseOut(d));

    this.barGroups = this.chart1.selectAll(".textvalue1")
      .remove()
      .exit()
      .data(data)

    this.barGroups
      .enter()
      .append('text')
      .attr('class', 'textvalue1')
      .attr('x', (d) => this.xScale(d.catagory) + this.xScale.bandwidth() / 2)
      .attr('y', (d) => this.yScale(d.Value) - 15)
      .attr('text-anchor', 'middle')
      .attr("font-size", "12")
      .attr("fill", "#000000")
      //.text((d) => this.TypeChart == 'Value' ? d.Value : d.Value + '%')

      //.text((d) => {
      //  if (this.TypeChart == 'Value') {
      //    return d.Value
      //  } else if (this.TypeChart == 'Percent') {
      //    //-- Return blan in case 0%
      //    if (d.Value > 0) {
      //      return d.Value + '%'
      //    } else {
      //      return ''
      //    }
      //  } else {
      //    return d.Value
      //  }
      //})
    this.chart1.selectAll('#limit').remove()

    var line = this.chart1.append('line')
      .attr('id', 'limit')
      .attr('x1', 2)
      .attr('y1', (d) => this.yScale(target)) 
      .attr('x2', this.width - 16)
      .attr('y2', (d) => this.yScale(target))

    this.svg.select('.linetext').remove()

    this.chart1.append("text")
      .attr('class', 'linetext')
      .attr("transform", "translate(" + (this.width - 10) + "," + this.yScale(target) + ")")
      .attr("dy", ".35em")
      .attr("text-anchor", "start")
      .style("fill", "red")
      .text(target + '%');


    this.svg.select('.textH').remove()

    this.chart1.append('text')
      .attr('class', 'textH')
      .attr('x', this.width / 2)
      .attr('y', 0)
      .attr('text-anchor', 'middle')
      .attr("font-size", "20")
      .style("font-weight", 'bold')
      .style("fill", "#595959")
      .text(this.chartTitle)
  }

  handleMouseOver = (d, Value) => {
    this.chart1.append("text")
      .attr('class', 'textHover')
      .attr('x', (this.xScale(d.catagory) + this.xScale.bandwidth() / 2)-10)
      .attr('y', this.yScale(d.Value) - 10)
      .attr("font-size", "7pt")
      .style("font-weight", 'bold')
      .style("fill", "#000000")
      .text(this.TypeChart == 'Value' ? d.Value : d.Value + '%')
  }

  handleMouseOut = (d) => {
    this.chart1.select('.textHover').remove()
  }


  GenerateYmaxDomain(data : any, target: number) {

    var yMaxDomain: number = 100;
   
      yMaxDomain = 100
      yMaxDomain = this.d3.max(data, d => d.Value > target ? Number(d.Value) : target )
      yMaxDomain = yMaxDomain + ((yMaxDomain * 5) / 100);
  
    return yMaxDomain
  }

  GenerateYminDomain(data : any, target: number) {
    var yMinDomain: number = 0;
    return yMinDomain
  }


}


export interface ChartData {
  Values: Array<valueModel>;
  Legend: Array<legendModel>
}


export interface valueModel {
  catagory: string;
  Value: number;
  //Value2: number;
  //Value3: number;


}

export interface legendModel {
  legend: string;
  color: string;

}
