import { Component, OnInit, OnChanges, ElementRef, Input, ViewChild, ViewEncapsulation } from '@angular/core';
import * as d3 from 'd3';
//import { daLocale } from 'ngx-bootstrap/chronos/i18n/da';
import { HttpService } from 'src/app/core/http.service';


@Component({
  selector: 'bar-with-line',
  templateUrl: './bar-with-line.component.html',
  styleUrls: ['./bar-with-line.component.css']
})
export class BarWithLineComponent implements OnInit, OnChanges {
  constructor(
    private http: HttpService
  ) { }

  @Input() serviceName: string;
  @Input() FY: string;

  @Input() headerSSI: string = '';
  @Input() headerCSI: string = '';
  @Input() ID: string;
  @Input() Showtable: boolean = true;
  @Input() TypeChart: string;
  //add by BF.24/09/2019
  @Input() yAxisMax: number;
  @Input() yAxisMin: number = 0;
  @Input() showDM: boolean = true;
    @Input() titleSSI: string = '';
    @Input() titleCSI: string = '';

  //end add BF.24/09/2019
  @ViewChild('chart', { static: true }) chartContainer: ElementRef;
  //private chartContainer: ElementRef;

  private svg: any;
  private svg2: any;
  private yScale: any;
  private yScale2: any;
  private xScale: any;
  private xAxis: any;
  private yAxis: any;
  private yAxis2: any;
  private chart1: any;
  private chart2: any;
  private barGroups: any;
  private barGroups2: any;
  public ColumnsSSI: string[] = [];
  public ColumnsCSI: string[] = [];
  public headerListSSI: Array<any> = [];
  public headerListCSI: Array<any> = [];
  public dataListSSI: any;
  public dataListCSI: any;

  private NewColumnSSI: any = []
  private NewColumnCSI: any = []
  public YearList: any;

  private MAX_WIDTH = 600;
  private MAX_HEIGHT = 320;
  public ChartID1: string;
  public ChartID2: string;

  ngOnInit() {
    this.ChartID1 = 'c1' + this.ID
    this.ChartID2 = 'c2' + this.ID
   //this.CreateDropdownList()
  }

  ngOnChanges(): void {

  }

  ngAfterViewInit() {
    this.SetUpchart(this.ChartID1, this.ChartID2)
  }



  GetData() {

    let dataSSI = [], dataCSI = []
    this.http.get("System", this.serviceName, this.FY)
      .subscribe(result => {

        dataSSI = result.Table;
        dataCSI = result.Table1;

        this.dataListSSI = result.Table2
        this.dataListCSI = result.Table3
        this.NewColumnSSI = result.Table2
        this.NewColumnCSI = result.Table3

        this.CreateChart1(dataSSI)
        this.CreateChart2(dataCSI)

        if (this.Showtable) {
          this.headerListSSI = this.GenerateHearderListSSI(this.NewColumnSSI);
          this.headerListCSI = this.GenerateHearderListCSI(this.NewColumnCSI);
          this.ColumnsSSI = this.dataListSSI.map(v => v.catagory);

          this.ColumnsCSI = this.dataListCSI.map(v => v.catagory);
            this.headerSSI.length > 0 ? this.ColumnsSSI.unshift(this.headerSSI) : this.ColumnsSSI.unshift(this.NewColumnSSI[0].Column_Name)
            this.headerCSI.length > 0 ? this.ColumnsCSI.unshift(this.headerCSI) : this.ColumnsCSI.unshift(this.NewColumnCSI[0].Column_Name)
        }

      })

  }


  GenerateHearder(text) {
    let month = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
    let newtext = text.substring(4)
    return month[Number(newtext) - 1]

  }


  SetUpchart(id1, id2) {

    var margin = { top: 20, right: 20, bottom: 40, left: 40 },
      width = this.MAX_WIDTH - margin.left - margin.right,
      height = this.MAX_HEIGHT - margin.top - margin.bottom;
      

    this.svg = d3.select('#' + id1).append('svg')
      .attr("id", 'barchart1' + id1)
      .attr('width', width + margin.left + margin.right)
      .attr('height', height + margin.top + margin.bottom)
      .style("background", "rgb(255, 255, 255");

    this.svg2 = d3.select('#' + id2).append('svg')
      .attr("id", 'barchart2' + id2)
      .attr('width', width + margin.left + margin.right)
      .attr('height', height + margin.top + margin.bottom)
      .style("background", "rgb(255, 255, 255");

    this.yScale = d3.scaleLinear()
      .range([height, 50])

    this.yScale2 = d3.scaleLinear()
      .range([height, 50])

    this.xScale = d3.scaleBand()
      .range([0, width - 20])
      .padding(0.4)

    this.xAxis = d3.axisBottom(this.xScale);

    this.yAxis = d3.axisLeft(this.yScale).ticks(5).tickFormat(d => this.TypeChart == 'Value' ? d + '' : d + '%');;
    this.yAxis2 = d3.axisLeft(this.yScale2).ticks(5).tickFormat(d => this.TypeChart == 'Value' ? d + '' : d + '%');;
    
    this.chart1 = this.svg.append('g')
      .attr('transform', `translate(${margin.left}, ${margin.top})`);
    

    this.chart1.append('g')
      .attr("class", "y axis")
      .call(this.yAxis);

    this.chart1.append('g')
      .attr("class", "xAxis")
      .attr('transform', `translate(0, ${height})`)
      .call(this.xAxis)
      .selectAll("text")
      .style("text-anchor", "end")



    this.chart2 = this.svg2.append('g')
      .attr('transform', `translate(${margin.left}, ${margin.top})`);

    this.chart2.append('g')
      .attr("class", "y axis")
      .call(this.yAxis2);

    this.chart2.append('g')
      .attr("class", "xAxis")
      .attr('transform', `translate(0, ${height})`)
      .call(this.xAxis)
      .selectAll("text")
      .style("text-anchor", "end")

    this.GetData()
  }
   
  CreateChart1(gData) {

    var margin = { top: 20, right: 20, bottom: 40, left: 40 },
      width = this.MAX_WIDTH - margin.left - margin.right,
      height = this.MAX_HEIGHT - margin.top - margin.bottom;

    let data: valueModel[];
    let hearder: string;
    let target: number;

    data = gData
    target = gData[0].TARGET

    if (this.yAxisMax > this.yAxisMin && this.yAxisMax > 0) {
      this.yScale.domain([this.yAxisMin, this.yAxisMax])
    } else {
      this.yScale.domain([this.GenerateYminDomain(data, target), this.GenerateYmaxDomain(data, target)]);
    }

    this.xScale.domain(data.map((d) => d.catagory))

    this.chart1
      .select(".y")
      .call(this.yAxis);

    this.chart1
      .select(".xAxis")
      .attr('transform', `translate(0, ${height})`)
      .call(this.xAxis)
      .selectAll("text")

    this.svg.select('.grid').remove();//remove then create new because each data had scale of it.
    var makeYLines = () => d3.axisLeft(this.yScale);
    this.chart1.append('g')
      .attr('class', 'grid')
      .call(makeYLines().ticks(5).tickSize(-width).tickFormat((d) => ""));

    let xScaleOfBar = this.xScale;
    let yScaleOfBar = this.yScale;

    let chart = this.chart1;

    this.barGroups = this.chart1.selectAll(".bar1")
      .data(data)


    this.barGroups
      .enter()
      .append("rect")
      .merge(this.barGroups)
      .attr("class", "bar1")
      //.transition().duration(100)
      .attr('x', (d) => this.xScale(d.catagory))
      .attr('y', (d) => this.yScale(d.Value > 0 ? d.Value : 0))
      //.attr('height', (s) => height - this.yScale(s.Value))//comment by BF. 24/09/2019
      //add by BF. 24/09/2019 when user input [yAxisMax] [yAxisMin] then effect to yScale(s.Value)
      .attr('height', (s) => {
        let iHeightBar = height - this.yScale(s.Value);
        if (iHeightBar <= 0) { return 0; }
        else { return iHeightBar; }
      })//end add
      .attr('width', this.xScale.bandwidth())
      .attr("fill", function (d) {
        return "#538ed5";
      }).on('mouseenter', function (actual, i) {

        d3.selectAll('.value')
          .attr('opacity', 0)

        d3.select(this)
          .transition()
          .duration(300)
          .attr('opacity', 0.6)
          .attr('x', (d) => xScaleOfBar(actual.catagory) - 5)
          .attr('width', xScaleOfBar.bandwidth() + 10)

        var y = yScaleOfBar(actual.Value);
        var line = chart.append('rect')
          .transition()
          .duration(500)
          .attr('id', 'showzone')
          .attr('x', 0)
          .attr('y', 0)
          .attr('width', width)
          .attr('height', y)
          .attr("fill", "blue")
          .attr('opacity', 0.1)
          ;

      })
      .on('mouseleave', function (actual) {
        d3.selectAll('.value')
          .attr('opacity', 1)

        d3.select(this)
          .transition()
          .duration(300)
          .attr('opacity', 1)
          .attr('x', (a) => xScaleOfBar(actual.catagory))
          .attr('width', xScaleOfBar.bandwidth())

        chart.selectAll('#showzone').remove()

      });

    this.barGroups = this.chart1.selectAll(".textvalue1")
      .remove()
      .exit()
      .data(data)

    this.barGroups
      .enter()
      .append('text')
      .attr('class', 'textvalue1')
      .attr('x', (d) => this.xScale(d.catagory) + this.xScale.bandwidth() / 2)
      .attr('y', (d) => this.yScale(d.Value) - 15)
      .attr('text-anchor', 'middle')
      .attr("font-size", "12")
      .attr("fill", "#000000")
      //.text((d) => this.TypeChart == 'Value' ? d.Value : d.Value + '%')

      .text((d) => {
        if (this.TypeChart == 'Value') {
          return d.Value
        } else if (this.TypeChart == 'Percent') {
          //-- Return blan in case 0%
          if (d.Value > 0) {
            return d.Value + '%'
          } else {
            return ''
          }
        } else {
          return d.Value
        }
      })
    this.chart1.selectAll('#limit').remove()

    var line = this.chart1.append('line')
      .attr('id', 'limit')
      .attr('x1', 2)
      .attr('y1', (d) => this.yScale(target))
      .attr('x2', width - 30)
      .attr('y2', (d) => this.yScale(target))

    this.svg.select('.linetext').remove()

    this.chart1.append("text")
      .attr('class', 'linetext')
      .attr("transform", "translate(" + (width - 20) + "," + this.yScale(target) + ")")
      .attr("dy", ".35em")
      .attr("text-anchor", "start")
      .style("fill", "red")
      .text(target);



    this.svg.select('.textH').remove()

    this.chart1.append('text')
      .attr('class', 'textH')
      .attr('x', width / 2)
      .attr('y', 0)
      .attr('text-anchor', 'middle')
      .attr('font-weight', 'bold')
      .text(() => this.titleSSI.length > 0 ? this.titleSSI : 'SSI : ' + this.NewColumnSSI[0].Column_Name)



  }

  CreateChart2(gData) {

    var margin = { top: 20, right: 20, bottom: 40, left: 40 },
      width = this.MAX_WIDTH - margin.left - margin.right,
      height = this.MAX_HEIGHT - margin.top - margin.bottom;

    let data: valueModel[];
    let hearder: string;
    let target: number;


    data = gData
    target = gData[0].TARGET

    //add by BF. 24/09/2019        
    if (this.yAxisMax > this.yAxisMin && this.yAxisMax > 0) {
      this.yScale2.domain([this.yAxisMin, this.yAxisMax])
    } else {
      this.yScale2.domain([this.GenerateYminDomain(data, target), this.GenerateYmaxDomain(data, target)]);
    }

    this.xScale.domain(data.map((d) => d.catagory))

    this.chart2
      .select(".y")
      .call(this.yAxis2);

    this.chart2
      .select(".xAxis")
      .attr('transform', `translate(0, ${height})`)
      .call(this.xAxis)
      .selectAll("text")


    //create range of grid line from yScale
    this.svg2.select('.grid').remove();//remove then create new because each data had scale of it.
    var makeYLines = () => d3.axisLeft(this.yScale2);
    this.chart2.append('g')
      .attr('class', 'grid')
      .call(makeYLines().ticks(5).tickSize(-width).tickFormat((d) => ""));

    let xScaleOfBar = this.xScale;
    let yScaleOfBar = this.yScale2;

    let chart = this.chart2;

    this.barGroups2 = this.chart2.selectAll(".bar2")
      .data(data)

    this.barGroups2
      .enter()
      .append("rect")
      .merge(this.barGroups2)
      .attr("class", "bar2")
      .attr('x', (d) => this.xScale(d.catagory))
      .attr('y', (d) => this.yScale2(d.Value > 0 ? d.Value : 0))
      .attr('height', (s) => height - this.yScale2(s.Value))
      .attr('width', this.xScale.bandwidth())
      .attr("fill", function (d) {
        return "#538ed5";
      }).on('mouseenter', function (actual, i) {

        d3.selectAll('.value')
          .attr('opacity', 0)

        d3.select(this)
          .transition()
          .duration(300)
          .attr('opacity', 0.6)
          .attr('x', (d) => xScaleOfBar(actual.catagory) - 5)
          .attr('width', xScaleOfBar.bandwidth() + 10)

        var y = yScaleOfBar(actual.Value);
        var line = chart.append('rect')
          .transition()
          .duration(500)
          .attr('id', 'showzone')
          .attr('x', 0)
          .attr('y', 0)
          .attr('width', width)
          .attr('height', y)
          .attr("fill", "blue")
          .attr('opacity', 0.1)
          ;

      })
      .on('mouseleave', function (actual) {
        d3.selectAll('.value')
          .attr('opacity', 1)

        d3.select(this)
          .transition()
          .duration(300)
          .attr('opacity', 1)
          .attr('x', (a) => xScaleOfBar(actual.catagory))
          .attr('width', xScaleOfBar.bandwidth())

        chart.selectAll('#showzone').remove()

      });

    this.barGroups2 = this.chart2.selectAll(".textvalue2")
      .remove()
      .exit()
      .data(data)

    this.barGroups2
      .enter()
      .append('text')
      .attr('class', 'textvalue2')
      .attr('x', (d) => this.xScale(d.catagory) + this.xScale.bandwidth() / 2)
      .attr('y', (d) => this.yScale2(d.Value) - 15)
      .attr('text-anchor', 'middle')
      .attr("fill", "#000000")
      .attr("font-size", "13.5")
      .text((d) => {
        if (this.TypeChart == 'Value') {
          return d.Value
        } else if (this.TypeChart == 'Percent') {
          //-- Return blan in case 0%
          if (d.Value > 0) {
            return d.Value + '%'
          } else {
            return ''
          }
        } else {
          return d.Value
        }
      })

    this.chart2.selectAll('#limit').remove()

    var line = this.chart2.append('line')
      .attr('id', 'limit')
      .attr('x1', 2)
      .attr('y1', (d) => this.yScale2(target))
      .attr('x2', width - 30)
      .attr('y2', (d) => this.yScale2(target))



    this.svg2.select('.linetext').remove()

    this.chart2.append("text")
      .attr('class', 'linetext')
      .attr("transform", "translate(" + (width - 20) + "," + this.yScale2(target) + ")")
      .attr("dy", ".35em")
      .attr("text-anchor", "start")
      .style("fill", "red")
      .text(target);


    this.svg2.select('.textH').remove()

    this.chart2.append('text')
      .attr('class', 'textH')
      .attr('x', width / 2)
      .attr('y', 0)

      .attr('text-anchor', 'middle')
      .attr('font-weight', 'bold')
      .text(()=> this.titleCSI.length > 0 ? this.titleCSI: 'CSI : ' + this.NewColumnCSI[0].Column_Name)


  }

  GenerateHearderListSSI(data) {
    let newData = [];
    let NewCol: string;
    for (var key in data[0]) {
      if (key != 'Column_Name' && key != 'catagory') {
        NewCol = key == 'Value' ? this.NewColumnSSI[0].Column_Name : key
        newData.push({ NewCol: NewCol, Col: key })
      }

    }
    return newData
  }

  GenerateHearderListCSI(data) {
    let newData = [];
    let NewCol: string;
    for (var key in data[0]) {
      if (key != 'Column_Name' && key != 'catagory') {
        NewCol = key == 'Value' ? this.NewColumnCSI[0].Column_Name : key
        newData.push({ NewCol: NewCol, Col: key })
      }

    }
    return newData
  }

  GenerateYmaxDomain(data: valueModel[], target: number) {

      var yMaxDomain: number = 0;
      yMaxDomain = d3.max(data, d => Number(d.Value))
      yMaxDomain = yMaxDomain + ((yMaxDomain * 5) / 100);
    return yMaxDomain
  }

  GenerateYminDomain(data: valueModel[], target: number) {
    var yMinDomain: number = 0;
    return yMinDomain
  }

  FYChange() {
    this.GetData()
  }

}


export interface ChartData {
  Values: Array<valueModel>;
  Legend: Array<legendModel>
}


export interface valueModel {
  catagory: string;
  Value: number;
  //Value2: number;
  //Value3: number;


}

export interface legendModel {
  legend: string;
  color: string;

}
