import { Component, OnInit } from '@angular/core';
import * as d3 from 'd3';

@Component({
  selector: 'app-bar-chart-MonthOfYear',
  templateUrl: './barChartMonthOfYear.component.html',
  styleUrls: ['./barChartMonthOfYear.component.css']
})
export class barChartMonthOfYear implements OnInit {


  constructor() { }
  private labelxScalemonth12 = ["Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", "Jan", "Feb", "Mar"];
  private dataInput: dataJsonMonthOfYear[] = [
    {
      monthOf: 'Jan',
      value: 78.9,
      color: '#000000'
    },
    {
      monthOf: 'Feb',
      value: 75.1,
      color: '#00a2ee'
    },
    {
      monthOf: 'Mar',
      value: 68.0,
      color: '#fbcb39'
    },
    {
      monthOf: 'Apr',
      value: 67.0,
      color: '#007bc8'
    },
    {
      monthOf: 'May',
      value: 65.6,
      color: '#65cedb'
    },
    {
      monthOf: 'Jun',
      value: 65.1,
      color: '#ff6e52'
    },
    {
      monthOf: 'Jul',
      value: 61.9,
      color: '#f9de3f'
    },
    {
      monthOf: 'Aug',
      value: 60.4,
      color: '#5d2f8e'
    },
    {
      monthOf: 'Sep',
      value: 59.6,
      color: '#008fc9'
    },
    {
      monthOf: 'Oct',
      value: 59.6,
      color: '#507dca'
    }
  ];

  ngOnInit() {
    this.createBarChartV1(this.dataInput);
  }

  private createBarChartV1(sample: dataJsonMonthOfYear[]): void {
    //Set Text of x scale, format is 'Jan','Feb' that it must math "string" of sample.monthOf from dataJsonMonthOfYear
    const month12 = this.labelxScalemonth12;

    const svg = d3.select('#chartBart01');

    const MARGIN = 80;
    const WIDTH = 1000 - 2 * MARGIN;
    const HEIGHT = 600 - 2 * MARGIN;

    const chart = svg.append('g')
      .attr('transform', `translate(${MARGIN}, ${MARGIN})`);

    const xScale = d3.scaleBand()
      .range([0, WIDTH])
      .domain(month12.map((s) => s))
      .padding(0.4)


    //set dynamic range of yScale for optimize chart to cool.
    let idomainMax = d3.max(sample, d => d.value);
    idomainMax = idomainMax + ((idomainMax * 5) / 100);//+5%
    let idomainMin = d3.min(sample, d => d.value);
    idomainMin = idomainMin - ((idomainMin * 5) / 100);//+5%    
    const yScale = d3.scaleLinear()
      .range([HEIGHT, 0])
      .domain([idomainMin, idomainMax]);

    chart.append('g')
      .attr('transform', `translate(0, ${HEIGHT})`)
      .call(d3.axisBottom(xScale));

    chart.append('g')
      .call(d3.axisLeft(yScale));

    //get range of grid line from yScale
    const makeYLines = () => d3.axisLeft(yScale);

    //create grid line y position allow range that get.
    chart.append('g')
      .attr('class', 'grid')
      .call(makeYLines().tickSize(-WIDTH).tickFormat((d) => ""));

    //create bar by draw rect at chart.
    const barGroups = chart.selectAll()
      .data(sample)
      .enter()
      .append('g')    
    barGroups
      .append('rect')
      .attr('class', 'bar')
      .attr('x', (g) => xScale(g.monthOf))
      .attr('y', (g) => yScale(g.value))
      .attr('height', (g) => HEIGHT - yScale(g.value))
      .attr('width', xScale.bandwidth())
      .on('mouseenter', function (actual, i) {
        d3.selectAll('.value')
          .attr('opacity', 0)
        //effect +opacity +width of bar
        d3.select(this)
          .transition()
          .duration(300)
          .attr('opacity', 0.6)
          .attr('x', (a: dataJsonMonthOfYear) => xScale(a.monthOf) - 5)
          .attr('width', xScale.bandwidth() + 10)
        //effect box
        const y = yScale(actual.value)
        var line = chart.append('rect')
          .transition()
          .duration(500)
          .attr('id', 'showzone')
          .attr('x', 0)
          .attr('y', 0)
          .attr('width', WIDTH)
          .attr('height', y)
          .attr("fill", "blue")
          .attr('opacity', 0.1)
          ;
               
      })
      .on('mouseleave', function () {
        d3.selectAll('.value')
          .attr('opacity', 1)

        d3.select(this)
          .transition()
          .duration(300)
          .attr('opacity', 1)
          .attr('x', (a: dataJsonMonthOfYear) => xScale(a.monthOf))
          .attr('width', xScale.bandwidth())

        chart.selectAll('#showzone').remove()
      })


    //Text of value data show at on bar.
    barGroups.append('text')
      //.attr('class', 'value')
      .attr('class', 'txtValue')
      .attr('x', (a) => xScale(a.monthOf) + xScale.bandwidth() / 2)
      .attr('y', (a) => yScale(a.value) - 5)
      .attr('text-anchor', 'middle')
      //.text((a) => `${a.value}%`)
      .text((a) => `${a.value}`)
       
    //text titel of chart
    svg.append('text')
      .attr('class', 'title')
      .attr('x', WIDTH / 2 + MARGIN)
      .attr('y', MARGIN) //for titel show on upper chart allway.
      .attr('text-anchor', 'middle')
      .text('CSI payment(mTHB)')
        
  }

}

export interface dataJsonMonthOfYear {
  monthOf: string,
  value: number,
  color: string
}
