import { Component, OnInit } from '@angular/core';
import * as d3 from 'd3';
import { legendModel } from '../dual-scale/dual-scale.component';

@Component({
  selector: 'bar-with-dual-line',
  templateUrl: './barchart-with-dual-line.component.html',
  styleUrls: ['././barchart-with-dual-line.component.css']
})
export class barWithDualLineComponent implements OnInit {



  constructor() { }
  private labelxScalemonth12 = ["FY18","Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", "Jan", "Feb", "Mar"];
  private dataInput: dataJsonMonthOfYear[] = [
   
    {
      monthOf: 'FY18',
      value: 95,
      target: 72,
      accumulate : 94,
      color: '#3a6293'
    },
    {
      monthOf: 'Apr',
      value: 67.0,
      target : 70,
      accumulate: 71,
      color: '#a0bf62'
    },
    {
      monthOf: 'May',
      value: 65.6,
      target: 71,
      accumulate: 72,
      color: '#a0bf62'
    },
    {
      monthOf: 'Jun',
      value: 65.1,
      target: 72,
      accumulate: 73,
      color: '#a0bf62'
    },
    {
      monthOf: 'Jul',
      value: 61.9,
      target: 73,
      accumulate: 74,
      color: '#a0bf62'
    },
    {
      monthOf: 'Aug',
      value: 60.4,
      target: 74,
      accumulate: 75,
      color: '#a0bf62'
    },
    {
      monthOf: 'Sep',
      value: 59.6,
      target: 75,
      accumulate: 76,
      color: '#a0bf62'
    },
    {
      monthOf: 'Oct',
      value: 89.6,
      target: 76,
      accumulate: 77,
      color: '#a0bf62'
    },
    {
      monthOf: 'Nov',
      value: 70.6,
      target: 77,
      accumulate: 78,
      color: '#a0bf62'
    },
    {
      monthOf: 'Dec',
      value: 80.6,
      target: 78,
      accumulate: 79,
      color: '#a0bf62'
    },
    {
      monthOf: 'Jan',
      value: 92.9,
      target: 79,
      accumulate: 80,
      color: '#a0bf62'
    },
    {
      monthOf: 'Feb',
      value: 75.1,
      target: 80,
      accumulate: 81,
      color: '#a0bf62'
    },
    {
      monthOf: 'Mar',
      value: 68.0,
      target: 81,
      accumulate: 82,
      color: '#a0bf62'
    },
  ];

  private legendList: LegenModel[] = [{ legend: 'FY19 Monthly', color: '#3b6394' }, { legend: 'FY19 Target', color: '#c00000' }, { legend: 'FY19 Accumulate', color: '#7f7f7f' }]

  ngOnInit() {
    this.createBarChartV1(this.dataInput);
  }

  private createBarChartV1(sample: dataJsonMonthOfYear[]): void {
    //Set Text of x scale, format is 'Jan','Feb' that it must math "string" of sample.monthOf from dataJsonMonthOfYear

    const month12 = this.labelxScalemonth12;

    const svg = d3.select('#dual-line');

    const MARGIN = 80;
    const WIDTH = 1000 - 2 * MARGIN;
    const HEIGHT = 600 - 2 * MARGIN;

    const chart = svg.append('g')
      .attr('transform', `translate(${MARGIN}, ${MARGIN})`);

    const xScale = d3.scaleBand()
      .range([0, WIDTH])
      .domain(month12.map((s) => s))
      .padding(0.4)


    //set dynamic range of yScale for optimize chart to cool.
    let idomainMax = d3.max(sample, d => d.value);
    idomainMax = idomainMax + ((idomainMax * 5) / 100);//+5%
    let idomainMin = d3.min(sample, d => d.value);
    idomainMin = idomainMin - ((idomainMin * 5) / 100);//+5%    
    const yScale = d3.scaleLinear()
      .range([HEIGHT, 0])
      .domain([idomainMin, idomainMax]);

    chart.append('g')
      .attr('transform', `translate(0, ${HEIGHT})`)
      .call(d3.axisBottom(xScale));

    chart.append('g')
      .call(d3.axisLeft(yScale));

    //get range of grid line from yScale
    const makeYLines = () => d3.axisLeft(yScale);

    //create grid line y position allow range that get.
    chart.append('g')
      .attr('class', 'grid')
      .call(makeYLines().tickSize(-WIDTH).tickFormat((d) => ""));

    var barGroups = chart.selectAll()
      .data(sample)
      .enter()

    //create bar by draw rect at chart.
     barGroups   
      .append('rect')
      .attr('class', 'bar1')
      .attr('x', (g) => xScale(g.monthOf))
      .attr('y', (g) => yScale(g.value))
      .attr('height', (g) => HEIGHT - yScale(g.value))
      .attr('width', xScale.bandwidth())
       .attr("fill", (d) => d.color)

    //Text of value data show at on bar.
    barGroups.append('text')
      .attr('class', 'txtValue')
      .attr('x', (a) => xScale(a.monthOf) + xScale.bandwidth() / 2)
      .attr('y', (a) => HEIGHT - 10)     
      .text((a) => `${a.value}`)
      .attr('text-anchor', 'middle')
      //.attr("dx", "-.8em")
      //.attr("dy", ".15em")
      //.attr("transform", (d) => "rotate(-65)");
       
    //text titel of chart
    svg.append('text')
      .attr('class', 'title')
      .attr('x', WIDTH / 2 + MARGIN)
      .attr('y', MARGIN - 20) //for titel show on upper chart allway.
      .attr('text-anchor', 'middle')
      .text('Overall NPS %')

    var line = d3.line<dataJsonMonthOfYear>()
      .x(function (d, i) { return xScale(d.monthOf) + xScale.bandwidth() / 2;})
      .y(function (d) { return yScale(d.value) + 5; })
      .curve(d3.curveMonotoneX);

    barGroups.append("path")
      .attr("class", "line") // Assign a class for styling
      .attr("d", line(sample)); // 11. Calls the line generator

    barGroups.append("circle") // Uses the enter().append() method
      .attr("class", "dot") // Assign a class for styling
      .attr("cx", function (d, i) { return xScale(d.monthOf) + xScale.bandwidth() / 2; })
      .attr("cy", function (d) { return yScale(d.value) + 3; })
      .attr("r", 10);


    var line2 = d3.line<dataJsonMonthOfYear>()
      .x(function (d, i) { return xScale(d.monthOf) + xScale.bandwidth() / 2; })
      .y(function (d) { return yScale(d.target); })
      .curve(d3.curveMonotoneX);

    barGroups.append("path")
      .attr("class", "line2") // Assign a class for styling
      .attr("d", line2(sample)); // 11. Calls the line generator

    barGroups.append("rect") // Uses the enter().append() method
      .attr('class', 'rec')
      .attr('x', (d) => xScale(d.monthOf) + xScale.bandwidth() / 2)
      .attr('y', (d) => yScale(d.target) - 15)
      .attr('height', (g) => 25)
      .attr('width', xScale.bandwidth() + 10)
      .attr("fill", '#7f7f7f')


    //Text of value data show at on bar.
    barGroups.append('text')
      .attr('class', 'txttarget')
      .attr('x', (d) => xScale(d.monthOf) + xScale.bandwidth())
      .attr('y', (d) => yScale(d.target))
      .attr('text-anchor', 'middle')
      //.text((a) => `${a.value}%`)
      .text((d) => `${d.target}`)
      .attr("fill", "white")


    this.CreateLegend(svg, WIDTH, HEIGHT)
  }
  CreateLegend(svg,WIDTH : number, HEIGHT : number) {
    var nodeWidth = (d) => d.getBBox().width;
    const cScale = d3.scaleOrdinal(d3.schemeCategory10);

    const legend = svg.append('g')
      .attr('class', 'legend')
      .attr('transform', 'translate(0,0)');

    const lg = legend.selectAll('g')
      .data(this.legendList)
      .enter()
      .append('g')
      .attr('transform', (d, i) => `translate(${i * 100},${HEIGHT + 15})`);

    lg.append('rect')
      .style('fill', d => cScale(d.color))
      .attr('x', 30)
      .attr('y', 110)
      .attr('width', 70)
      .attr('height', 15);

    lg.append('text')
      .style('font-family', 'Georgia')
      .style('font-size', '15px')
      .attr('x', 115)
      .attr('y', 121)
      .text(d => d.legend);

    let offset = 0;
    lg.attr('transform', function (d, i) {
      let x = offset;
      offset += nodeWidth(this) + 10;
      return `translate(${x},${HEIGHT + 10})`;
    });

    legend.attr('transform', function () {
      return `translate(${(WIDTH - nodeWidth(this)) / 2},${0})`
    });

  }

}

export interface dataJsonMonthOfYear {
  monthOf: string,
  value: number,
  target : number,
  color: string,
  accumulate: number
}

export interface LegenModel {
  legend: string,
  color: string
}
