import { Component, OnInit, OnChanges, ElementRef, Input, ViewChild, ViewEncapsulation, SimpleChanges } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import * as D3 from 'd3';
//import { stack } from 'd3';

@Component({
  selector: 'custom-bar',
  templateUrl: './custom-bar.component.html',
  styleUrls: ['./custom-bar.component.css']
})
export class CustomBarComponent implements OnInit, OnChanges {



  constructor(
    private http: HttpService
  ) { }

    @Input() ID : string;
    @Input() Data : any = [];
    @Input() gValue : string;
    @Input() Width: any;
    @Input() Heigth: any;
    @Input() TypeData : string;

    private d3 : any = D3;

    private svg: any;
    private yScale: any;
    private xScale: any;
    private xAxis: any;
    private yAxis: any;
    private chart: any;
    private chart2: any;
    private chart3: any;
    private barGroups: any;

    private margin = { top: 30, right: 20, bottom: 40, left: 55 }
    private width = 600 - this.margin.left - this.margin.right
    private height = 350 - this.margin.top - this.margin.bottom;

    private config = {
      Chart1translateX : 0,
      Chart2translateX : 0,
      Chart3translateX : 0,
      xScaleRange : 0,
      TextRecttranslateX : 0,
      TextBartranslateX : 0,
      TextCattranslateX :0,
      TextCatFontSize : '',
      LegendTargetTextX : 0,
      LegendTargetRectX :0,
      LegendTargetLineX1 : 0,
      LegendTargetLineX2 : 0 
    }

    ngOnInit(){

     

      if (this.Width) {
        this.width = this.Width - this.margin.left - this.margin.right
      }
      if (this.Heigth) {
        this.height = this.Heigth - this.margin.top - this.margin.bottom
      }

      //this.gValue = 'a'
      if(this.Data.length < 1){
       
        this.Data = []
      }

      if(this.TypeData == 'SSI'){
        this.config.Chart1translateX = (this.width / 1.35) + this.margin.left
        this.config.Chart2translateX = (this.width / 1.35) + this.margin.left
        this.config.Chart3translateX = (this.width / 1.35) + this.margin.left
        this.config.TextBartranslateX = 15
        this.config.TextCattranslateX = -(this.width / 7)
        this.config.TextRecttranslateX = -19
        this.config.xScaleRange = this.width / 3.5;
        this.config.TextCatFontSize = '8pt'
        this.config.LegendTargetLineX1 = -40
        this.config.LegendTargetLineX2 = -40
        this.config.LegendTargetRectX = -42.5
        this.config.LegendTargetTextX = -30
      }else{
        this.config.Chart1translateX = (this.width / 1.20) + this.margin.left
        this.config.Chart2translateX = (this.width / 1.20) + this.margin.left + 23
        this.config.Chart3translateX = (this.width / 1.20) + this.margin.left + 23
        this.config.TextBartranslateX = 25
        this.config.TextCattranslateX = -(80)
        this.config.TextRecttranslateX = -40
        this.config.xScaleRange = this.width / 5;
        this.config.TextCatFontSize = '7.5pt'
        this.config.LegendTargetLineX1 = -95
        this.config.LegendTargetLineX2 = -95
        this.config.LegendTargetRectX = -97
        this.config.LegendTargetTextX = -80
      }
     
    }

    ngOnChanges(changes: SimpleChanges): void {

      console.log('this.gValue',this.gValue)
      if (!changes["Data"].isFirstChange()) {
        
        this.CreateChart();
      }
  
    }


    
    ngAfterViewInit() {
      
      this.SetUpChart()
    }


    SetUpChart(){


      this.svg = this.d3.select('#' + this.ID).append('svg')
      .attr("id", 'chart3d' + this.ID)
      .attr('width', this.width + this.margin.left + this.margin.right)
      .attr('height', this.height + this.margin.top + this.margin.bottom)
      .style("background", "rgb(243, 243, 243");

      this.yScale = this.d3.scaleBand()
      .range([this.height, 0])
      .padding(0.3)

      this.xScale = this.d3.scaleLinear()
      .range([0, this.config.xScaleRange])

      this.xAxis = this.d3.axisTop(this.xScale);

      this.yAxis = this.d3.axisLeft(this.yScale)
      
      this.chart = this.svg.append('g')
      .attr('transform', `translate(${this.config.Chart1translateX}, ${this.margin.top + this.margin.top})`);

      this.chart2 = this.svg.append('g')
      .attr('transform', `translate(${this.config.Chart2translateX}, ${this.margin.top + 9 + this.margin.top})`);

      this.chart3 = this.svg.append('g')
      .attr('transform', `translate(${this.config.Chart3translateX}, ${this.margin.top + 14 + this.margin.top})`);


      this.CreateChart();
    }


    CreateChart(){

     
      this.xScale.domain([0, this.generatemaxDomain(this.Data)]);

      this.yScale.domain(this.Data.map(d => d.catagory));



    //-------------------------------------circle-----------------------------------------//

      var circle = this.chart2.selectAll('.circle')
      .data(this.Data)
      .enter().append('g')
      .attr("class", "circle")
      .attr("transform", (d, i) => {
      {
          return "translate(-65," + i * ((this.height / this.Data.length)) + ")"
      }})

      circle.append('circle')
      // .attr("x", 0)
      // .attr("y", 0)
      .attr("r", 10)
      //.attr("width",20)
      .attr('fill',(d) => '#ffffff')
      .attr('stroke',(d) => d.Target < 2 ? '#ff9900' : '#666666')
      
     var Textcircle = this.chart3.selectAll(".textvalue")
      .remove()
      .exit()
      .data(this.Data)

      Textcircle
      .enter()
      .append('text')
      .attr('class', 'textvalue')
      // .attr('x', (d) => this.yScale(d.catagory) + this.yScale.bandwidth() / 2)
      // .attr('y', (d) => this.xScale(d.Value) - 15)
      .attr('text-anchor', 'middle')
      .attr("fill", (d) => d.Target < 2 ? '#ff9900' : '#666666')
      .attr("font-size", "13.5")
      .text((d) => '!')
      .attr("transform", (d, i) => {
        {
            return "translate(-65," + i * ((this.height / this.Data.length)) + ")"
        }})

//-----------------------------------------------------------------------------------------------//


//-----------------------------------------------rect--------------------------------------------//

      this.chart.selectAll('.rect').remove().exit()

      var rect = this.chart.selectAll('.rect')
      .data(this.Data)
      .enter().append('g')
      .attr("class", "rect")
      .attr("transform", (d, i) => {
      {
          return "translate(-30," + i * ((this.height / this.Data.length)) + ")"
      }})

      rect.append('rect')
      .attr("x", 0)
      .attr("y", 0)
      .attr("width",25)
      .attr("height", this.yScale.bandwidth())
      .style("fill", (d, i) => d.Target < 2 ? '#ff9900' : '#666666')

      var   Textrect = this.chart3.selectAll(".textrect")
      .remove()
      .exit()
      .data(this.Data)

      Textrect
      .enter()
      .append('text')
      .attr('class', 'textrect')
      // .attr('x', (d) => this.yScale(d.catagory) + this.yScale.bandwidth() / 2)
      // .attr('y', (d) => this.xScale(d.Value) - 15)
      .attr('text-anchor', 'middle')
      .attr("fill", "#ffffff")
      .attr("font-size", "8pt")
      // .text((d) => this.random())
      .text((d) => d.Target)
      .attr("transform", (d, i) => {
        {
            return "translate("+(this.config.TextRecttranslateX)+"," + i * ((this.height / this.Data.length)) + ")"
        }})


//-----------------------------------------------------------------------------------------------//

//-------------------------------------------bar-------------------------------------------------//


      this.chart.selectAll('.bar').remove().exit()

      var bar = this.chart.selectAll('.bar')
      .data(this.Data)
      .enter().append('g')  
      .attr("class", "bar")
      .attr("transform", (d, i) => {
      {
          return "translate(0," + i * ((this.height / this.Data.length)) + ")"
      }})


      bar.append('rect')
            // .attr("x", 0)
            // .attr("y", 0)
            .merge(bar)
            .transition().duration(3000)
            .attr("width",(d) => this.xScale(d[this.gValue]))
            .attr("height", this.yScale.bandwidth())
            .style("fill", (d, i) => '#cccbcb')


      var   TextBar = this.chart3.selectAll(".textbar")
            .remove()
            .exit()
            .data(this.Data)
      
            TextBar
            .enter()
            .append('text')
            .attr('class', 'textbar')
            // .attr('x', (d) => this.yScale(d.catagory) + this.yScale.bandwidth() / 2)
            // .attr('y', (d) => this.xScale(d.Value) - 15)
            .attr('text-anchor', 'middle')
            .attr("fill", "#7e7e7f")
            .attr("font-size", "10pt")
            .text((d) => d[this.gValue])
            .attr("transform", (d, i) => {
              {
                  return "translate("+(this.config.TextBartranslateX)+"," + i * ((this.height / this.Data.length)) + ")"
              }})

     
//-----------------------------------------------------------------------------------------------//

//----------------------------------------Text Catagory------------------------------------------//

            var  TextCat = this.chart3.selectAll(".textcat")
            .remove()
            .exit()
            .data(this.Data)

            TextCat
            .enter()
            .append('text')
            .attr('class', 'textcat')
            // .attr('x', (d) => this.yScale(d.catagory) + this.yScale.bandwidth() / 2)
            // .attr('y', (d) => this.xScale(d.Value) - 15)
            .attr('text-anchor', 'end')
            .attr("fill", "#7e7e7f")
            .attr("font-size", this.config.TextCatFontSize)
            .text((d) => d.catagory)
            .attr("transform", (d, i) => {
              {
                  return "translate("+(this.config.TextCattranslateX)+"," + i * ((this.height / this.Data.length)) + ")"
              }})


//-----------------------------------------------------------------------------------------------//

//----------------------------------------Line Target--------------------------------------------//

    this.chart.selectAll('.linetarget').remove()
    .exit()

    var line = this.chart.selectAll('.linetarget')
      .data(this.Data)
      .enter().append('g')
      .attr("class", "linetarget")
      .attr("transform", (d, i) => {
      {
          return "translate("+this.xScale(d.Target)+"," + i * ((this.height / this.Data.length)) + ")"
      }})

      line.append('line')
      .transition().duration(3000)
      .attr("x1", this.xScale(0))
      .attr("y1", 0)
      .attr("x2", this.xScale(0))
      .attr("y2", (this.yScale.bandwidth()))
      .attr("fill", "none")
      .attr("stroke-width", 1.5)
      .attr("stroke", "red") 

//-----------------------------------------------------------------------------------------------//


//----------------------------------------Rect Target--------------------------------------------//

      this.chart.selectAll('.rectTarget').remove().exit()

      var rect = this.chart.selectAll('.rectTarget')
      .data(this.Data)
      .enter().append('g')
      .attr("class", "rectTarget")
      .attr("transform", (d, i) => {
      {
          return "translate("+(this.xScale(d.Target) - 2.5)+"," + i * ((this.height / this.Data.length)) + ")"
      }})

      rect.append('rect')
      .transition().duration(3000)
      .attr("x", 0)
      .attr("y", 0)
      .attr("width",5)
      .attr("height", 5)
      .style("fill", (d, i) => "red")

//-----------------------------------------------------------------------------------------------//


//----------------------------------------Legend Rect--------------------------------------------//

      let legendList = [{legend : 'Overall Satisfaction',color: '#cccbcb'},
                        {legend : 'Core Satisfaction Attribute',color: '#666666'},
                        {legend : 'High Impact Focus Area',color: '#ff9900'}]

      let height = this.height;
      let width = this.width;

      var nodeWidth = (d) => d.getBBox().width;
      //const cScale = this.d3.scaleOrdinal();

      const legend = this.svg.append('g')
        .attr('class', 'legend')
        .attr('transform', 'translate(0,0)');

      const lg = legend.selectAll('g')
        .data(legendList)
        .enter()
        .append('g')
        .attr('transform', (d, i) => `translate(${i * 100},${height + 15})`);

      lg.append('rect')
        .style('fill', d => d.color)
        .attr('x', -40)
        .attr('y', 55)
        .attr('width', 10)
        .attr('height', 10);

      lg.append('text')
        //.style('font-family', 'Georgia')
        .style('font-size', '8pt')
        .attr('x', -25)
        .attr('y', 65)
        .attr("fill","black")
        .text(d => d.legend);

      let offset = 0;
      lg.attr('transform', function (d, i) {
        let x = offset;
        offset += nodeWidth(this) + 10;
        return `translate(${x},${-40})`;
      });

      legend.attr('transform', function () {
        return `translate(${(width - nodeWidth(this)) / 2},${0})`
      });

//-----------------------------------------------------------------------------------------------//



//----------------------------------------Legend Target------------------------------------------//

         
        this.chart.selectAll('.LegendTargetRect').remove().exit()

        var LegendTargetRect = this.chart.selectAll('.LegendTargetRect')
        .data([0])
        .enter().append('g')
        .attr("class", "LegendTargetRect");

        LegendTargetRect.append('rect')
        .attr("x", this.config.LegendTargetRectX)
        .attr("y", -(this.margin.bottom + 10))
        .attr("width",5)
        .attr("height", 5)
        .style("fill", (d, i) => "red")


        this.chart.selectAll('.LegendTargetLine').remove().exit()

        var line = this.chart.selectAll('.LegendTargetLine')
          .data([0])
          .enter().append('g')
          .attr("class", "LegendTargetLine")

        line.append('line')
        .attr("x1", this.config.LegendTargetLineX1)
        .attr("y1", -48)
        .attr("x2", this.config.LegendTargetLineX1)
        .attr("y2", -31)
        .attr("fill", "none")
        .attr("stroke-width", 1.5)
        .attr("stroke", "red") 

        this.chart.selectAll('.LegendTargetText').remove().exit()


        var  LegendTargetText = this.chart.selectAll(".LegendTargetText")
        .remove()
        .exit()
        .data([0])

        LegendTargetText
        .enter()
        .append('text')
        //.style('font-family', 'Georgia')
        .attr('class', 'LegendTargetText')
        .attr('x', (d) => this.config.LegendTargetTextX)
        .attr('y', (d) => -(this.margin.bottom - 5))
        //.style('font-weight','lighter')
        .attr("fill","black")
        .attr("font-size", "8pt")
        .text((d) => "Target")


//-----------------------------------------------------------------------------------------------//

}

    random(){
      return Math.floor(Math.random() * 10)
    }



    generatemaxDomain(data: any) {
      
      var MaxDomain: number = 0;
      MaxDomain = this.d3.max(data, d => d[this.gValue])
      //  //yMaxDomain = 100  
      MaxDomain = MaxDomain + ((MaxDomain * 5) / 100);
    
      return MaxDomain
    }
  }

