import { Component, OnInit, OnChanges, Input, SimpleChanges } from '@angular/core';
import * as d3 from 'd3';

@Component({
    selector: 'donut-chart',
    templateUrl: './donut-chart.component.html',
    styleUrls: ['./donut-chart.component.css']
})
export class DonutChartComponent implements OnInit, OnChanges {
    // https://bl.ocks.org/tezzutezzu/c2653d42ffb4ecc01ffe2d6c97b2ee5e
    private data: any[];
    @Input("rawData") rawData: any[] = []
    @Input("keyName") keyName: string;
    @Input("valueName") valueName: string;
    @Input("id") id: string;
    @Input("formatText") formatText: string = '';

    // @Input("width") width: number;
    // @Input("height") height: number;

    private duration;
    private firstTime;
    private width;
    private height;
    private radius;
    private color;

    private svg;
    private pie;
    private arc;

    private textArea;

    _current: any;
    _previous: any;

    constructor() {
        this.duration = 600;
        this.firstTime = true;
        this.width = 800;
        this.height = 600;
        this.radius = Math.min(this.width, this.height) / 2.2;
        this.color = d3.scaleOrdinal(d3.schemeCategory10);
    }

    ngOnInit(): void {
        //this.createChart(this.id);

        if (this.rawData.length > 0) {
            this.data = this.serializeData(this.rawData, this.keyName, this.valueName);
            this.changeData(this.svg, this.pie, this.arc, this.textArea);
        }
    }

    ngOnChanges(changes: SimpleChanges) {
        if (!changes["rawData"].isFirstChange()) {
            d3.select("#" + "_" + this.id).remove();
            d3.select("#" + "text_" + this.id).remove();
            this.createChart(this.id);
            if (this.rawData.length > 0) {
                this.data = this.serializeData(this.rawData, this.keyName, this.valueName);
                this.changeData(this.svg, this.pie, this.arc, this.textArea);
            }
            //if (this.rawData.length > 0)
            //{ 
            //    this.data = this.serializeData(this.rawData, this.keyName, this.valueName);
            //    this.changeData(this.svg, this.pie, this.arc);
            //}
        }
    }

    public createChart(id) {
        //Create pie graph function base data on value key.
        this.pie = d3.pie()
            .value((d: any) => d.value)
            .sort(null);

        //Pie's arc radius
        this.arc = d3.arc()
            .innerRadius(this.radius - 150) //Pie-chart set innerRadius = 0
            .outerRadius(this.radius - 20);

        //Create SVG tag with width & height and make g tag as center of SVG
        this.svg = d3.select("#" + id + " .chart-container").append("svg")
            .attr("width", this.width)
            .attr("height", this.height)
            .attr("id", "_" + id)
            .call(responsivefy)
            .append("g")
            .attr("transform", "translate(" + this.width / 2 + "," + this.height / 2 + ")");

        this.textArea = d3.select("#" + id + " .text-container").append("div")
            .attr("width", "120px")
            .attr("height", "120px")
            .attr("id", "text_" + id)

            
        function responsivefy(svg) {
            // get container + svg aspect ratio
            var container = d3.select(svg.node().parentNode),
                width = parseInt(svg.style("width")),
                height = parseInt(svg.style("height")),
                aspect = width / height;
            
        
            // add viewBox and preserveAspectRatio properties,
            // and call resize so that svg resizes on inital page load
            svg.attr("viewBox", "0 0 " + width + " " + height)
                .attr("perserveAspectRatio", "xMinYMid")
                .call(resize);
        
            // to register multiple listeners for same event type, 
            // you need to add namespace, i.e., 'click.foo'
            // necessary if you call invoke this function for multiple svgs
            // api docs: https://github.com/mbostock/d3/wiki/Selections#on
            d3.select(window).on("resize." + svg.attr("id"), resize);
        
            // get width of container and resize svg to fit it
            function resize() {

                var targetWidth = parseInt(container.style("width"));
                
                svg.attr("width", targetWidth);
                svg.attr("height", Math.round(targetWidth / aspect));
            }
        }

    }
    public changeData(svg, pie, arc, textArea) {
        //Create path tag with provided data and make a transition between new and old data.
        svg.selectAll("text").remove();
        textArea.selectAll("div").remove();

        //Text

        textArea.selectAll("mylabels")
            .data(this.data)
            .enter()
            .append("text")
            .attr("class", "text-area-item")
            //.attr("x", 120)
            //.attr("y", function (d, i) { return 100 + i * 25 }) // 100 is where the first dot appears. 25 is the distance between dots
            .style("color", (d) => { return this.color(d.name) })
            .style("display", "list-item")
            .style("list-style", "square")
            .text(function (d) { return d.name })
            .attr("text-anchor", "left")
            .style("alignment-baseline", "middle")
            .style("font-size", "9pt")



        //Pie

        var path = svg.selectAll("path");
        var data0 = path.data(),
            data1 = pie(this.data);
        
        path = path.data(data1, key);

        path.transition("fillMove")
            .duration(this.duration)
            .attrTween("d", arcTween)

        path.enter()
            .append("g")
            .attr("class", "pieslice")
            .attr("id", (d: any, i: number) => {
                return "".concat("id_", i.toString());
            })
            .append("path")
            .each((d, i) => {
                var narc = findNeighborArc(i, data0, data1, key);
                if (narc) {
                    this._current = narc;
                    this._previous = narc;
                } else {
                    this._current = d;
                }
            })
            // .attr("id", (d: any, i: number) => {
            //     return "".concat("id_", i.toString());
            // })
            .attr("fill", (d: any, i: number) => {
                return this.color(d.data.name)
            })
            .on("mouseenter", (d: any, i: number) => {
                var id = this.getSVGGroupID(d, i);
                
                //For path
                var thisPath = svg.select(id + " path");

                var movePoint = this.findLocationXY(d.startAngle, d.endAngle, 15);

                thisPath.transition("movePath")
                    .duration(200)
                    .attr('transform', "translate(" + movePoint.x.toString() + "," + movePoint.y.toString() + "), scale(1.05)");


                //For text
                var thisText = svg.select(id + " text");
                
                let movePointText = this.findLocationXY(d.startAngle, d.endAngle, 15)

                thisText.transition("moveText")
                    .duration(200)
                    .attr('x', movePointText.x.toString())
                    .attr('y', movePointText.y.toString())
                    // .attr('transform', "translate(" + movePointText.x.toString() + "," + movePointText.y.toString() + ")");
            })
            .on("mouseout", (d: any, i: number) => {
                var id = this.getSVGGroupID(d, i);

                //For Path
                var thisPath = svg.select(id + " path");

                thisPath.transition("movePath")
                    .duration(200)
                    .attr('transform', 'translate(0,0), scale(1)')

                //For text
                var thisText = svg.select(id + " text");
                
                let movePoint = this.findLocationXY(d.startAngle, d.endAngle, this.radius)

                thisText.transition("moveText")
                    .duration(200)
                    .attr('x', "0")
                    .attr('y', "0");
            })
            .transition("fillMove")
            .duration(this.duration)
            .attrTween("d", arcTween);


        var text = svg.selectAll(".pieslice")
            .data(data1)

        text.append("text")
            .attr("class", "nodetext")
            .text((d: any) => {
                //let percent:string = this.getPercent(d);
                //return d.data.name + ": " + percent + "%";
                if (this.formatText == 'textValue') {
                    return d.value + "%";
                } else {
                    return d.data.name + ": " + d.value;
                }
            })
            .style("opacity", 0)
            .style("font-weight", "bold")
            .attr("fill", (d: any, i: number) => {
                return this.color(d.data.name)
            })
            .attr("transform", (d: any, i: number) => {
                var id = this.getSVGGroupID(d, i);
                var thisText = svg.select(id + " text");
                var thisTextWidth = thisText.node().getBBox().width;

                let movePoint = this.findLocationXY(d.startAngle, d.endAngle, this.radius)
                
                if(movePoint.x < 0) {
                    movePoint.x -= thisTextWidth;
                }
                
                return "translate(" + movePoint.x.toString() + " , " + movePoint.y.toString() + ")";
            })
            .attr('x', "0")
            .attr('y', "0")
            .transition("showText")
            .delay(this.duration - 50)
            .style("opacity", 1);


        function key(d) {
            return d.data.name;
        }

        function type(d) {
            d.value = +d.value;
            return d;
        }

        function findNeighborArc(i, data0, data1, key) {
            var d: any;
            if (d = findPreceding(i, data0, data1, key)) {

                var obj: any = cloneObj(d)
                obj.startAngle = d.endAngle;
                return obj;

            } else if (d = findFollowing(i, data0, data1, key)) {

                var obj: any = cloneObj(d)
                obj.endAngle = d.startAngle;
                return obj;

            }

            return null


        }

        // Find the element in data0 that joins the highest preceding element in data1.
        function findPreceding(i, data0, data1, key) {
            var m = data0.length;
            while (--i >= 0) {
                var k = key(data1[i]);
                for (var j = 0; j < m; ++j) {
                    if (key(data0[j]) === k) return data0[j];
                }
            }
        }

        // Find the element in data0 that joins the lowest following element in data1.
        function findFollowing(i, data0, data1, key) {
            var n = data1.length, m = data0.length;
            while (++i < n) {
                var k = key(data1[i]);
                for (var j = 0; j < m; ++j) {
                    if (key(data0[j]) === k) return data0[j];
                }
            }
        }

        function arcTween(d) {
            var i = d3.interpolate(this._current, d);
            this._current = i(0);
            return function (t) {
                return arc(i(t))
            }
        }

        function cloneObj(obj) {
            var o = {};
            for (var i in obj) {
                o[i] = obj[i];
            }
            return o;
        }
    }

    //Find location X and Y for mouse over event
    findLocationXY(startAngle, endAngle, baseMove) {
        let middleAngle: number = ((360 * startAngle) / (Math.PI * 2) + (360 * endAngle) / (Math.PI * 2)) / 2;
        let radian = 0;
        let rx;
        let ry;

        //Find radian
        if (middleAngle > 0 && middleAngle < 180) {
            radian = Math.abs(90 - middleAngle) * Math.PI / 180;
        } else if (middleAngle > 180 && middleAngle < 360) {
            radian = Math.abs(270 - middleAngle) * Math.PI / 180;
        }

        //Additional condition for each part of circle
        if (middleAngle > 270) {
            rx = -1;
            ry = -1;
        } else if (middleAngle > 180) {
            rx = -1;
            ry = 1;
        } else if (middleAngle > 90) {
            rx = 1;
            ry = 1;
        } else {
            rx = 1;
            ry = -1;
        }

        let _mx = Math.abs(Math.cos(radian)) * baseMove * rx;
        let _my = Math.abs(Math.sin(radian)) * baseMove * ry;

        // let translateResult = "".concat("translate(", _mx.toString(), ",", _my.toString(), "), scale(1.05)")

        return { x: _mx, y: _my };
    }

    //Transform raw data into usable data for chart
    serializeData(data, key, value) {
        return data.map(v => {
            return {
                name: v[key],
                value: v[value]
            }
        })
    }

    //Get percent
    //getPercent(d): string {
    //    let angle = d.endAngle - d.startAngle
    //    let percent = Math.round((1000 * angle) / (Math.PI * 2)) / 10
        
    //    return percent.toString();
    //}

    getSVGGroupID (d: any, i: number) {
        var id = "#".concat("id_", i.toString());
        return id
    }
}
