import { Component, OnInit, OnChanges, ElementRef, Input, ViewChild, ViewEncapsulation } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import * as d3 from 'd3';
import { MatTableDataSource } from '@angular/material/table';


@Component({
  selector: 'double-chart',
  templateUrl: './double-chart.component.html',
  styleUrls: ['./double-chart.component.css']
})
export class DoubleChartComponent implements OnInit, OnChanges {

  constructor(
    private http: HttpService
  ) { }

  @Input() serviceName: string;
  @Input() FY: string;

  @Input() headerSSI: string;
  @Input() headerCSI: string;
  @Input() ID: string;
  @Input() Showtable: boolean = true;
  @Input() TypeChart: string;
  //add by BF.24/09/2019
  @Input() yAxisMax:number;
  @Input() yAxisMin: number = 0;
    @Input() showDM: boolean = true;

  //end add BF.24/09/2019
  @ViewChild('chart', { static: true }) chartContainer: ElementRef;
  //private chartContainer: ElementRef;

  private svg: any;
  private svg2: any;
  private yScale: any;
  private yScale2: any;
  private xScale: any;
  private xAxis: any;
  private yAxis: any;
  private yAxis2: any;
  private chart1: any;
  private chart2: any;
  private barGroups: any;
  private barGroups2: any;
  private data: any
  public ColumnsSSI: string[] = [];
  public ColumnsCSI: string[] = [];
  public headerListSSI: Array<any> = [];
  public headerListCSI: Array<any> = [];
  public dataListSSI: any;
  public dataListCSI: any;
  public YearList: any;


  private MAX_WIDTH = 560;
  private MAX_HEIGHT = 320;

    public ChartID1: string;
    public ChartID2: string;


    ngOnInit() {
        this.ChartID1 = 'c1' + this.ID
        this.ChartID2 = 'c2' + this.ID

       
    this.CreateDropdownList()

  }

  ngOnChanges(): void {
    //this.SetUpchart()
  }

  ngAfterViewInit() {
    this.SetUpchart(this.ChartID1,this.ChartID2)
  }

  ngAfterContentInit() {

  }


  GenerateHearder(text) {
    let month = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
    let newtext = text.substring(4)
    return month[Number(newtext) - 1];

  }

  CreateDropdownList() {

    this.YearList = [];
    var min = new Date().getFullYear() - 1
    var max = min + 2
    //this.FY = (min + 1).toString().slice(2, 4)

    for (let i = min; i <= max; i++) {
      this.YearList.push({ YearName: i, Value: i.toString().slice(2, 4) })
    }
  }

  SetUpchart(id1,id2) {

    var margin = { top: 20, right: 20, bottom: 40, left: 40 },
      width = this.MAX_WIDTH - margin.left - margin.right,
      height = this.MAX_HEIGHT - margin.top - margin.bottom;

    let data: valueModel[];


    data = [{ catagory: 'XXX', Value: 50 }, { catagory: 'ZZZ', Value: 90 }]

      this.svg = d3.select('#' + id1).append('svg')
      .attr("id", 'barchart1' + id1)
      .attr('width', width + margin.left + margin.right)
      .attr('height', height + margin.top + margin.bottom)
      .style("background", "rgb(255, 255, 255");

      this.svg2 = d3.select('#' + id2).append('svg')
      .attr("id", 'barchart2' + id2)
      .attr('width', width + margin.left + margin.right)
      .attr('height', height + margin.top + margin.bottom)
      .style("background", "rgb(255, 255, 255");
           

    this.yScale = d3.scaleLinear()
      .range([height, 50])

    this.yScale2 = d3.scaleLinear()
      .range([height, 50])

    this.xScale = d3.scaleBand()
      .range([0, width])
      .paddingInner(0.35)
      .paddingOuter(0.1)

    this.xAxis = d3.axisBottom(this.xScale);

    this.yAxis = d3.axisLeft(this.yScale).ticks(5).tickFormat(d => this.TypeChart == 'Value' ? d + '' : d + '%');
    this.yAxis2 = d3.axisLeft(this.yScale2).ticks(5).tickFormat(d => this.TypeChart == 'Value' ? d + '' : d + '%');

    this.chart1 = this.svg.append('g')
      .attr('transform', `translate(${margin.left}, ${margin.top})`);

    this.chart1.append('g')
      .attr("class", "y axis")
      .call(this.yAxis);

    this.chart1.append('g')
      .attr("class", "xAxis")
      .attr('transform', `translate(0, ${height})`)
      .call(this.xAxis)

    this.chart2 = this.svg2.append('g')
      .attr('transform', `translate(${margin.left}, ${margin.top})`);

    this.chart2.append('g')
      .attr("class", "y axis")
      .call(this.yAxis2);

    this.chart2.append('g')
      .attr("class", "xAxis")
      .attr('transform', `translate(0, ${height})`)
      .call(this.xAxis)
      .selectAll("text")
      .style("text-anchor", "end")

    this.GetData()

  }

  GetData() {
    let test = '%PASS'
    let dataSSI = [], dataCSI = []
    this.http.get("System", this.serviceName, this.FY)
      .subscribe(result => {
        //console.log('result',result)
        dataSSI = result.Table;
        dataCSI = result.Table1;
        this.CreateChart1(dataSSI)
        this.CreateChart2(dataCSI)

        this.dataListSSI = dataSSI
        this.dataListCSI = dataCSI

        //console.log('this.dataListSSI', this.dataListSSI)

        //--- Show table data ----
        if (this.Showtable) {
          this.headerListSSI = this.GenerateHearderList(dataSSI);
          this.headerListCSI = this.GenerateHearderList(dataCSI);
          this.ColumnsSSI = this.dataListSSI.map(v => v.catagory.substring(0, 3));
          this.ColumnsSSI.unshift(test)
          this.ColumnsCSI = this.dataListCSI.map(v => v.catagory.substring(0, 3));
          this.ColumnsCSI.unshift(test)
        }
        //this.dataList = new MatTableDataSource<any>(result.Table);    

      })

  }
  CreateChart1(gData) {
      
    var margin = { top: 20, right: 20, bottom: 40, left: 40 },
      width = this.MAX_WIDTH - margin.left - margin.right,
      height = this.MAX_HEIGHT - margin.top - margin.bottom;

    let data: valueModel[];
    let hearder: string;
    let target: number;

    data = gData

    //let NoZero  = data.filter(d => d.Value != 0)
//add by BF. 24/09/2019        
    if(this.yAxisMax > this.yAxisMin && this.yAxisMax > 0){      
      this.yScale.domain([this.yAxisMin, this.yAxisMax])      
    }else{
      this.yScale.domain([this.GenerateYminDomain(data), this.GenerateYmaxDomain(data)]);
    }
    //comment old code by BF. 24/09/2019
    // var yMinDomain = this.generateYminDomain(data);
    // var yMaxDomain = this.generateYmaxDomain(data);   

    //this.yScale.domain([yMinDomain, yMaxDomain])
    //end comment by BF. 24/09/2019  
    //var yMinDomain = 75 //this.GenerateYminDomain(data)
    //var yMaxDomain = this.GenerateYmaxDomain(data)

   this.xScale.domain(data.map((d) => d.catagory))

    this.chart1
      .select(".y")
      .call(this.yAxis)

    this.chart1
      .select(".xAxis")
      .attr('transform', `translate(0, ${height})`)
      .call(this.xAxis)
      .selectAll("text")


    //create range of grid line from yScale
    this.svg.select('.grid').remove();//remove then create new because each data had scale of it.
    var makeYLines = () => d3.axisLeft(this.yScale);
    this.chart1.append('g')
      .attr('class', 'grid')
      .call(makeYLines().ticks(5).tickSize(-width).tickFormat((d) => ""));

    let xScaleOfBar = this.xScale;
    let yScaleOfBar = this.yScale;

    let chart = this.chart1;

    this.barGroups = this.chart1.selectAll(".bar1")
      .data(data)

    this.barGroups
      .enter()
      .append("rect")
      .merge(this.barGroups)
      .attr("class", "bar1")
      //.transition().duration(100)
      .attr('x', (d) => this.xScale(d.catagory))
      .attr('y', (d) => this.yScale(d.Value > 0 ? d.Value : 0))

     //.attr('height', (s) => height - this.yScale(s.Value))//comment by BF. 24/09/2019
      //add by BF. 24/09/2019 when user input [yAxisMax] [yAxisMin] then effect to yScale(s.Value)
      .attr('height', (s) => { 
        let iHeightBar = height - this.yScale(s.Value);
        if(iHeightBar<=0)
        { return 0;}
        else
        { return  iHeightBar;}
      })//end add
      .attr('width', this.xScale.bandwidth())
      .attr("fill", function (d) {
        return "#538ed5";
      }).on('mouseenter', function (actual, i) {
        d3.selectAll('.value')
          .attr('opacity', 0)

        d3.select(this)
          .transition()
          .duration(300)
          .attr('opacity', 0.6)
          .attr('x', (d) => xScaleOfBar(actual.catagory) - 5)
          .attr('width', xScaleOfBar.bandwidth() + 10)

        var y = yScaleOfBar(actual.Value);

        var line = chart.append('rect')
          .transition()
          .duration(500)
          .attr('id', 'showzone')
          .attr('x', 0)
          .attr('y', 0)
          .attr('width', width)
          .attr('height', y)
          .attr("fill", "blue")
          .attr('opacity', 0.1)
          ;

      })
      .on('mouseleave', function (actual) {
        d3.selectAll('.value')
          .attr('opacity', 1)

        d3.select(this)
          .transition()
          .duration(300)
          .attr('opacity', 1)
          .attr('x', (a) => xScaleOfBar(actual.catagory))
          .attr('width', xScaleOfBar.bandwidth())

        chart.selectAll('#showzone').remove()

      });


    this.barGroups = this.chart1.selectAll(".textvalue")
      .remove()
      .exit()
      .data(data)

    this.barGroups
      .enter()
      .append('text')
      .attr('class', 'textvalue')
      .attr('x', (d) => this.xScale(d.catagory) + this.xScale.bandwidth() / 2)
      .attr('y', (d) => this.yScale(d.Value) - 15)
      .attr('text-anchor', 'middle')
      .attr("font-size", "12")
      .attr("fill", "#000000")
      //---

      //.text((d) => this.TypeChart == 'Value' ? d.Value : d.Value + '%')
      
      .text((d)=> {
        if (this.TypeChart == 'Value') {
          return d.Value
        } else if (this.TypeChart == 'Percent') {
          //-- Return blan in case 0%
          if (d.Value > 0) {
            return d.Value + '%'
          } else {
            return ''
          }
        } else {
          return d.Value
        }
      })


    //console.log(' XX ' + this.TypeChart + ' ' + this.TypeChart.length);

    this.svg.select('.textH').remove()

    this.chart1.append('text')
      .attr('class', 'textH')
      .attr('x', width / 2)
      .attr('y', 0)
      .attr('text-anchor', 'middle')
        .attr('font-weight', 'bold')
      .text(this.headerSSI)


  }

  CreateChart2(gData) {

    var margin = { top: 20, right: 20, bottom: 40, left: 40 },
      width = this.MAX_WIDTH - margin.left - margin.right,
      height = this.MAX_HEIGHT - margin.top - margin.bottom;

    let data: valueModel[];
    let hearder: string;
    let target: number;


    data = gData
    //add by BF. 24/09/2019        
    if(this.yAxisMax > this.yAxisMin && this.yAxisMax > 0){      
      this.yScale2.domain([this.yAxisMin, this.yAxisMax])      
    }else{
      this.yScale2.domain([this.GenerateYminDomain(data), this.GenerateYmaxDomain(data)]);
    }
    // comment by BF. 24/09/2019
    // let NoZero = data.filter(d => d.Value != 0)
    // var yMinDomain = d3.min(data, d => Number(d.Value))
    // yMinDomain = yMinDomain - ((yMinDomain * 5) / 100)
    // var yMaxDomain = this.generateYmaxDomain(data)
    // this.yScale2.domain([yMinDomain, yMaxDomain])
    // end comment by BF. 24/09/2019
    this.xScale.domain(data.map((d) => d.catagory))

    //create range of grid line from yScale
    this.svg2.select('.grid').remove();//remove then create new because each data had scale of it.
    var makeYLines = () => d3.axisLeft(this.yScale2);
    this.chart2.append('g')
      .attr('class', 'grid')
      .call(makeYLines().ticks(5).tickSize(-width).tickFormat((d) => ""));

    let xScaleOfBar = this.xScale;
    let yScaleOfBar = this.yScale2;

    let chart = this.chart2;


    this.chart2
      .select(".y")
      .call(this.yAxis2);

    this.chart2
      .select(".xAxis")
      .attr('transform', `translate(0, ${height})`)
      .call(this.xAxis)
      .selectAll("text")


    this.barGroups2 = this.chart2.selectAll(".bar2")
      //.remove()
      //.exit()
      .data(data)

    this.barGroups2
      .enter()
      .append("rect")
      .merge(this.barGroups2)
      .attr("class", "bar2")
      //.transition().duration(100)
      .attr('x', (d) => this.xScale(d.catagory))
      .attr('y', (d) => this.yScale2(d.Value > 0 ? d.Value : 0))
    //.attr('height', (s) => height - this.yScale2(s.Value))//comment by BF. 24/09/2019
      //add by BF. 24/09/2019 when user input [yAxisMax] [yAxisMin] then effect to yScale(s.Value)
      .attr('height', (s) => { 
        let iHeightBar = height - this.yScale2(s.Value);
        if(iHeightBar<=0)
        { return 0;}
        else
        { return iHeightBar;}
      })//end add
      .attr('width', this.xScale.bandwidth())
      .attr("fill", function (d) {
        return "#538ed5";
      }).on('mouseenter', function (actual, i) {


        d3.selectAll('.value')
          .attr('opacity', 0)

        d3.select(this)
          .transition()
          .duration(300)
          .attr('opacity', 0.6)
          .attr('x', (d) => xScaleOfBar(actual.catagory) - 5)
          .attr('width', xScaleOfBar.bandwidth() + 10)

        var y = yScaleOfBar(actual.Value);

        var line = chart.append('rect')
          .transition()
          .duration(500)
          .attr('id', 'showzone')
          .attr('x', 0)
          .attr('y', 0)
          .attr('width', width)
          .attr('height', y)
          .attr("fill", "blue")
          .attr('opacity', 0.1)

      })
      .on('mouseleave', function (actual) {
        d3.selectAll('.value')
          .attr('opacity', 1)

        d3.select(this)
          .transition()
          .duration(300)
          .attr('opacity', 1)
          .attr('x', (a) => xScaleOfBar(actual.catagory))
          .attr('width', xScaleOfBar.bandwidth())

        chart.selectAll('#showzone').remove()

      });



    this.barGroups2 = this.chart2.selectAll(".textvalue2")
      .remove()
      .exit()
      .data(data)

    this.barGroups2
      .enter()
      .append('text')
      .attr('class', 'textvalue2')
      .attr('x', (d) => this.xScale(d.catagory) + this.xScale.bandwidth() / 2)
      .attr('y', (d) => this.yScale2(d.Value) - 15)
      .attr('text-anchor', 'middle')
      .attr("fill", "#000000")
      .attr("font-size", "12")
      //.text((d) => this.TypeChart == 'Value' ? d.Value : d.Value + '%')

      .text((d) => {
        if (this.TypeChart == 'Value') {
          return d.Value
        } else if (this.TypeChart == 'Percent') {
          //-- Return blan in case 0%
          if (d.Value > 0) {
            return d.Value + '%'
          } else {
            return ''
          }
        } else {
          return d.Value
        }
      })

    this.svg2.select('.textH').remove()

    this.chart2.append('text')
      .attr('class', 'textH')
      .attr('x', width / 2)
      .attr('y', 0)
      .attr('text-anchor', 'middle')
        .attr('font-weight','bold')
      .text(this.headerCSI)


  }



  GenerateHearderList(data) {
    let newData = [];
    for (var key in data[0]) {
      if (key != 'FY' && key != 'catagory' && key != 'Value') {
        newData.push(key)
      }

    }
    return newData
  }

  GenerateYmaxDomain(data: valueModel[]) {

    var yMaxDomain: number;

    if (this.TypeChart == 'Value') {
      yMaxDomain = 200

    } else {

      yMaxDomain = 100;
    }

    return yMaxDomain
  }

  GenerateYminDomain(data: valueModel[]) {
    var yMinDomain: number = 0;
    return yMinDomain
  }

  FYChange() {
    this.GetData()
  }
}


export interface ChartData {
  Values: Array<valueModel>;
  Legend: Array<legendModel>
}


export interface valueModel {
  catagory: string;
  Value: number;


}

export interface legendModel {
  legend: string;
  color: string;

}
