import { Component, OnInit, OnChanges, Input, SimpleChanges } from '@angular/core';
import * as D3 from 'd3';
import { HttpService } from 'src/app/core/http.service';

@Component({
  selector: 'dual-chart-3d',
  templateUrl: './dual-chart-3d.component.html',
  styleUrls: ['./dual-chart-3d.component.css']
})
export class DualChart3DComponent implements OnInit, OnChanges {
  constructor(
    private http: HttpService
  ) { }

  @Input() serviceName: string;
  @Input() ID: string;
  @Input() LegenList: any = [{ legend: "PASSsssss", color: '#99cc00' }, { legend: "FAILlllll", color: '#c00000'}]
  @Input() title: string = "Demo DualChart 3D";
  @Input() chartData: any = [];
  @Input() areaName1: string;
  @Input() areaName2: string;
  @Input() qRefresh: string;
  @Input() WIDTH: number;
  @Input() HEIGHT: number;
  @Input() qShowRegion: boolean = false;
  @Input() colors : any = [{forward : "#bfbfbf",topAndside:"#919191"},{forward : "#8397b1",topAndside:"#647285"},{forward : "#364c6e",topAndside:"#283a54"}];

  private svg: any;
  private yScale: any;
  private xScale: any;
  private xScale1: any;
  private xScale0: any;
  private xAxis: any;
  private yAxis: any;
  private chart: any;
  private z: any;
  private barGroups: any;
  private margin = { top: 30, right: 20, bottom: 20, left: 35 }
  private width1: number = 1200 - this.margin.left - this.margin.right
  private height1: number = 350 - this.margin.top - this.margin.bottom;

  private d3 : any = D3

  private topX2 = 1;

  ngOnInit() {

    //this.colors = [{forward : "#bfbfbf",topAndside:"#919191"},{forward : "#8397b1",topAndside:"#647285"},{forward : "#364c6e",topAndside:"#283a54"}]
    //this.LegenList = [{ legend: "PASSsssss", color: '#99cc00' }, { legend: "FAILlllll", color: '#c00000'}]

    if (this.WIDTH) {
        this.width1 = this.WIDTH - this.margin.left - this.margin.right
    }

    if (this.HEIGHT) {
        this.height1 = this.HEIGHT - this.margin.top - this.margin.bottom;
    }
      
    if(this.chartData.length == 0){
        this.chartData = [{
            'catagory': 'Apr',
            'Value3': 70,
            'Value2': 50,
            'Value1': 3
            
          }, {
            'catagory': 'May',
            'Value3': 50,
            'Value2': 45,
            'Value1': 4.5
          },
          {
            'catagory': 'Jun',
            'Value3': 55,
            'Value2': 40,
            'Value1': 2.2
          }, {
            'catagory': 'Jul',
            'Value3': 61,
            'Value2': 53,
            'Value1': 2.5
          }, {
            'catagory': 'Aug',
            'Value3': 52,
            'Value2': 60,
            'Value1': 3.1
          }, {
            'catagory': 'Sep',
            'Value3': 54,
            'Value2': 51,
            'Value1': 1.8
          }, {
            'catagory': 'Oct',
            'Value3': 49,
            'Value2': 52,
            'Value1': 5.2
          },
          {
            'catagory': 'Nov',
            'Value3': 59,
            'Value2': 51,
            'Value1': 1.8
          }, {
            'catagory': 'Dec',
            'Value3': 48,
            'Value2': 56,
            'Value1': 2.7
          }, {
            'catagory': 'Jan',
            'Value3': 69,
            'Value2': 55,
            'Value1': 2.9
          }, {
            'catagory': 'Feb',
            'Value3': 48,
            'Value2': 36,
            'Value1': 3.6
             },
             {
               'catagory': 'Mar',
               'Value3': 57,
               'Value2': 44,
               'Value1': 3.9
             }
          ];
    }


  }

   
  ngOnChanges(changes: SimpleChanges): void {

    if (!changes["chartData"].isFirstChange()) {
        this.d3.select("#" + 'dual3D' + this.ID).remove();
        this.setUpChart(this.ID)

      }

  }


  ngAfterViewInit() {
    //this.getData()
    this.setUpChart(this.ID)
  }


  setUpChart(id) {


    this.svg = this.d3.select('#' + id).append('svg')
      .attr("id", 'dual3D' + id)
      .attr('width', this.width1 + this.margin.left + this.margin.right)
      .attr('height', this.height1 + this.margin.top + this.margin.bottom )
      .style("background", "rgb(255, 255, 255");

    this.createChart(this.chartData)

  }

  private createChart(data): void {
        

    let gData: any[];
    gData = data

      var ymax = this.generateYmaxDomain(this.chartData)

    this.yScale = this.d3.scaleLinear()
      .range([this.height1, this.margin.top])
      .domain([0, ymax])


    this.xScale = this.d3.scaleBand()
      .range([this.margin.left, this.width1 - this.margin.right])
      .domain(gData.map((d) => d.catagory))
      .padding(0.3)

    this.xScale1 = this.d3.scaleBand()
      .domain(['Value1', 'Value2','Value3'])
      .range([0, this.xScale.bandwidth()])


    //this.xAxis = this.d3.axisBottom(this.xScale);

    //this.yAxis = this.d3.axisLeft(this.yScale);


    this.chart = this.svg.append('g')
      .attr('transform', `translate(${this.margin.left / 2}, ${this.margin.top - 30})`);

    this.chart.append('g')
      .call(this.d3.axisLeft(this.yScale).ticks(6).tickFormat(d => d+'%'))
      .attr("transform", `translate(${this.margin.left},0)`)

    this.chart.append('g')
      .attr('transform', `translate(0, ${this.height1})`)
      .call(this.d3.axisBottom(this.xScale))
      .selectAll("text")
      .style("text-anchor", "middle")

      //create range of grid line from yScale
     this.svg.select('.grid').remove();//remove then create new because each data had scale of it.
     var makeYLines = () => this.d3.axisLeft(this.yScale);
     this.chart.append('g')
       .attr('class', 'grid')
       .attr('transform', `translate(0, ${this.height1 - this.height1 })`)
       .call(makeYLines().ticks(6).tickSize(-this.width1).tickFormat((d) => ""));

    const barGroups = this.chart.selectAll()
      .data(gData)
      .enter()
      .append('g')
      .attr("transform", d => `translate(${this.xScale(d.catagory)},0)`);

    /*************************************Bar 1********************************************/

        barGroups.selectAll(".bar.Value")
          .data(d => [d])
          .enter()
          .append("rect")
          .style("fill", this.colors[0].forward)
          .attr("x", d => this.xScale1('Value1'))
          .attr("y", d => this.yScale(d.Value1))
          .attr("width",(d) => d.Value1 == 0 ? 0 : this.xScale1.bandwidth())
          .attr('height', (d) => d.Value1 == 0 ? 0 : this.height1 - this.yScale(d.Value1))
          .on("mouseover", (d) => this.handleMouseOver(d, 'Value1'))
          .on("mouseout", (d) => this.handleMouseOut(d));

        //top
        barGroups.selectAll(".bar1.ValueTop")
        .data(d => [d])
        .enter()
        .append("rect")          
        .attr("x",function(d) {
            return 8;
        })
        .attr("y", 15)
        .attr("fill", this.colors[0].topAndside)
        .attr("width",(d) => d.Value1 == 0 ? 0 : this.xScale1.bandwidth())
        .attr("height",(d) => d.Value1 == 0 ? 0 : 5)
        .attr ("transform",(d) => "translate ("+(20)+","+(this.yScale(d.Value1) - this.margin.bottom)+") skewX("+-55+")");

    /**************************************************************************************/

    /***********************************Bar 2**********************************************/

        barGroups.selectAll(".bar.Value2")
        .data(d => [d])
        .enter()
        .append("rect")
        .style("fill", this.colors[1].forward)
        .attr("x", d => this.xScale1('Value2'))
        .attr("y", d => this.yScale(d.Value2) + 5)
        .attr("width",(d) => d.Value2 == 0 ? 0 : this.xScale1.bandwidth())
          .attr('height', (d) => d.Value2 == 0 ? 0 : (this.height1 - 5) - this.yScale(d.Value2))
          .on("mouseover", (d) => this.handleMouseOver(d, 'Value2'))
          .on("mouseout", (d) => this.handleMouseOut(d));


        //top
        barGroups.selectAll(".bar2.ValueTop")
        .data(d => [d])
        .enter()
        .append("rect")          
        .attr("fill", this.colors[1].topAndside)
        .attr("width",(d) => d.Value2 == 0 ? 0 : this.xScale1.bandwidth())
        .attr("height",(d) => d.Value2 == 0 ? 0 : 5)
        .attr ("transform",(d) => "translate ("+(this.xScale1('Value2')+7)+","+(this.yScale(d.Value2))+") skewX("+-55+")");

        //side
        barGroups.selectAll(".bar2.ValueSide")
        .data(d => [d])
        .enter()
        .append("rect")       
        .attr("fill", this.colors[1].topAndside)
        .attr("width",(d) => d.Value2 > d.Value3 ? 7 : 0)
        .attr('height', (d) => d.Value2 > d.Value3 ? ((((this.height1) - this.yScale(d.Value2)) - ((this.height1) - this.yScale(d.Value3))) +4) : 0)
        .attr ("transform",(d) => "translate ("+(this.xScale.bandwidth() / 1.5)+","+( this.yScale(d.Value2) +4)+") skewY("+-30+")");

    /**************************************************************************************/

     /***********************************Bar 3**********************************************/

      barGroups.selectAll(".bar.Value3")
      .data(d => [d])
      .enter()
      .append("rect")
      .style("fill", this.colors[2].forward)
      .attr("x", d => this.xScale1('Value3'))
      .attr("y", d => this.yScale(d.Value3) + 5)
      .attr("width",(d) => d.Value3 == 0 ? 0: this.xScale1.bandwidth())
      .attr('height', (d) => d.Value3 == 0 ? 0 : (this.height1 - 5) - this.yScale(d.Value3))
        .on("mouseover", (d) => this.handleMouseOver(d, 'Value3'))
        .on("mouseout", (d) => this.handleMouseOut(d));

       //top
     barGroups.selectAll(".bar3.ValueTop")
     .data(d => [d])
     .enter()
     .append("rect")          
     .attr("fill", this.colors[2].topAndside)
     .attr("width",(d) => d.Value3 == 0 ? 0 : this.xScale1.bandwidth() + 2)
     .attr("height",(d) => d.Value3 == 0 ? 0 : 5)
     .attr ("transform",(d) => "translate ("+(this.xScale1('Value3')+7)+","+(this.yScale(d.Value3)+0.5)+") skewX("+-55+")");


      //side
      barGroups.selectAll(".bar3.ValueSide")
      .data(d => [d])
      .enter()
      .append("rect")       
      .attr("fill", this.colors[2].topAndside)
      .attr("width",(d) => d.Value3 == 0 ? 0 : 10)
      .attr('height', (d) => d.Value3 == 0 ? 0 : ((this.height1) - this.yScale(d.Value3) -4.9))
      .attr ("transform",(d) => "translate ("+(this.xScale.bandwidth() - 0.5)+","+( this.yScale(d.Value3)+6)+") skewY("+-30+")");

    /**************************************************************************************/


    barGroups.append('text')
      .attr('x', (d) => this.xScale.bandwidth() / 5)
      .attr('y', (d) => this.yScale(d.Value1) - 10)
      .attr("font-size", "8pt")
      .attr('text-anchor', 'middle')
      .attr("fill", "#000000")
      //.text((d) => this.d3.format(".1f")(d.Value1))
         // .text((d) => d.Value1 > 0 ? d.Value1 + '%' : '')


    barGroups.append('text')
      .attr('x', (d) => (this.xScale1.bandwidth() * 1.5) + 2)
      .attr('y', (d) => this.yScale(d.Value2) - 10)
      .attr("font-size", "8pt")
      .attr('text-anchor', 'middle')
      .attr("fill", "#000000")
    //  .text((d) => d.Value2 > 0 ? d.Value2 + '%' : '')

      barGroups.append('text')
      .attr('x', (d) => (this.xScale1.bandwidth() * 3))
      .attr('y', (d) => this.yScale(d.Value3) - 10)
      .attr("font-size", "8pt")
      .attr('text-anchor', 'middle')
      .attr("fill", "#000000")
      //.text((d) => d.Value3 > 0 ? d.Value3 + '%' : '')


    this.svg.append('text')
      .attr('x', this.width1 / 2 + this.margin.right)
      .attr('y', 20)
      .attr('text-anchor', 'middle')
      .attr("font-size", "20")
      .style("font-weight", 'bold')
      .style("fill", "#595959")
      .text(this.title)

    let legendList = this.LegenList
    if (this.qShowRegion) {
      //let legendList = [{ legend: this.areaName1, color: '#4f81bd' }, { legend: this.areaName2, color: '#c00000' }]
      let height = this.height1;
      let width = this.width1;

      var nodeWidth = (d) => d.getBBox().width;

      const legend = this.svg.append('g')
        .attr('class', 'legend')
        .attr('transform', 'translate(0,0)');

      const lg = legend.selectAll('g')
        .data(legendList)
        .enter()
        .append('g')
        .attr('transform', (d, i) => `translate(${i * 100},${height + 15})`);

        lg.append('rect')
            .style('fill', d => d.color)
            .attr('x', 30)
            .attr('y', 30)
            .attr('width', 25)
            .attr('height', 10);

        lg.append('text')
            .style('font-family', 'sans-serif')
            .style('font-size', '9px')
            .attr('x', 58)
            .attr('y', 39)
            .text(d => d.legend);

      let offset = 0;
      lg.attr('transform', function (d, i) {
        let x = offset;
        offset += nodeWidth(this) + 10;
        return `translate(${x},${height + 10})`;
      });

      legend.attr('transform', function () {
        return `translate(${(width - nodeWidth(this)) / 2},${0})`
      })

    }

    let xScale = this.xScale
    let xScale1 = this.xScale1
    let y = this.yScale
    let chart = this.chart
  
  }

  handleMouseOver = (d, Value) => {
    let xValue = Value == 'Value1' ? 30 : 14;
    this.chart.append("text")
      .attr('class', 'textHover')
      .attr("x", (this.xScale(d.catagory) + this.xScale.bandwidth()) - xValue)
      .attr("y", this.yScale(d[Value]) - 5)
      .attr("font-size", "7pt")
      .style("font-weight", 'bold')
      .style("fill", "#000000")
      .text(d[Value] + '%')
  }

  handleMouseOut = (d) => {
    this.chart.select('.textHover').remove()
  }

  generateYmaxDomain(data: any) {
    //var max = data.reduce((prev, current) => (prev.LAST_UPD_DATE > current.LAST_UPD_DATE) ? prev : current)

    var yMaxDomain: number = 0;
    yMaxDomain = this.d3.max(data,d => Math.max(d.Value1,d.Value2,d.Value3))
    yMaxDomain = yMaxDomain + ((yMaxDomain * 10) / 100);
    return yMaxDomain
    }


}


