import { Component, OnInit, OnChanges, ElementRef, Input, ViewChild, ViewEncapsulation } from '@angular/core';
import * as d3 from 'd3';
import { stack, pie } from 'd3';

@Component({
  selector: 'dual-scale',
  templateUrl: './dual-scale.component.html',
  styleUrls: ['./dual-scale.component.css']
})
export class DualScaleComponent implements OnInit, OnChanges {
  constructor() { }

    @Input() serviceName: string;
    @Input() data: ChartData[];
  @ViewChild('chart', { static: true }) chartContainer: ElementRef;
  //private chartContainer: ElementRef;

  margin = { top: 20, right: 20, bottom: 30, left: 40 };

  ngOnInit() {
    if (!this.data) { return; }
    this.createChart();
  }

  ngOnChanges(): void {
  }

  ngAfterViewInit() {

  }

  private createChart(): void {

    const data = this.data[0].Values;
    const legendList = this.data[0].Legend

    const margin = 60;
    const width = 700 - 2 * margin;
    const height = 400 - 2 * margin;

    var formatValue = d3.format(".0%");

    //const svg = d3.select('svg');
    //const svgContainer = d3.select('#container');
    const svg = d3.select('#dual-scale').append('svg')
      .attr('width', 750)
      .attr('height', 500)
      .style("background", "#e6e7ec");;

    const yScale = d3.scaleLinear()
      .range([height, 0])
      .domain([0,d3.max(data,d => Math.max(d.Value,d.Value2,d.Value3) + 500)])

   const yScale1 = d3.scaleLinear()
      .range([height, 0])
     .domain([0, d3.max(data, d => Math.max(d.Value, d.Value2, d.Value3) + 500) / 2])

    const xScale = d3.scaleBand()
      .range([0, width])
      .domain(data.map((d) => d.catagory))
      .padding(0.2)

    const xScale1 = d3.scaleBand()
      .domain(['Value', 'Value2','Value3'])
      .range([0, xScale.bandwidth()])

    const chart = svg.append('g')
      .attr('transform', `translate(${margin}, ${margin})`);

    chart.append('g')
      .call(d3.axisLeft(yScale).ticks(8))
      .attr("transform", "translate(0,0)")
      

    chart.append('g')
      .call(d3.axisRight(yScale1).ticks(8))
      .attr("transform", "translate(" + (width) + ",0)")

    chart.append('g')
      .attr('transform', `translate(0, ${height})`)
      .call(d3.axisBottom(xScale))
      .selectAll("text")
      .style("text-anchor", "end")
      .attr("dx", "-.8em")
      .attr("dy", ".15em")
    //.attr("transform", (d) => "rotate(-65)");


    const barGroups = chart.selectAll()
      .data(data)
      .enter()
      .append('g')
      .attr("transform", d => `translate(${xScale(d.catagory)},0)`);

    barGroups.selectAll(".bar.Value")
      .data(d => [d])
      .enter()
      .append("rect")
      .style("fill", "#4f81bd")
      .attr("x", d => xScale1('Value'))
      .attr("y", d => yScale(d.Value))
      .attr("width", xScale1.bandwidth())
      .attr('height', (d) => height - yScale(d.Value))

    barGroups.selectAll(".bar.Value2")
      .data(d => [d])
      .enter()
      .append("rect")
      .style("fill", "#c00000")
      .attr("x", d => xScale1('Value2'))
      .attr("y", d => yScale(d.Value2))
      .attr("width", xScale1.bandwidth())
      .attr('height', (d) => height - yScale(d.Value2))
      .append('text')

    barGroups.selectAll(".bar.Value3")
      .data(d => [d])
      .enter()
      .append("rect")
      .style("fill", "#00b050")
      .attr("x", d => xScale1('Value3'))
      .attr("y", d => yScale(d.Value3))
      .attr("width", xScale1.bandwidth())
      .attr('height', (d) => height - yScale(d.Value3))
      .append('text')

    barGroups.append('text')
      .attr('x', (d) => xScale.bandwidth() / 5)
      .attr('y', (d) => yScale(d.Value) - 20)
      .attr('text-anchor', 'middle')
      .attr("fill", "#000000")
      .text((d) => `${d.Value}`)

    barGroups.append('text')
      .attr('x', (d) => xScale1.bandwidth() * 1.5)
      .attr('y', (d) => yScale(d.Value2) - 20)
      .attr('text-anchor', 'middle')
      .attr("fill", "#000000")
      .text((d) => `${d.Value2}`)

    barGroups.append('text')
      .attr('x', (d) => xScale1.bandwidth() * 2.5)
      .attr('y', (d) => yScale(d.Value3) - 20)
      .attr('text-anchor', 'middle')
      .attr("fill", "#000000")
      .text((d) => `${d.Value3}`)


    svg.append('text')
      .attr('x', width / 2 + margin)
      .attr('y', 40)
      .attr("font-size", "11px")
      .attr('text-anchor', 'middle')
      .text('Simple BarChart')


    var nodeWidth = (d) => d.getBBox().width;
    const cScale = d3.scaleOrdinal(d3.schemeCategory10);

    const legend = svg.append('g')
      .attr('class', 'legend')
      .attr('transform', 'translate(0,0)');

    const lg = legend.selectAll('g')
      .data(legendList)
      .enter()
      .append('g')
      .attr('transform', (d, i) => `translate(${i * 100},${height + 15})`);

    lg.append('rect')
      .style('fill', d => cScale(d.color))
      .attr('x', 30)
      .attr('y', 90)
      .attr('width', 15)
      .attr('height', 15);

    lg.append('text')
      .style('font-family', 'Georgia')
      .style('font-size', '15px')
      .attr('x', 50)
      .attr('y', 104)
      .text(d => d.legend);

    let offset = 0;
    lg.attr('transform', function (d, i) {
      let x = offset;
      offset += nodeWidth(this) + 10;
      return `translate(${x},${height + 10})`;
    });

    legend.attr('transform', function () {
      return `translate(${(width - nodeWidth(this)) / 2},${0})`
    });

  }
}


export interface ChartData {
  Values: Array<valueModel>;
  Legend: Array<legendModel>
}


export interface valueModel {
  catagory: string;
  Value: number;
  Value2: number;
  Value3: number;


}

export interface legendModel {
  legend: string;
  color: string;

}
