import { Component, OnInit, OnChanges, ElementRef, Input, ViewChild, ViewEncapsulation, SimpleChanges } from '@angular/core';
import * as D3 from 'd3';
import { HttpService } from 'src/app/core/http.service';

@Component({
  selector: 'dual-with-line-i',
  templateUrl: './dual-with-line-i.component.html',
  styleUrls: ['./dual-with-line-i.component.css']
})
export class DualChartWithLine_IComponent implements OnInit, OnChanges {
  constructor(
    private http: HttpService
  ) { }

  @Input() serviceName: string;
  @Input() ID: string;
  @Input() LegenList: any;//[{ legend: "PASSsssss", color: '#99cc00' }, { legend: "FAILlllll", color: '#c00000'}]
  @Input() title: string;
  @Input() chartData: any;
  @Input() chartTitle: string;
  @Input() areaName1: string;
  @Input() areaName2: string;
  @Input() qRefresh: string;
  @Input() WIDTH: number;
  @Input() HEIGHT: number;
  @Input() qShowRegion: boolean;
  @Input("formatText") formatText: string = '';

  private svg: any;
  private yScale: any;
  private yScale1: any;
  private xScale: any;
  private xScale1: any;
  private xScale0: any;
  private xAxis: any;
  private yAxis: any;
  private chart: any;
  private z: any;
  private barGroups: any;
  private margin = { top: 30, right: 20, bottom: 20, left: 35 }

  private data: any = [];
  private width1: number;
  private height1: number;

  private d3 : any = D3;

  ngOnInit() {


    this.width1 = this.WIDTH - this.margin.left - this.margin.right
    this.height1 = this.HEIGHT - this.margin.top - this.margin.bottom;
  }

  ngOnChanges(changes: SimpleChanges): void {

    if (!changes["chartData"].isFirstChange()) {
      this.d3.select("#" + 'bardual' + this.ID).remove();
      this.setUpChart(this.ID)
      //this.getData()
      //this.setUpChart(this.ID)
    }

  }


  ngAfterViewInit() {
    //this.getData()
    this.setUpChart(this.ID)
  }


  setUpChart(id) {


    this.svg = this.d3.select('#' + id).append('svg')
      .attr("id", 'bardual' + id)
      .attr('width', this.width1 + this.margin.left + this.margin.right)
      .attr('height', this.height1 + this.margin.top + this.margin.bottom )
      .style("background", "rgb(255, 255, 255");

    this.createChart(this.chartData)

  }

  private createChart(data): void {


      let gData: any[];
      //gData = data.filter(x => x.catagory == null) 
    gData = data

    this.yScale = this.d3.scaleLinear()
      .range([this.height1, this.margin.top])
      .domain([0, this.d3.max(gData, d => Math.max(d.Value1, d.Value2) + 5)])

      this.yScale1 = this.d3.scaleLinear()
      .range([this.height1, this.margin.top])
      .domain([0, 100])




    this.xScale = this.d3.scaleBand()
      .range([this.margin.left, this.width1 - this.margin.right])
      .domain(gData.map((d) => d.catagory))
      .padding(0.3)

    this.xScale1 = this.d3.scaleBand()
      .domain(['Value1', 'Value2'])
      .range([0, this.xScale.bandwidth()])


    //this.xAxis = this.d3.axisBottom(this.xScale);

    //this.yAxis = this.d3.axisLeft(this.yScale);

    this.chart = this.svg.append('g')
      .attr('transform', `translate(${this.margin.left / 2}, ${this.margin.top - 30})`);

    this.chart.append('g')
      .call(this.d3.axisLeft(this.yScale).ticks(7))
      .attr("transform", `translate(${this.margin.left},0)`)

      this.chart.append('g')
         // .call(this.d3.axisRight(this.yScale1).ticks(7).tickFormat(d => d + '%'))
          .call(this.d3.axisRight(this.yScale1).ticks(7).tickFormat(d => d))
      .attr("transform", `translate(${this.width1 - this.margin.right},0)`)

    this.chart.append('g')
      .attr('transform', `translate(0, ${this.height1})`)
      .call(this.d3.axisBottom(this.xScale))
      .selectAll("text")
      .style("text-anchor", "middle")

    //create range of grid line from yScale
    this.svg.select('.grid').remove();//remove then create new because each data had scale of it.
    var makeYLines = () => this.d3.axisLeft(this.yScale);
    this.chart.append('g')
      .attr('class', 'grid')
      .attr('transform', `translate(${this.margin.top + 5}, ${this.height1 - this.height1})`)
      .call(makeYLines().ticks(7).tickSize(-(this.width1 - this.margin.right - this.margin.left)).tickFormat((d) => ""));

    const barGroups = this.chart.selectAll()
      .data(gData)
      .enter()
      .append('g')
      .attr("transform", d => `translate(${this.xScale(d.catagory)},0)`);
  

    barGroups.selectAll(".bar.Value")
      .data(d => [d])
      .enter()
      .append("rect")
      .style("fill", this.LegenList[0].color)
      .attr("x", d => this.xScale1('Value1'))
      .attr("y", d => this.yScale(d.Value1))
      .attr("width", (d) => d.Value1 == 0 ? 0 : this.xScale1.bandwidth())
      .attr('height', (d) => d.Value1 == 0 ? 0 : this.height1 - this.yScale(d.Value1))



    barGroups.selectAll(".bar.Value2")
      .data(d => [d])
      .enter()
      .append("rect")
      .style("fill", this.LegenList[1].color)
      .attr("x", d => this.xScale1('Value2'))
      .attr("y", d => this.yScale(d.Value2))
      .attr("width", (d) => d.Value2 == 0 ? 0 : this.xScale1.bandwidth())
      .attr('height', (d) => d.Value2 == 0 ? 0 : this.height1 - this.yScale(d.Value2))

    barGroups.append('text')
      .attr('x', (d) => this.xScale.bandwidth() / 5)
      .attr('y', (d) => this.yScale(d.Value1) - 5)
      .attr("font-size", "7pt")
      .attr('text-anchor', 'middle')
      .attr("fill", "#000000")
      .text((d) => d.Value1 > 0 ? d.Value1 + this.CheckTextValue() : '')


    barGroups.append('text')
      .attr('x', (d) => (this.xScale1.bandwidth() * 1.5) + 2)
      .attr('y', (d) => this.yScale(d.Value2) - 5)
      .attr("font-size", "7pt")
      .attr('text-anchor', 'middle')
      .attr("fill", "#000000")
      //.text((d) => this.d3.format(".1f")(d.Value2))
      //.text((d) => d.Value2 > 0 ? d.Value2 + '%' : '')
        .text((d) => d.Value2 > 0 ? d.Value2 + this.CheckTextValue() : '')


      var line = this.d3.line()
      .x((d, i) => this.xScale(d.catagory) + this.xScale.bandwidth() / 2)
      .y((d) => this.yScale1(d.Value3))
      //.curve(this.d3.curveMonotoneX);

    this.chart.append("path")
      .attr("fill", "none")
      .attr("stroke-width", 1.5)
      .attr("stroke", this.LegenList[2].color)   
      .attr("d", line(gData)); 



    this.chart.selectAll("bar")
      .data(gData)
      .enter()
      .append("rect")
      .attr("x", d=> (this.xScale(d.catagory) + this.xScale.bandwidth() / 2) + 5)
      .attr("y", d=> this.yScale1(d.Value3) - 10)
      .attr("width", d=> (this.xScale.bandwidth() / 2) + 5)
      .attr("height", d=> 10)
      .attr("fill", this.LegenList[2].color)

      this.chart.append("g").selectAll("text")
        .data(gData)
        .enter()
        .append("text")
        .attr("x", (d) => (this.xScale(d.catagory) + this.xScale.bandwidth() / 2) + 8)
        .attr("y", (d) => this.yScale1(d.Value3))
        .attr("fill", "black")
        .attr("font-size", 9)
        //.text((d) =>  d.Value3 + "%" );
        .text((d) => d.Value3 > 0 ? d.Value3 + this.CheckTextValue() : '')
         
      

    this.svg.append('text')
      .attr('x', this.width1 / 2 + this.margin.right)
      .attr('y', 14)
      .style("font-size", "12pt")
      .attr('text-anchor', 'middle')
      .style("font-weight", 'bold')
      .style("fill", "#595959")
      .text(this.title)

    let legendList = this.LegenList
    if (this.qShowRegion) {
      //let legendList = [{ legend: this.areaName1, color: '#4f81bd' }, { legend: this.areaName2, color: '#c00000' }]
      let height = this.height1;
      let width = this.width1;

      var nodeWidth = (d) => d.getBBox().width;

      const legend = this.svg.append('g')
        .attr('class', 'legend')
        .attr('transform', 'translate(0,0)');

      const lg = legend.selectAll('g')
        .data(legendList)
        .enter()
        .append('g')
        .attr('transform', (d, i) => `translate(${i * 100},${height + 15})`);

      lg.append('rect')
        .style('fill', d => d.color)
        .attr('x', 30)
        .attr('y', 30)
        .attr('width', 25)
        .attr('height', 10);

      lg.append('text')
        .style('font-family', 'sans-serif')
        .style('font-size', '9pt')
        .attr('x', 58)
        .attr('y', 39)
        .text(d => d.legend);

      let offset = 0;
      lg.attr('transform', function (d, i) {
        let x = offset;
        offset += nodeWidth(this) + 10;
        return `translate(${x},${height + 10})`;
      });

      legend.attr('transform', function () {
        return `translate(${(width - nodeWidth(this)) / 2},${0})`
      });


    }
  
  }

    CheckTextValue() {

        if (this.formatText == 'textValue') {

            return ''
        } else {
            return '%'
        }
    }

}


export interface ChartData {
  Values: Array<valueModel>;
  Legend: Array<legendModel>
}


export interface valueModel {
  catagory: string;
  Value1: number;
  Value2: number;
  Value3: number;


}

export interface legendModel {
  legend: string;
  color: string;

}
