import { Component, OnInit, OnChanges, Input, SimpleChanges } from '@angular/core';
import * as D3 from 'd3';
import { keys } from 'd3';
import { HttpService } from 'src/app/core/http.service';

@Component({
  selector: 'guage-i-chart',
  templateUrl: './guage-i-chart.component.html',
  styleUrls: ['./guage-i-chart.component.css']
})
export class GuageIComponent implements OnInit, OnChanges {
  constructor(
    private http: HttpService
  ) { }

  @Input() serviceName: string;
  @Input() ID: string;
  @Input() LegenList: any = [{ legend: "PASSsssss", color: '#99cc00' }, { legend: "FAILlllll", color: '#c00000' }]
  @Input() title: string = "Demo DualChart 3D";
  @Input() chartData: any = [];
  @Input() areaName1: string;
  @Input() areaName2: string;
  @Input() qRefresh: string;
  @Input() WIDTH: number;
  @Input() HEIGHT: number;
  @Input() qShowRegion: boolean = false;
  @Input() colors: any = [{ forward: "#bfbfbf", topAndside: "#919191" }, { forward: "#8397b1", topAndside: "#647285" }, { forward: "#364c6e", topAndside: "#283a54" }];

  // private svg: any;
  private yScale: any;
  private xScale: any;
  private xScale1: any;
  private xScale0: any;
  private xAxis: any;
  private yAxis: any;
  private chart: any;
  private z: any;
  private barGroups: any;
  // private margin = { top: 30, right: 20, bottom: 20, left: 35 }
  //private width1: number = 1200 - this.margin.left - this.margin.right
  //private height1: number = 350 - this.margin.top - this.margin.bottom;

  private d3: any = D3;
  private needle: any;
  private width: any;
  private height: any;
  private margin: any;
  private n: any;
  private radius: any;
  private needleRad: any;
  private pi: any;
  private halfPi: any;
  private endAngle: any;
  private startAngle: any;
  private data: any;
  private _data: any;
  private tt: any;
  private scale: any;
  private colorScale: any;
  private svg: any;
  private text: any;


  ngOnInit() {

    //this.SetUpChart(this.ID);

  }


  ngOnChanges(changes: SimpleChanges): void {

    if (!changes["qRefresh"].isFirstChange()) {
      this.d3.select("#" + 'guage-i' + this.ID).remove();
      this.SetUpChart(this.ID)

    }

  }


  ngAfterViewInit() {
    this.SetUpChart(this.ID)
  }

  //-------------- New -------------------


  public SetUpChart(pID) {
    //------------
    //let d3: any = d3;
    console.log('this.ID ' + this.ID);

    this.width = 300;
    this.height = 200;
    this.margin = 10;
    this.n = 20;
    this.radius = this.width / 2 - (this.margin * 2);
    this.needleRad = 10; //-- mile size
    this.pi = Math.PI;
    this.halfPi = this.pi / 2;
    this.endAngle = this.pi / 2;
    this.startAngle = -this.endAngle;
    console.log(' this.pi' + this.pi);
    console.log(' this.startAngle' + this.startAngle);
    console.log(' this.endAngle' + this.endAngle);
    this.data = this.d3.range(this.startAngle, this.endAngle, this.pi / this.n);
    this._data = this.data.slice(0);
    this.tt = 3000;
    this.scale = this.d3.scaleLinear().range([this.startAngle, this.endAngle]);
    this.colorScale = this.d3.scaleSequential(this.d3.interpolateRdYlGn)
      .domain([this.data[0], this.data[this.data.length - 1]]);

    this.svg = this.d3.select('svg')
      .append('g')
      .attr('transform', 'translate(' + this.width / 2 + ',' + (this.height - this.margin) + ')');


    this._data.push(this.endAngle);

    var arc = this.d3.arc()
      .innerRadius(this.radius - (this.radius / 5)-10)
      .outerRadius(this.radius)
      .startAngle(function (d) { return d; })
      .endAngle((d, i) => this._data[i + 1]);

    var slice = this.svg.append('g').selectAll('path.slice').data(this.data);

    slice
      .enter()
      .append('path')
      .attr('class', 'slice')
      .attr('d', arc)
      .attr('padding', '1px')
      .attr('fill', (d) => this.colorScale(d));
      //.attr('fill', '#44f714');

    this.needle = this.svg.append('g')
      .append('path')
      .attr('class', 'needle')
      .attr('fill-opacity', .7)
      .attr('stroke', 'black');

    this.text = this.svg.append('g')
      .append('text')
      .attr('class', 'text')
      .attr('text-anchor', 'middle')
      .attr('dy', '-0.45em')
      .classed('monospace', true);

    var value = 0.5;
    this.svg.append('text')
      .attr('class', 'textH')
      .attr('x',0)
      .attr('y', -50)
      .attr("font-size", "20px")
      .attr('text-anchor', 'middle')
      .text(value)


    this.UpdateGuage(this.scale(0), this.scale(value));
  }


  private UpdateGuage(oldValue, newValue) {
    this.needle
      .datum({ oldValue: oldValue })
      .transition().duration(this.tt)
      .attrTween('d', this.LineTween(newValue));

    this.text
      .datum({ oldValue: oldValue })
      .transition().duration(this.tt)
      .attrTween('transform', this.TransformTween(newValue))
      .tween('text', this.TextTween(newValue));
  }

  private TextTween(newValue) {
    return (d) => {

      var that = this.d3.select(this),
        i = this.d3.interpolate(d.oldValue, newValue);

      return (t) => {
        that.text(this.d3.format('.1%')(this.scale.invert(i(t))));
      };
    };
  }

  private TransformTween(newValue) {
    return (d) => {
      var interpolate = this.d3.interpolate(d.oldValue, newValue);

      return (t) => {
        var _in = interpolate(t) - this.halfPi,
          centerX = (this.radius + 20) * Math.cos(_in),
          centerY = (this.radius + 20) * Math.sin(_in);
        return 'translate(' + centerX + ',' + centerY + ')';
      };
    };
  }

  private LineTween(newValue) {
    return (d) => {

      var interpolate = this.d3.interpolate(d.oldValue, newValue);

      return (t) => {

        var _in = interpolate(t) - this.halfPi,
          _im = _in - this.halfPi,
          _ip = _in + this.halfPi;

        var topX = this.radius * Math.cos(_in),
          topY = this.radius * Math.sin(_in);

        var leftX = this.needleRad * Math.cos(_im),
          leftY = this.needleRad * Math.sin(_im);

        var rightX = this.needleRad * Math.cos(_ip),
          rightY = this.needleRad * Math.sin(_ip);

        return this.d3.line()([[topX, topY], [leftX, leftY], [rightX, rightY]]) + 'Z';
      };
    };
  }
}


