import { Component, OnInit, OnChanges, Input, SimpleChanges } from '@angular/core';
import * as D3 from 'd3';
import { keys } from 'd3';
//import { tip } from 'src/assets/tip.js';
import { HttpService } from 'src/app/core/http.service';

@Component({
  selector: 'guage-ii-chart',
  templateUrl: './guage-ii-chart.component.html',
  styleUrls: ['./guage-ii-chart.component.css']
})
export class GuageIIComponent implements OnInit, OnChanges {
  constructor(
    private http: HttpService
  ) { }

  @Input() serviceName: string;
  @Input() ID: string;
  @Input() LegenList: any = [{ legend: "Promoters", color: '#10b225' }, { legend: "Passives", color: '#ffa200' }, { legend: "Detractors", color: '#e61403' }]
  @Input() title: string = "Demo DualChart 3D";
  @Input() chartData: any = [];
  @Input() areaName1: string;
  @Input() areaName2: string;
  @Input() qRefresh: string;
  @Input() WIDTH: number;
  @Input() HEIGHT: number;
  @Input() qShowRegion: boolean = false;
  @Input() colors: any = [{ forward: "#bfbfbf", topAndside: "#919191" }, { forward: "#8397b1", topAndside: "#647285" }, { forward: "#364c6e", topAndside: "#283a54" }];
  @Input() dataTicks: any = '[40,  70]';
  @Input() iGvalue: number = 58;
  @Input() dataTarget: number = 40;
  @Input() dataValue: number = 70;
  @Input() adjustLine: string = 'Y';
  @Input() pMonth: string = '';


  private svg: any;
  private yScale: any;
  private xScale: any;
  private xScale1: any;
  private xScale0: any;
  private xAxis: any;
  private yAxis: any;
  private chart: any;
  private z: any;
  private barGroups: any;
  private margin = { top: 30, right: 20, bottom: 20, left: 35 }
  //private width1: number = 1200 - this.margin.left - this.margin.right
  //private height1: number = 350 - this.margin.top - this.margin.bottom;

  private d3: any = D3;

  //private dataTicks: any = '[0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100]';

  ngOnInit() {

    //this.SetUpChart(this.ID);
    //this.onDocumentReady();
  }


  ngOnChanges(changes: SimpleChanges): void {

    if (!changes["iGvalue"].isFirstChange()) {

      //this.d3.select("#" + this.ID).remove();
      // this.SetUpChart(this.ID, this.iGvalue, this.dataValue, this.dataTarget);
      this.UpdateReadings(this.iGvalue, this.dataValue, this.dataTarget, this.adjustLine);
    } else {

    }

  }


  ngAfterViewInit() {

    if (this.iGvalue == undefined) {
      this.iGvalue = 0;
      this.dataTicks = [50, 100];
      this.dataTarget = 100;
      this.dataValue = 50;
      this.adjustLine = 'Y';
    }
    this.SetUpChart(this.ID, this.iGvalue, this.dataValue, this.dataTarget, this.adjustLine);
  }

  public powerGauge: any;
  private SetUpChart(pID, pValue, pFrom, pTo, pAdjustLine) {
    console.log('SetUpChart iValue ' + pValue);
    this.powerGauge = this.RenderGauge('#' + pID, {
      size: 160,
      clipWidth: 250,
      clipHeight: 150,
      ringWidth: 15,
      maxValue: 100,
      transitionMs: 2000,
      //arcColorFn: this.d3.scaleOrdinal().range(['#3182bd', '#9ecae1'])
      arcColorFn: this.d3.scaleOrdinal().range(['#10b225', '#ffa200', '#e61403'])

      //
    });
    //console.log('SetUpChart 23 iGvalue ' + this.iGvalue);
    this.powerGauge.render(pValue, pFrom, pTo, pAdjustLine );
    this.UpdateReadings(pValue, pFrom, pTo, pAdjustLine );


    //this.CreateLegen([]);

    //setInterval(function () {
    //  UpdateReadings();
    //}, 3 * 1000);
  }

  private UpdateReadings(pValue, pFrom, pTo, pAdjustLine) {

    this.powerGauge.update(pValue, pFrom, pTo, pAdjustLine);
  }

  private RenderGauge(container, configuration) {
    
    var that: any = {};
    var chart: any
    var config = {
      size: 160,
      clipWidth: 250,
      clipHeight: 150,
      ringInset: 15,
      ringWidth: 15,

      pointerWidth: 10,
      pointerTailLength: 5,
      pointerHeadLengthPercent: 0.9,

      minValue: 0,
      maxValue: 100,

      minAngle: -90,
      maxAngle: 90,

      transitionMs: 750,

      majorTicks: 5,
      labelFormat: this.d3.format('.3'),
      labelInset: 10,

      arcColorFn: this.d3.interpolateHsl(this.d3.rgb('#e8e2ca'), this.d3.rgb('#3e6c0a'), this.d3.rgb('#666'))
    };

    this.WIDTH = config.clipWidth;
    this.HEIGHT = config.clipHeight;
    var range = undefined;
    var r = undefined;
    var pointerHeadLength = undefined;
    var value = 0;

    var svg = undefined;
    var arc = undefined;
    var arcInside = undefined;
    var scale = undefined;
    var ticks = undefined;
    var tickData = undefined;
    var pointer = undefined;
    var gagues = undefined;

    //var tip = this.d3.tip()
    //  .attr('class', 'd3-tip')
    //  .offset([-10, 0])
    //  .html(function (d) {
    //    return "<strong>Value:</strong> <span style='color:red'>" + d.value + "</span><br \><strong>Target:</strong> <span style='color:red'>" + d.target + "</span>";
    //  });

    var donut = this.d3.pie();

    function deg2rad(deg) {
      return deg * Math.PI / 180;
    }

    function newAngle(d) {
      var ratio = scale(d);
      var newAngle = config.minAngle + (ratio * range);
      return newAngle;
    }

    var configure = (configuration) => {
      var prop = undefined;
      for (prop in configuration) {
        config[prop] = configuration[prop];
      }

      range = config.maxAngle - config.minAngle;
      r = config.size / 2;
      pointerHeadLength = Math.round(r * config.pointerHeadLengthPercent);

      // a linear scale that maps domain values to a percent from 0..1
      scale = this.d3.scaleLinear()
        .range([0, 1])
        .domain([config.minValue, config.maxValue]);

      arc = this.d3.arc()
        .innerRadius(r - config.ringWidth - config.ringInset)
        .outerRadius(r - config.ringInset)
        .startAngle(function (d, i) {
          var ratio = d[0] * range / (config.maxValue - config.minValue);
          return deg2rad(config.minAngle + ratio);
        })
        .endAngle(function (d, i) {
          var ratio = d[1] * range / (config.maxValue - config.minValue);
          return deg2rad(config.minAngle + ratio);
        });


      //arcInside = this.d3.arc()
      //  .innerRadius(r - config.ringWidth - config.ringInset -20)
      //  .outerRadius(r - config.ringInset-20)
      //  .startAngle(function (d, i) {
      //    var ratio = d[0] * range / (config.maxValue - config.minValue);
      //    return deg2rad(config.minAngle + ratio);
      //  })
      //  .endAngle(function (d, i) {
      //    var ratio = d[1] * range / (config.maxValue - config.minValue);
      //    return deg2rad(config.minAngle + ratio);
      //  });
    }
    that.configure = configure;

    function centerTranslation() {
      return 'translate(' + r + ',' + r + ')';
    }

    function isRendered() {
      return (svg !== undefined);
    }
    that.isRendered = isRendered;

    var render = (newValue, dValue, dTarget, pAdjustLine) => {
      chart = this.d3.selectAll(container).datum(function () { return this.dataset; });

      svg = chart
        .append('svg:svg')
        .attr('class', 'gauge')
        .attr('width', config.clipWidth)
        .attr('height', config.clipHeight);

      //svg.call(tip);

      var centerTx = centerTranslation();

      var arcs = svg.append('g')
        .attr('class', 'arc')
        .attr('transform', centerTx);

      arcs.selectAll('path')
        .data((d) => {

          var ticks = JSON.parse('[' + dValue + ' ,' + dTarget + ']');
          //var ticks = JSON.parse(d.ticks);
          return this.d3.pairs(ticks.concat(config.minValue, config.maxValue).sort(this.d3.ascending));
        })
        .enter().append('path')
        .attr('fill', function (d, i) {
          return config.arcColorFn(i);
        })
        .attr('d', arc);

      var lg = svg.append('g')
        .attr('class', 'label')
        .attr('transform', centerTx);

      if (pAdjustLine == 'N') {
        //* -- Add text --
        lg.selectAll('text')
          .data(function (d) {
            return JSON.parse('[' + dValue + ' ,' + dTarget + ']');
          })
          .enter().append('text')
          .attr("font-size", "8pt")
          .attr('transform', function (d) {
            var ratio = scale(d);

            var newAngle = config.minAngle + (ratio * range);

            return 'rotate(' + (newAngle - 10) + ') translate(0,' + (config.labelInset - r) + ')';
          })
          .text(config.labelFormat);

      } else {

        //* -- Add text --
        lg.selectAll('text')
          .data(function (d) {
            return JSON.parse('[' + dValue + ' ,' + (dTarget - dValue) + ',' + (100 - dTarget) + ']');
          })
          .enter().append('text')
          .attr("font-size", "8pt")
          .attr('transform', function (d) {
            var ratio = scale(d);

            var newAngle = config.minAngle + (ratio * range);
            if (newAngle < 0) {
              newAngle = newAngle * -1;
            }

           // console.log('config.minAngle ' + newAngle);
            if (ratio * 100 > 60) {
              newAngle = newAngle - 30
            } else if (ratio * 100 > 30) {
              newAngle = newAngle - 10
            } else if (ratio * 100 > 20) {
              newAngle = newAngle - 5
            }

            return 'rotate(' + newAngle + ') translate(0,' + (config.labelInset - r) + ')';
          })
          .text(config.labelFormat);
      }

      var lineData = [[config.pointerWidth / 2, 0],
      [0, -pointerHeadLength],
      [-(config.pointerWidth / 2), 0],
      [0, config.pointerTailLength],
      [config.pointerWidth / 2, 0]];

      var targetData = [[0, config.labelInset - r], [-config.pointerWidth / 2,
      config.labelInset - r - (config.pointerWidth / 2)], [config.pointerWidth / 2, config.labelInset - r - (config.pointerWidth / 2)]];


      // var pointerLine = this.d3.line().curve(this.d3.curveMonotoneX);

      //var pg = svg.append('g')
      //  .attr('class', 'pointer')
      //  .attr('transform', centerTx);

      //pointer = pg.append('path')
      //  .attr('d', pointerLine(lineData)/*function(d) { return pointerLine(d) +'Z';}*/)
      //  .attr('transform', 'rotate(' + config.minAngle + ')')
      //.on('mouseover', tip.show)
      // .on('mouseout', tip.hide);

      //pg.append('path')
      //  .attr("class", "target")
      //  .attr('d', pointerLine(targetData)/*function(d) { return pointerLine(d) +'Z';}*/)
      //  .attr('transform', 'rotate(' + config.minAngle + ')')
      // .on('mouseover', tip.show)
      //  .on('mouseout', tip.hide);

      
      update(newValue, dValue, dTarget, pAdjustLine, configure);
    }

    that.render = render;

    var update = (newValue, dValue, dTarget, pAdjustLine, newConfiguration) => {
      if (newConfiguration !== undefined) {
        configure(newConfiguration);
      }

      //--- clear first -------
      svg.selectAll('text').remove();
      svg.selectAll('g').remove();
      svg.selectAll('path').remove();
      //--------------- Add new for new curve --------------------
      var centerTx = centerTranslation();

      var arcs = svg.append('g')
        .attr('class', 'arc')
        .attr('transform', centerTx);

      arcs.selectAll('path')
        .data((d) => {
          var ticks = JSON.parse('[' + dValue + ' ,' + dTarget + ']');
          return this.d3.pairs(ticks.concat(config.minValue, config.maxValue).sort(this.d3.ascending));
        })
        .enter().append('path')
        .attr('fill', function (d, i) {
          return config.arcColorFn(i);
        })
        .attr('d', arc);

      var lg = svg.append('g')
        .attr('class', 'label')
        .attr('transform', centerTx);
      //console.log('pAdjustLine : ' + pAdjustLine);
      if (pAdjustLine == 'N') {
        //* -- Add text --
        lg.selectAll('text')
          .data(function (d) {
            return JSON.parse('[' + dValue + ' ,' + dTarget + ']');
          })
          .enter().append('text')
          .attr("font-size", "8pt")
          .attr('transform', function (d) {
            var ratio = scale(d);

            var newAngle = config.minAngle + (ratio * range);

            return 'rotate(' + (newAngle - 10) + ') translate(0,' + (config.labelInset - r) + ')';
          })
          .text(config.labelFormat);

      } else {

        //* -- Add text --
        lg.selectAll('text')
          .data(function (d) {
            return JSON.parse('[' + dValue + ' ,' + (dTarget - dValue) + ',' + (100 - dTarget) + ']');
          })
          .enter().append('text')
          .attr("font-size", "8pt")
          .attr('transform', function (d) {
            var ratio = scale(d);


            var newAngle = config.minAngle + (ratio * range);
            if (newAngle < 0) {
              newAngle = newAngle * -1;
            }

            if (ratio * 100 > 60) {
              newAngle = newAngle - 30
            } else if (ratio * 100 > 30) {
              newAngle = newAngle - 10
            } else if (ratio * 100 > 20) {
              newAngle = newAngle - 5
            }


            return 'rotate(' + (newAngle) + ') translate(0,' + (config.labelInset - r) + ')';
          })
          .text(config.labelFormat);
      }


      // console.log('config.labelFormat ' + config.labelFormat);

      //--------------- End curve ----------------
      svg.append('text')
        .attr('class', 'textH')
        .attr('x', (config.clipWidth / 2)-43)
        .attr('y', (config.clipHeight) - 80)
        .attr("font-size", "16pt")
        .attr('text-anchor', 'middle')
        .attr("fill", "#000")
        .text(newValue)

      svg.append('text')
        .attr('class', 'textH')
        .attr('x', (config.clipWidth / 2) - 43)
        .attr('y', (config.clipHeight) - 60)
        .attr("font-size", "9pt")
        .attr('text-anchor', 'middle')
        .attr("fill", "#9FAAAE")
        .text('NPS')

      svg.append('text')
        .attr('class', 'textH')
        .attr('x', (config.clipWidth / 2) - 43)
        .attr('y', (config.clipHeight) - 40)
        .attr("font-size", "9pt")
        .attr('text-anchor', 'middle')
        .attr("fill", "#000")
        .text(this.pMonth)

      this.CreateLegen(svg, config.clipWidth, config.clipHeight)
      var pointers = chart
        .select(".pointer path");

      pointers
        .transition()
        .duration(config.transitionMs)
        .ease(this.d3.easeElastic)
        .attr('transform', function (d) {
          var ratio = scale(d.value);
          var newAngle = config.minAngle + (ratio * range);
          return 'rotate(' + newAngle + ')';
        });

      chart
        .select(".pointer .target")
        .attr('transform', function (d) {
          var ratio = scale(d.target);
          var newAngle = config.minAngle + (ratio * range);
          return 'rotate(' + newAngle + ')';
        });;
    }
    that.update = update;

    configure(configuration);

    return that;

  };

  CreateLegen(svg : any,width,height) {
    let legendList = this.LegenList

    var nodeWidth = (d) => d.getBBox().width;
    //const cScale = this.d3.scaleOrdinal();

    const legend = svg.append('g')
      .attr('class', 'legend')
      .attr('transform', 'translate(0,0)');

    const lg = legend.selectAll('g')
      .data(legendList)
      .enter()
      .append('g')
      .attr('transform', (d, i) => `translate(${i * 100},${height + 15})`);

    lg.append('rect')
      .style('fill', d => d.color)
      .attr('x', 35)
      .attr('y', (height + this.margin.top + this.margin.bottom + this.margin.left))
      .attr('width', 15)
      .attr('height', 10);

    lg.append('text')
      //.style('font-family', 'Georgia')
      .style('font-size', '8pt')
      .attr('x', 55)
      .attr('y', (height + this.margin.top + this.margin.bottom + this.margin.left + 10))
      .text(d => d.legend);

    let offset = 0;
    lg.attr('transform', function (d, i) {
      let x = offset;
      offset += nodeWidth(this) + 10;
      return `translate(${x},${-40})`;
    });

    legend.attr('transform', function () {
      return `translate(${-30},${-70})`
    });
  }

 

}
