import { Component, OnInit, OnChanges, Input, SimpleChanges } from '@angular/core';
import * as D3 from 'd3';
import { keys } from 'd3';

import { HttpService } from 'src/app/core/http.service';

@Component({
  selector: 'guage-iii-chart',
  templateUrl: './guage-iii-chart.component.html',
  styleUrls: ['./guage-iii-chart.component.css']
})
export class GuageIIIComponent implements OnInit, OnChanges {
  constructor(
    private http: HttpService
  ) { }

  @Input() serviceName: string;
  @Input() ID: string;
  @Input() title: string = "Demo DualChart 3D";
  @Input() qRefresh: string;
  @Input() WIDTH: number;
  @Input() HEIGHT: number;
  @Input() iGvalue: number = 0;

  // private svg: any;
  private yScale: any;
  private xScale: any;
  private xScale1: any;
  private xScale0: any;
  private xAxis: any;
  private yAxis: any;
  private chart: any;
  private z: any;
  private barGroups: any;

  //private iValue: number;
  // private margin = { top: 30, right: 20, bottom: 20, left: 35 }
  //private width1: number = 1200 - this.margin.left - this.margin.right
  //private height1: number = 350 - this.margin.top - this.margin.bottom;

  private d3: any = D3;


  ngOnInit() {

  }

  ngOnChanges(changes: SimpleChanges): void {
    if (!changes["iGvalue"].isFirstChange()) {
      this.UpdateReadings(this.iGvalue);
    }
  }


  ngAfterViewInit() {
    this.SetUpChart(this.ID, this.iGvalue, 0);
  }

  public PowerGauge: any;
  private SetUpChart(pID, iValue, qFlag) {

    this.PowerGauge = this.RenderGauge('#' + pID, {
      size: 160,
      clipWidth: 160,
      clipHeight: 100,
      ringWidth: 30,
      maxValue: 1000,
      transitionMs: 4000,
    });
    this.PowerGauge.render();

    // every few seconds update reading values
    this.UpdateReadings(iValue);
    //setInterval(function () {
    //  UpdateReadings();
    //}, 6 * 1000);



  }

  UpdateReadings(iValue) {
    this.PowerGauge.update(iValue);
  }


  private RenderGauge(container, configuration) {
    var that: any = {};
    var chart: any
    var config = {
      size: 160,
      clipWidth: 160,
      clipHeight: 100,
      ringInset: 10,
      ringWidth: 30,

      pointerWidth: 10,
      pointerTailLength: 4,
      pointerHeadLengthPercent: 0.9,

      minValue: 0,
      maxValue: 1000,

      minAngle: -90,
      maxAngle: 90,

      transitionMs: 750,

      majorTicks: 10, //-- Tick
      labelFormat: this.d3.format('.3'),
      labelInset: 10,

      arcColorFn: this.d3.interpolateHsl(this.d3.rgb('#b6ff00'), this.d3.rgb('#0ed027'))

    };
    var range = undefined;
    var r = undefined;
    var pointerHeadLength = undefined;
    var value = 0;

    var svg = undefined;
    var arc = undefined;
    var scale = undefined;
    var ticks = undefined;
    var tickData = undefined;
    var pointer = undefined;

    //var donut = this.d3.layout.pie();

    function deg2rad(deg) {
      return deg * Math.PI / 180;
    }

    function newAngle(d) {
      var ratio = scale(d);
      var newAngle = config.minAngle + (ratio * range);
      return newAngle;
    }

    var configure = (configuration) => {
      var prop = undefined;
      for (prop in configuration) {
        config[prop] = configuration[prop];
      }

      range = config.maxAngle - config.minAngle;
      r = config.size / 2;
      pointerHeadLength = Math.round(r * config.pointerHeadLengthPercent);

      // a linear scale that maps domain values to a percent from 0..1
      scale = this.d3.scaleLinear()
        // scale = this.d3.scale.linear()
        .range([0, 1])
        .domain([config.minValue, config.maxValue]);

      ticks = scale.ticks(config.majorTicks);
      tickData = this.d3.range(config.majorTicks).map(function () { return 1 / config.majorTicks; });

      arc = this.d3.arc()
        // arc = this.d3.svg.arc()
        .innerRadius(r - config.ringWidth - config.ringInset)
        .outerRadius(r - config.ringInset)
        .startAngle(function (d, i) {
          var ratio = d * i;
          return deg2rad(config.minAngle + (ratio * range));
        })
        .endAngle(function (d, i) {
          var ratio = d * (i + 1);
          return deg2rad(config.minAngle + (ratio * range));
        });
    }
    that.configure = configure;

    function centerTranslation() {
      return 'translate(' + r + ',' + r + ')';
    }

    function isRendered() {
      return (svg !== undefined);
    }
    that.isRendered = isRendered;

    var render = (newValue) => {
      svg = this.d3.selectAll(container)
        .append('svg:svg')
        .attr('class', 'gauge')
        .attr('width', config.clipWidth)
        .attr('height', config.clipHeight + 10);

      var centerTx = centerTranslation();

      var arcs = svg.append('g')
        .attr('class', 'arc')
        .attr('transform', centerTx);

      arcs.selectAll('path')
        .data(tickData)
        .enter().append('path')
        .attr('fill', function (d, i) {
          return config.arcColorFn(d * i);
        })
        .attr('d', arc);

      var lg = svg.append('g')
        .attr('class', 'label')
        .attr('transform', centerTx);

      //write text
      //lg.selectAll('text')
      //  .data(ticks)
      //  .enter().append('text')
      //  .attr('transform', function (d) {
      //    var ratio = scale(d);
      //    var newAngle = config.minAngle + (ratio * range);
      //    return 'rotate(' + newAngle + ') translate(0,' + (config.labelInset - r) + ')';
      //  })
      //  .text(config.labelFormat);

      var lineData = [[config.pointerWidth / 2, 0],
      [0, -pointerHeadLength],
      [-(config.pointerWidth / 2), 0],
      [0, config.pointerTailLength],
      [config.pointerWidth / 2, 0]];

      var pointerLine = this.d3.line().curve(this.d3.curveMonotoneX); //this.d3.line().interpolate('monotone');
      var pg = svg.append('g').data([lineData])
        .attr('class', 'pointer')
        .attr('transform', centerTx);

      pointer = pg.append('path')
        .attr('d', pointerLine/*function(d) { return pointerLine(d) +'Z';}*/)
        .attr('transform', 'rotate(' + config.minAngle + ')');

      //Create and append rectangle element

      svg.append("circle")
        .attr("cx", config.clipWidth / 2)
        .attr("cy", config.clipWidth / 2)
        .attr("r", 32)
        .attr("class", "circle-text")
        .attr("fill", "#000")

      // bine the circle
      svg.append("rect")
        .attr("x", 0)
        .attr("y", config.size/2 )
        .attr("width", config.clipWidth)
        .attr("height", config.clipHeight /2)
        .attr("class", "rect-hide")
        .attr("fill", "#fff")

      // Text meter

      update(newValue, configure);


    }
    that.render = render;

    var update = (newValue, newConfiguration) => {
      if (newConfiguration !== undefined) {
        configure(newConfiguration);
      }


      //----- Pointer arrow
      //var centerTx = centerTranslation();

      //var lineData = [[config.pointerWidth / 2, 0],
      //[0, -pointerHeadLength],
      //[-(config.pointerWidth / 2), 0],
      //[0, config.pointerTailLength],
      //[config.pointerWidth / 2, 0]];

      //var pointerLine = this.d3.line().curve(this.d3.curveMonotoneX); //this.d3.line().interpolate('monotone');
      ////svg.selec
      //svg.select('pointer').remove();
      //var pg = svg.append('g').data([lineData])
      //  .attr('class', 'pointer')
      //  .attr('transform', centerTx);

      //pointer = pg.append('path')
      //  .attr('d', pointerLine/*function(d) { return pointerLine(d) +'Z';}*/)
      //  .attr('transform', 'rotate(' + config.minAngle + ')');


      // this.iValue = newValue % 1000;
      var value = newValue; //this.iGvalue;
      svg.selectAll('text').remove();
      svg.append('text')
        .attr('class', 'textH')
        .attr('x', config.clipWidth / 2)
        .attr('y', (config.clipWidth / 2 - 4) )
        .attr("font-size", "16px")
        .attr('text-anchor', 'middle')
        .attr("fill", "#fff")
        .text(value)


      
      var ratio = scale(newValue);
      var newAngle = config.minAngle;
      pointer.transition()
        .duration(config.transitionMs)
       // .ease('elastic')
        .attr('transform', 'rotate(0)');

      ratio = scale(newValue);
      newAngle = config.minAngle + (ratio * range);
      pointer.transition()
        .duration(config.transitionMs)
        // .ease('elastic')
        .attr('transform', 'rotate(' + newAngle + ')');
    }
    that.update = update;

    configure(configuration);

    return that;
  };



}
