import { Component, ElementRef, Input, OnInit, OnChanges, SimpleChanges } from '@angular/core';
import * as D3 from 'd3';


@Component({
  selector: 'line-i-chart',
  templateUrl: './line-i-chart.component.html',
  styleUrls: ['./line-i-chart.component.css']
})


export class LineIChartComponent implements OnInit, OnChanges {

  private htmlElement: HTMLElement;
  @Input("ID") ID: string;
  @Input("data") data: any = []
  @Input("lineColor") lineColor: string
  @Input("title") title: string
  @Input("WidthSizeChart") WidthSizeChart: any;
  @Input("HeigthSizeChart") HeigthSizeChart: any;
  @Input("gValue") gValue: string = 'value';
  @Input("gTarget") gTarget: string = 'TARGET_SCORE';

  constructor(elementRef: ElementRef) {
    this.htmlElement = elementRef.nativeElement;  // reference to <line-chart> element from the main template

    //console.log(D3);

  }

  private d3: any = D3;
  private margin = { top: 18, right: 30, bottom: 20, left: 30 }
  private width = 760 - this.margin.left - this.margin.right
  private height = 300 - this.margin.top - this.margin.bottom;

  ngOnInit() {
    //this.SetupChart(this.ID)
    if (this.WidthSizeChart) {
      this.width = this.WidthSizeChart - this.margin.left - this.margin.right
    }
    if (this.HeigthSizeChart) {
      this.height = this.HeigthSizeChart - this.margin.top - this.margin.bottom
    }
  }

  ngOnChanges(changes: SimpleChanges): void {

    if (!changes["data"].isFirstChange()) {


      setTimeout(() => {
        this.CreateChart(this.ID);
      }, 1000);

      // console.log('this.ID : ' + this.ID);
      // console.log('gValue : ' + this.gValue);
     

    }
  }

  ngAfterViewInit() {

    this.CreateChart(this.ID)
  }


  CreateChart(id) {



    this.d3.select("#" + 'linechart' + id).remove();

    this.data = this.data.filter(x => x[this.gValue] != null)

  
    // parse the date / time
    var parseDate = this.d3.timeParse("%Y%m");
    var formatTime = this.d3.timeFormat("%e %B %Y");

    // set the ranges
    var x = this.d3.scaleBand().range([0, this.width]);
    var y = this.d3.scaleLinear().range([this.height, 0]);

    // define the line
    var line = this.d3.line()
      .x(function (d) {
        return x(d.catagory) + 15;
      })
      .y((d) => {
        return y(d[this.gValue]);
      })
      .defined((d) => d[this.gValue] != 0);

    var line2 = this.d3.line()
      .x(function (d) {
        return x(d.catagory) + 15;
      })
      .y((d) => {
        return y(d[this.gTarget]);
      })
      .defined((d) => d[this.gTarget] != 0);

    var svg = this.d3.select('#' + id).append("svg")
      .attr("id", 'linechart' + id)
      .attr("width", this.width + this.margin.left + this.margin.right)
      .attr("height", this.height + this.margin.top + this.margin.bottom + 20)
      .append("g")
      .attr("transform", "translate(" + this.margin.left + "," + (this.margin.top) + ")");


    x.domain(this.data.map((d) => d.catagory))

    y.domain([this.GenerateYminDomain(this.data), 100]);

    // Add the X Axis
    svg.append("g")
      .attr("class", "x axis")
      .attr("transform", "translate(0," + (this.height) + ")")
      .call(this.d3.axisBottom(x));

    // Add the Y Axis
    svg.append("g")
      .call(this.d3.axisLeft(y).ticks(5).tickFormat(d => d))
      .append("text")
      .attr("transform", "rotate(-90)")
      .attr("y", 6)
      .attr("dy", ".71em")
      .style("text-anchor", "end");

    const serie = svg.append("g")
      .selectAll("g")
      .data(this.data)
      .join("g");


    //create range of grid line from yScale
    svg.select('.grid').remove();//remove then create new because each data had scale of it.
    var makeYLines = () => this.d3.axisLeft(y);
    serie.append('g')
      .attr('class', 'grid')
      .attr('transform', `translate(0, ${this.height - this.height})`)
      .call(makeYLines().ticks(5).tickSize(-this.width).tickFormat((d) => ""));

    let gValue = this.gValue
    svg.selectAll('circle')
      .data(this.data)
      .enter()
      .append('circle')
      .attr('fill', '#ffffff')
      .attr('stroke', '#aaa')
      .attr('cx', (d) => x(d.catagory) + 15)
      .attr('cy', (d) => y(d[this.gValue]))
      .attr('r', 5)
      .on("mouseover", handleMouseOver)
      .on("mouseout", handleMouseOut);
      

    // Add the line path.
    var lineV = serie.append("path")
      //.attr("class", "line")D
      .attr("fill", "none")
      .attr("stroke-width", 1.5)
      .attr("d", line(this.data))
      .attr("stroke", this.lineColor)
      //.attr("stroke-width","3px")    
      .attr("shape-rendering", "geometricPrecision")

    if(lineV.node()){
      var totalLength = lineV.node().getTotalLength();

      lineV.attr("stroke-dasharray", totalLength + " " + totalLength)
        .attr("stroke-dashoffset", totalLength)
        .transition(this.d3.linear)
        .duration(2000)
        .attr("stroke-dashoffset", 0);
    }

   
    //serie.append("g").selectAll("text")
    // .data(this.data)
    // .enter()
    // .append("text")
    //  .attr("x", function (d) { return x(d.catagory)})
    // .attr("y", (d) =>   y(d[this.gValue]) )
    // .attr("fill", "black")
    // .attr("font-size", 10)
    //  .text((d) => y(d[this.gValue]));


    // serie.selectAll('#limit').remove()

    let d3 : any = this.d3;
    svg.selectAll('dot')
      .data(this.data)
      .enter()
      .append('circle')
      .attr('fill', 'red')
      .attr('stroke', 'red')
      .attr('cx', (d) => x(d.catagory) + 15)
      .attr('cy', (d) => y(d[this.gTarget]))
      .attr('r', 1.5)

    var lineT = serie.append("path")
      //.attr("class", "line")D
      .attr("fill", "none")
      .attr("stroke-width", 1)
      .attr("d", line2(this.data))
      .attr("stroke", 'red')
      .style("stroke-dasharray", 0)
      //.attr("stroke-width","3px")    
      .attr("shape-rendering", "geometricPrecision")

    if(lineT.node()){
    var totalLength2 = lineT.node().getTotalLength();

    lineT.attr("stroke-dasharray", totalLength2 + " " + totalLength2)
      .attr("stroke-dashoffset", totalLength2)
      .transition(this.d3.linear)
      .duration(2000)
      .attr("stroke-dashoffset", 0);
    }



    svg.select('.linetext').remove()

    serie.append("text")
      .attr('class', 'linetext')
      .attr("transform", "translate(" + (this.width - 5) + "," + y(this.data[6][this.gTarget]) + ")")
      .attr("dy", ".35em")
      .attr("text-anchor", "start")
      .style("fill", "red")
      .style("font-size","8pt")
      .text(this.data[6][this.gTarget]);




    serie.append('text')
      .attr('class', 'textH')
      .attr('x', this.width / 2)
      .attr('y', -22)
      .attr('text-anchor', 'middle')
      .attr("font-size", "20")
      .style("font-weight", 'bold')
      .style("fill", "#595959")
      .style("font-family", "sans-serif")
      .text(this.title)

      function handleMouseOver(d, i) {  // Add interactivity
        console.log(d[gValue])
        // Specify where to put label of text
        svg.append("text")
        .attr('class', 'textHover')
        //.attr("id", 't' + d[gValue] + "-" + i)
        .attr('x', x(d.catagory) + 5)
        .attr('y',  y(d[gValue]) - 15)
        .attr("font-size", "10pt")
        .style("font-weight", 'bold')
        .style("fill", "#5B7BD6")
        .text(d[gValue])
      }

      function handleMouseOut(d, i) {   
        svg.select('.textHover').remove()
      }

  }


  GenerateYminDomain(data: any) {

    let d3: any = D3;

    let datafilter = data.filter(x => x[this.gValue] != 0)

    var yMinDomain = d3.min(datafilter, d => Number(d[this.gValue]))
    if (yMinDomain != 0) {
      yMinDomain = yMinDomain - ((yMinDomain * 5) / 100)
    }

    if(yMinDomain > 50) yMinDomain = 50;

    return yMinDomain
  }
}
