import { Component, ElementRef, Input, OnInit, OnChanges, SimpleChanges } from '@angular/core';
import * as D3 from 'd3';
import { formatDate } from '@angular/common';
import { ReactiveFormsModule } from '@angular/forms';

@Component({
    selector: 'line-ii-chart',
    templateUrl: './line-ii-chart.component.html', 
    styleUrls: ['./line-ii-chart.component.css']
})


export class Line_II_ChartComponent implements OnInit,OnChanges {

    private htmlElement: HTMLElement;
     @Input("ID") ID: string;
     @Input("data") data : any
    @Input("lineColor") lineColor: string
    @Input("title") title: string
    @Input("WidthSizeChart") WidthSizeChart: any;
    @Input("HeigthSizeChart") HeigthSizeChart: any;

    constructor(elementRef: ElementRef) {
        this.htmlElement = elementRef.nativeElement;  // reference to <line-chart> element from the main template
        
        console.log(D3);
        
    }

    private d3: any = D3;
    private margin = { top: 20, right: 80, bottom: 30, left: 50 }
    private width = 760 - this.margin.left - this.margin.right
    private height = 300 - this.margin.top - this.margin.bottom;

    ngOnInit(){
        //this.SetupChart(this.ID)
        if (this.WidthSizeChart) {
            this.width = this.WidthSizeChart - this.margin.left - this.margin.right
        }
        if (this.HeigthSizeChart) {
            this.height = this.HeigthSizeChart - this.margin.top - this.margin.bottom
        }
    }

    ngOnChanges(changes: SimpleChanges): void {
        
        if (!changes["data"].isFirstChange()) {
            //this.d3.select('#' + this.ID).remove();
            this.d3.select("#" + 'linechart' + this.ID).remove();
            this.CreateChart(this.ID);
          }
    }

    ngAfterViewInit() {
        this.CreateChart(this.ID)
      }


    CreateChart(id) {
        this.data = this.data.filter(x => x.value != null)
        //let data = this.data.filter(x => x.value != null)
        console.log('dd',this.data)
        //console.log('data', data)
        console.log(this.htmlElement);
        //this.margin = {
        //    top: 20,
        //    right: 80,
        //    bottom: 30,
        //    left: 50
        //},
        //    this.width = 760 - this.margin.left - this.margin.right,
        //    this.height = 300 - this.margin.top - this.margin.bottom;

        // parse the date / time
        var parseDate = this.d3.timeParse("%Y%m");
        var formatTime = this.d3.timeFormat("%e %B %Y");

        // set the ranges
        var x = this.d3.scaleBand().range([0, this.width]);
        var y = this.d3.scaleLinear().range([this.height, 0]);

        // define the line
        var line = this.d3.line()
            .x(function (d) {
                return x(d.date) + 28;
            })
            .y(function (d) {
                return y(d.value);
            })
            ;
        
        var svg = this.d3.select('#' + id).append("svg")
            .attr("id", 'linechart' + id)
            .attr("width", this.width + this.margin.left + this.margin.right)
            .attr("height", this.height + this.margin.top + this.margin.bottom + 50)
            .append("g")
            .attr("transform", "translate(" + this.margin.left + "," + (this.margin.top + 40) + ")");

        // format the data
        // this.data.forEach(function (d) {
        //     d.date = parseDate(d.date);

        // });

        // x.domain(this.d3.map(this.data, function (d) {
        //     return d.date;
        // }));

        x.domain(this.data.map((d) => d.date))

        y.domain([this.generateYminDomain(this.data), 100]);

        // Add the X Axis
        svg.append("g")
            .attr("class", "x axis")
            .attr("transform", "translate(0," + (this.height)+ ")")
            .call(this.d3.axisBottom(x));

        // Add the Y Axis
        svg.append("g")
            .call(this.d3.axisLeft(y).ticks(5).tickFormat(d => d + '%'))
            .append("text")
            .attr("transform", "rotate(-90)")
            .attr("y", 6)
            .attr("dy", ".71em")
            .style("text-anchor", "end");

        const serie = svg.append("g")
        .selectAll("g")
        .data(this.data)
        .join("g");


            //create range of grid line from yScale
        svg.select('.grid').remove();//remove then create new because each data had scale of it.
        var makeYLines = () => this.d3.axisLeft(y);
        serie.append('g')
        .attr('class', 'grid')
            .attr('transform', `translate(0, ${this.height - this.height})`)
            .call(makeYLines().ticks(5).tickSize(-this.width).tickFormat((d) => ""));

        // Add the line path.
        //serie.append("path")
        //    //.attr("class", "line")
        //    .attr("fill", "none")
        //    .attr("stroke-width", 1.5)
        //    .attr("d", line(this.data))
        //    .attr("stroke", this.lineColor);    
            
        serie.append("g")
            .attr("font-family", "sans-serif")
            .attr("font-size", 9)
            .attr("stroke-linecap", "round")
            .attr("stroke-linejoin", "round")
            .attr("text-anchor", "middle")
            .selectAll("text")
            .data(this.data)
            .join("text")
            .text(d => d.value + '%')
            .attr("dy", "0.35em")
            .attr("x", d => x(d.date)+28)
            .attr("y", d => y(d.value)-10)
            .clone(true).lower()
            .attr("fill", "none")
            .attr("stroke", "white")
            .attr("stroke-width", 6);

        // change line to look at searches
        // line.y(function (d) {
        //     return y(d.searches);
        // });

        // Add the second line path.
        svg.append("path")
            .attr("class", "line")
            .style("fill", "none")
            .attr("stroke-width", 3)
            .attr("d", line(this.data))  
            .style("stroke", this.lineColor);


        // Add the scatterplot

        var div = this.d3.select("body").append("div")
            .attr("class", "tooltip")
            .style("opacity", 0);

        // serie.append("g").selectAll("text")
        // .data(this.data)
        // .enter()
        // .append("text")
        // .attr("x", function(d) { return x(d.date) + 23})
        // .attr("y", function(d) { return y(d.value) })
        // .attr("fill", "black")
        // .attr("font-size", 10)
        // .text(function(d) { return d.value });

        serie.append('text')
            .attr('class', 'textH')
            .attr('x', this.width / 2)
            .attr('y', -22)
            .attr('text-anchor', 'middle')
            .attr("font-size", "20")
            .style("font-weight", 'bold')
            .style("fill", "#595959")
            .style("font-family", "sans-serif")
            .text(this.title)
   
    }


    generateYminDomain(data: any) {

        let d3: any = D3;
        
        var yMinDomain = d3.min(data, d => Number(d.value))
        if (yMinDomain != 0) {
            yMinDomain = yMinDomain - ((yMinDomain * 5) / 100)
        }
        return yMinDomain
    }
}
