import { Component, OnInit, OnChanges, ElementRef, Input, ViewChild, ViewEncapsulation, SimpleChanges } from '@angular/core';
import * as d3 from 'd3';
import { stack } from 'd3';
import { HttpService } from '../../../../core/http.service';

@Component({
    selector: 'multiple-bar-chart-management',
    templateUrl: './multiple-bar-chart-management.component.html',
    styleUrls: ['./multiple-bar-chart-management.component.css']
})
     


//*
//<multiple-bar-chart [ID]="'C1'" [dataLegend]="dataLegend" [serviceName]="'GetData'" [FY]="'19'" > </multiple-bar-chart>
//*

export class MultipleBarChartManagementComponent implements OnInit, OnChanges {
    constructor(
        private http: HttpService
    ) { }

    @Input() dataLegend: ChartData[];
    @Input() serviceName: string;
    @Input("ID") ID: string;
    @Input() OutletID: string;
    @Input() FY: string;
    @Input("chartname") chartname: string;
    //@Input() SP_NAME: string;

    @Input() yAxisMax: number;
    @Input() xAxisMax: number;

    @ViewChild('chart', { static: true }) chartContainer: ElementRef;

    private margin = { top: 30, right: 20, bottom: 40, left: 40 }
    private width = 580 - this.margin.left - this.margin.right
    private height = 350 - this.margin.top - this.margin.bottom;

    private nodeWidth: any;

    ngOnInit() {
        //if (!this.inputData) { return; }
        //this.createChart()
    }

    ngOnChanges(changes: SimpleChanges): void {
        //if (!this.data || this.data.length == 0) { return; }
        //this.createChart(this.data)
        if (!changes["OutletID"].isFirstChange()) {
            d3.select("#" + 'multichart' + this.ID).remove();
            this.GetData(this.ID)
        }
    }


    ngAfterViewInit() {

        //this.createChart()
        this.GetData(this.ID)
    }


    GetData(id) {
        if (this.OutletID != '') {
            this.http.get("System", this.serviceName, this.FY, this.OutletID)
                .subscribe(result => {
                    this.CreateChart(result.Table, id);
                })
        }
        else {
            this.http.get('System', this.serviceName, this.FY)
                .subscribe(result => {
                    this.CreateChart(result.Table, id);
                })
        }
    }

    private CreateChart(resultData, id): void {
        //this.nodeWidth = (d) => d.getBBox().width;

        //const data = this.data2[0].Values;
        const legendList = this.dataLegend[0].Legend
        console.log('legendList', legendList)
        const data: any[] = resultData;

        console.log('xxxxxx :', data);

        let datamax: any;
        let datamax2: any;

        datamax = data.map(a => Math.max(a.Value));
        datamax2 = data.map(a => Math.max(a.Value2));
        console.log('loadmax1', datamax)
        console.log('loadmax2', datamax2)

        var l = datamax.length;
        var max1 = -Infinity;
        var i;
        for (i = 0; l > i; i++) {

            if (datamax[i] > max1) {

                max1 = datamax[i];

            }

        }

        console.info("maxmaxmax1", max1);

        var l = datamax2.length;
        var max2 = -Infinity;
        var i;
        for (i = 0; l > i; i++) {

            if (datamax2[i] > max2) {

                max2 = datamax2[i];

            }

        }
        console.info("maxmaxmax2", max2);

        var max3;
        if (max1 > max2) {
            max3 = max1
        }
        else {
            max3 = max2
        }



        let h = this.height;
        let w = this.width;

        //var formatValue = d3.format(".0%");

        const svg = d3.select('#' + id).append('svg')
            .attr("id", 'multichart' + id)
            .attr('width', this.width + this.margin.left + this.margin.right)
            .attr('height', this.height + this.margin.top + this.margin.bottom)
            .style("background", "rgb(255, 255, 255");



        const yScale = d3.scaleLinear()
            .range([this.height, 0])
            .domain([0, max3 + 10]);


        const xScale = d3.scaleBand()
            .range([0, this.width])
            .domain(data.map((d) => d.catagory))
            .padding(0.2)

        const xScale1 = d3.scaleBand()
            .domain(['Value', 'Value2'])
            .range([0, xScale.bandwidth() + 5])

        const chart = svg.append('g')
            .attr('transform', `translate(${this.margin.left}, ${this.margin.top})`);

        chart.append('g')
            .call(d3.axisLeft(yScale).ticks(5))
        //.tickFormat((d) => d + "%"));

        chart.append('g')
            .attr('transform', `translate(0, ${this.height})`)
            .call(d3.axisBottom(xScale))
            .selectAll("text")
            .style("text-anchor", "end")
            .attr("dx", "1em")
            .attr("dy", ".75em")
        //.attr("transform", (d) => "rotate(-65)");


        const barGroups = chart.selectAll()
            .data(data)
            .enter()
            .append('g')
            .attr("transform", d => `translate(${xScale(d.catagory)},0)`);

        barGroups.selectAll(".bar.Value")
            .data(d => [d])
            .enter()
            .append("rect")
            //.attr("class", "bar field1")
            .style("fill", "#376092")
            .attr("x", d => xScale1('Value'))
            .attr("y", d => yScale(d.Value))
            .attr("width", xScale1.bandwidth())
            .attr('height', (s) => this.height - yScale(s.Value))



        barGroups.selectAll(".bar.Value2")
            .data(d => [d])
            .enter()
            .append("rect")
            //.attr("class", "bar field1")
            .style("fill", "#93cddd")
            .attr("x", d => xScale1('Value2'))
            .attr("y", d => yScale(d.Value2))
            .attr("width", xScale1.bandwidth())
            .attr('height', (s) => this.height - yScale(s.Value2))
            .append('text')

        barGroups.append('text')
            //.attr('class', 'Value')
            .attr('x', (d) => xScale.bandwidth() / 3.5)
            .attr('y', (d) => yScale(d.Value) - 20)
            .attr('text-anchor', 'middle')
            .attr("fill", "Red")
            .attr("font-size", "12")
            .text((d) => `${d.Value}`)
            .text((d) => d.Value > 0 ? d.Value : '')

        barGroups.append('text')
            //.attr('class', 'Value2')
            .attr('x', (d) => xScale1.bandwidth() * 1.5)
            .attr('y', (d) => yScale(d.Value2) - 20)
            .attr('text-anchor', 'middle')
            .attr("fill", "Blue")
            .attr("font-size", "12")
            .text((d) => `${d.Value2}`)
            .text((d) => d.Value2 > 0 ? d.Value2 : '')


        svg.append('text')
            .attr('x', this.width / 2)
            .attr('y', 14)
            .style("font-size", "20px")
            .style("font-weight", 'bold')
            .style('text-anchor', 'middle')
            .text(this.chartname)


        let nodeWidth = (d) => d.getBBox().width;
        //const cScale = d3.scaleOrdinal(d3.sche);

        const legend = svg.append('g')
            .attr('class', 'legend')
            .attr('transform', 'translate(0,0)');

        const lg = legend.selectAll('g')
            .data(legendList)
            .enter()
            .append('g')
        //.attr('transform', (d, i) => `translate(${i * 100},${height + 15})`);

        lg.append('rect')
            .style('fill', d => d.color)
            .attr('x', 20)
            .attr('y', -258)
            .attr('width', 15)
            .attr('height', 15);

        lg.append('text')
            .style('font-family', 'Georgia')
            .style('font-size', '15px')
            .attr('x', 40)
            .attr('y', -245)
            .text(d => d.legend);

        let offset = 0;
        lg.attr('transform', function (d, i) {
            let x = offset;
            offset += nodeWidth(this) + 10;
            return `translate(${x},${h + 10})`;
        });

        legend.attr('transform', function (d, i) {
            return `translate(${(w - nodeWidth(this)) / 2},${0})`
        });

    }


}


export interface ChartData {
    Values: Array<valueModel>;
    Legend: Array<legendModel>
}


export interface valueModel {
    catagory: string;
    Value: number;
    Value2: number;
    Value3: number;


}

export interface legendModel {
    legend: string;
    color: string;

}
