import { Component, OnInit, OnChanges, ElementRef, Input, ViewChild, ViewEncapsulation, SimpleChanges } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import * as d3 from 'd3';
import { keys } from 'd3';
//import { stack } from 'd3';

@Component({
  selector: 'payment',
  templateUrl: './payment.component.html',
  styleUrls: ['./payment.component.css']
})
export class PaymentComponent implements OnInit, OnChanges {

  constructor(
    private http: HttpService
  ) { }

  @Input() serviceName: string;
  @Input() OutletID: string;
  @Input() FY: string;
  @Input() QTR: string;
  @Input() ID: string;
  @Input() Showtable: boolean = true;

  @ViewChild('chart', { static: true }) chartContainer: ElementRef;
  //private chartContainer: ElementRef;

  private dataSSI: any = {};
  private dataCSI: any = {};
  private dataAll: any = {};

  private svg: any;
  private svg2: any;
  private svg3: any;
  private yScale: any;
  private yScale2: any;
  private yScale3: any;
  private xScale: any;
  private xAxis: any;
  private yAxis: any;
  private yAxis2: any;
  private yAxis3: any;
  private chart1: any;
  private chart2: any;
  private chart3: any;
  private barGroups: any;
  private barGroups2: any;
  private barGroups3: any;
  private data: any
  public dataListTabel: any;
  private YearList: any;
  // public FY: any;
  public cCurrency: string;

  private WIDTH: number;
  private HEIGHT: number;
  public dataLegend: [{

    Legend: [{ legend: 'SSI', color: '#376092' },
      { legend: 'CSI', color: '#93cddd' }]
  }]

  private labelxScalemonth12 = ["Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", "Jan", "Feb", "Mar"];
  public HeaderList = ["VDM Bonus Amount", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", "Jan", "Feb", "Mar"];
  public FieldList = [{ Title: "SSI payment", Field: "SSI" }, { Title: "CSI payment", Field: "CSI" }, { Title: "SSI & CSI payment", Field: "ALL_PAYMENT" }]

  ngOnInit() {
    this.CreateDropdownList()

    if (this.OutletID != '') {
      this.cCurrency = 'kTHB';
    } else {
      this.cCurrency = 'mTHB';
    }

    this.WIDTH = 420;
    this.HEIGHT = 280;
    this.dataLegend = [{

      Legend: [{ legend: 'SSI', color: '#376092' },
      { legend: 'CSI', color: '#93cddd' }]
    }]

  }

  ngOnChanges(changes: SimpleChanges): void {

    if (this.OutletID != '') {
      if (!changes["OutletID"].isFirstChange()) {


        d3.select("#" + 'pay1_ssi').remove();
        d3.select("#" + 'pay1_csi').remove();
        d3.select("#" + 'pay1_all').remove();
        this.SetUpchart();
      }
    } else {
      if (!changes["FY"].isFirstChange()) {
        d3.select("#" + 'pay1_ssi').remove();
        d3.select("#" + 'pay1_csi').remove();
        d3.select("#" + 'pay1_all').remove();
        this.SetUpchart()
      }
    }

  }

  ngAfterViewInit() {
    this.SetUpchart()
  }

  AfterContentInit() {

  }

  GetData() {

    //  'Get_DSB_Payment_SSI_CSI_FY'
    let dataSSI = [], dataCSI = [], dataAll

    if (this.OutletID != '') {
      this.http.get("System", this.serviceName, this.FY, this.OutletID)
        .subscribe(result => {

          this.dataSSI = result.Table;
          this.dataCSI = result.Table1;
          this.dataAll = result.Table2;
          this.dataListTabel = result.Table3
          this.CreateChart1(this.dataSSI);
          this.CreateChart2(this.dataCSI);
          this.CreateChart3(this.dataAll);

        })
    }
    if (this.QTR != '') {
      this.http.get("System", this.serviceName, this.FY, this.QTR)
        .subscribe(result => {
          this.dataSSI = result.Table;
          this.dataCSI = result.Table1;
          this.dataAll = result.Table2;
          this.dataListTabel = result.Table3
          this.CreateChart1(this.dataSSI);
          this.CreateChart2(this.dataCSI);
          this.CreateChart3(this.dataAll);
        })
    }
    else {
      this.http.get("System", this.serviceName, this.FY)
        .subscribe(result => {
          this.dataSSI = result.Table;
          this.dataCSI = result.Table1;
          this.dataAll = result.Table2;
          this.dataListTabel = result.Table3
          this.CreateChart1(this.dataSSI)
          this.CreateChart2(this.dataCSI)
          this.CreateChart3(this.dataAll)

        })
    }
    
  }

  CreateDropdownList() {

    this.YearList = [];
    var min = new Date().getFullYear() - 1
    var max = min + 2

    for (let i = min; i <= max; i++) {
      this.YearList.push({ YearName: i, Value: i.toString().slice(2, 4) })
    }
  }

  SetUpchart() {
    
    var margin = { top: 30, right: 20, bottom: 40, left: 40 },
      width = this.WIDTH  - margin.left - margin.right,
      height = this.HEIGHT - margin.top - margin.bottom;
    

    this.svg = d3.select('#ssi').append('svg')
      .attr("id", 'pay1_ssi')
      .attr('width', width + margin.left + margin.right)
      .attr('height', height + margin.top + margin.bottom)
      .style("background", "rgb(255, 255, 255");

    this.svg2 = d3.select('#csi').append('svg')
      .attr("id", 'pay1_csi')
      .attr('width', width + margin.left + margin.right)
      .attr('height', height + margin.top + margin.bottom)
      .style("background", "rgb(255, 255, 255");

    this.svg3 = d3.select('#all').append('svg')
      .attr("id", 'pay1_all')
      .attr('width', width + margin.left + margin.right)
      .attr('height', height + margin.top + margin.bottom)
      .style("background", "rgb(255, 255, 255");

    this.yScale = d3.scaleLinear()
      .range([height, 0])

    this.yScale2 = d3.scaleLinear()
      .range([height, 0])

    this.yScale3 = d3.scaleLinear()
      .range([height, 0])

    this.xScale = d3.scaleBand()
      .range([0, width])
      .paddingInner(0.35)
      .paddingOuter(0.1)

    this.xAxis = d3.axisBottom(this.xScale);

    this.yAxis = d3.axisLeft(this.yScale).ticks(5)
    this.yAxis2 = d3.axisLeft(this.yScale2).ticks(5)
    this.yAxis3 = d3.axisLeft(this.yScale3).ticks(5)
    
    this.chart1 = this.svg.append('g')
      .attr('transform', `translate(${margin.left}, ${margin.top})`);
    
    this.chart1.append('g')
      .attr("class", "y axis")
      .call(this.yAxis);

    this.chart1.append('g')
      .attr("class", "xAxis")
      .attr('transform', `translate(0, ${height})`)
      .call(this.xAxis)

    this.chart2 = this.svg2.append('g')
      .attr('transform', `translate(${margin.left}, ${margin.top})`);

    this.chart2.append('g')
      .attr("class", "y axis2")
      .call(this.yAxis2);

    this.chart2.append('g')
      .attr("class", "xAxis")
      .attr('transform', `translate(0, ${height})`)
      .call(this.xAxis)
      .selectAll("text")
      .style("text-anchor", "end")

    this.chart3 = this.svg3.append('g')
      .attr('transform', `translate(${margin.left}, ${margin.top})`);

    this.chart3.append('g')
      .attr("class", "y axis3")
      .call(this.yAxis3);

    this.chart3.append('g')
      .attr("class", "xAxis")
      .attr('transform', `translate(0, ${height})`)
      .call(this.xAxis)
      .selectAll("text")
      .style("text-anchor", "end")

    this.GetData()

  }


  CreateChart1(gData) {

    //  console.log('gdata', gData)
    var margin = { top: 30, right: 20, bottom: 40, left: 40 },
      width = this.WIDTH - margin.left - margin.right,
      height = this.HEIGHT - margin.top - margin.bottom;
    let data: valueModel[];

    data = gData
    //let NoZero = data.filter(d => d.Value != 0)
    var yMinDomain = this.generateYminDomain(this.dataAll)
    var yMaxDomain = this.generateYmaxDomain(this.dataAll)

    this.yScale.domain([yMinDomain, yMaxDomain + 2])
    this.xScale.domain(this.labelxScalemonth12.map(d => d))

    this.chart1
      .select(".y")
      .call(this.yAxis)

    this.chart1
      .select(".xAxis")
      .attr('transform', `translate(0, ${height})`)
      .call(this.xAxis)
      .selectAll("text")
    
    //create range of grid line from yScale
    this.svg.select('.grid').remove();//remove then create new because each data had scale of it.
    var makeYLines = () => d3.axisLeft(this.yScale);
    this.chart1.append('g')
      .attr('class', 'grid')
      .call(makeYLines().ticks(5).tickSize(-width).tickFormat((d) => ""));

    let xScaleOfBar = this.xScale;
    let yScaleOfBar = this.yScale;

    let chart = this.chart1;

    this.barGroups = this.chart1.selectAll(".bar1")
      .data(data)

    this.barGroups
      .enter()
      .append("rect")
      .merge(this.barGroups)
      .attr("class", "bar1")
      //.transition().duration(100)
      .attr('x', (d) => this.xScale(d.catagory))
      .attr('y', (d) => this.yScale(d.Value > 0 ? d.Value : 0))
      .attr('height', (s) => height - this.yScale(s.Value))
      .attr('width', this.xScale.bandwidth())
      .attr("fill", function (d) {
        return "#80cbc4";
      }).on('mouseenter', function (actual, i) {

        d3.selectAll('.value')
          .attr('opacity', 0)

        d3.select(this)
          .transition()
          .duration(300)
          .attr('opacity', 0.6)
          .attr('x', (d) => xScaleOfBar(actual.catagory) - 5)
          .attr('width', xScaleOfBar.bandwidth() + 10)

        var y = yScaleOfBar(actual.Value);

        var line = chart.append('rect')
          .transition()
          .duration(500)
          .attr('id', 'showzone')
          .attr('x', 0)
          .attr('y', 0)
          .attr('width', width)
          .attr('height', y)
          .attr("fill", "#2F4A6D")
          .attr('opacity', 0.1)
          ;

      })
      .on('mouseleave', function (actual) {
        d3.selectAll('.value')
          .attr('opacity', 1)

        d3.select(this)
          .transition()
          .duration(300)
          .attr('opacity', 1)
          .attr('x', (a) => xScaleOfBar(actual.catagory))
          .attr('width', xScaleOfBar.bandwidth())

        chart.selectAll('#showzone').remove()

      });
       
    this.barGroups = this.chart1.selectAll(".textvalue")
      .remove()
      .exit()
      .data(data)

    this.barGroups
      .enter()
      .append('text')
      .attr('class', 'textvalue')
      .attr('x', (d) => this.xScale(d.catagory) + this.xScale.bandwidth() / 2)
      .attr('y', (d) => this.yScale(d.Value) - 15)
      .attr('text-anchor', 'middle')
      .style("font-size", "12")
      .attr("fill", "#000099")
      .text((d) => d.Value > 0 ? d.Value : '')


    this.svg.select('.textH').remove()

    this.chart1.append('text')
      .attr('class', 'textH')
      .attr('x', width / 2)
      .attr('y', -8)
      .style("font-size", "16")
      .attr('text-anchor', 'middle')
      .style("font-weight", 'bold')
      .style("fill", "black")
      .text('SSI Payment(' + this.cCurrency + ')')
       
  }

  CreateChart2(gData) {

    //console.log('gdata', gData)

    var margin = { top: 30, right: 20, bottom: 40, left: 40 },
      width = this.WIDTH - margin.left - margin.right,
      height = this.HEIGHT - margin.top - margin.bottom;

    let data: valueModel[];
    data = gData
    //let NoZero = data.filter(d => d.Value != 0)

    var yMinDomain = this.generateYminDomain(this.dataAll)
    var yMaxDomain = this.generateYmaxDomain(this.dataAll)


    this.yScale2.domain([yMinDomain, yMaxDomain + 2])
    this.xScale.domain(this.labelxScalemonth12.map(d => d))
         
    this.chart2
      .select(".y")
      .call(this.yAxis2);

    this.chart2
      .select(".xAxis")
      .attr('transform', `translate(0, ${height})`)
      .call(this.xAxis)
      .selectAll("text")

    //create range of grid line from yScale
    this.svg2.select('.grid').remove();//remove then create new because each data had scale of it.
    var makeYLines = () => d3.axisLeft(this.yScale2);
    this.chart2.append('g')
      .attr('class', 'grid')
      .call(makeYLines().ticks(5).tickSize(-width).tickFormat((d) => ""));

    let xScaleOfBar = this.xScale;
    let yScaleOfBar = this.yScale2;

    let chart = this.chart2;
    this.barGroups2 = this.chart2.selectAll(".bar2")

      .data(data)

    this.barGroups2
      .enter()
      .append("rect")
      .merge(this.barGroups2)
      .attr("class", "bar2")
      //.transition().duration(100)
      .attr('x', (d) => this.xScale(d.catagory))
      .attr('y', (d) => this.yScale2(d.Value > 0 ? d.Value : 0))
      .attr('height', (s) => height - this.yScale2(s.Value))
      .attr('width', this.xScale.bandwidth())
      .attr("fill", function (d) {
        return "#93cddd";
      }).on('mouseenter', function (actual, i) {


        d3.selectAll('.value')
          .attr('opacity', 0)

        d3.select(this)
          .transition()
          .duration(300)
          .attr('opacity', 0.6)
          .attr('x', (d) => xScaleOfBar(actual.catagory) - 5)
          .attr('width', xScaleOfBar.bandwidth() + 10)

        var y = yScaleOfBar(actual.Value);

        var line = chart.append('rect')
          .transition()
          .duration(500)
          .attr('id', 'showzone')
          .attr('x', 0)
          .attr('y', 0)
          .attr('width', width)
          .attr('height', y)
          .attr("fill", "blue")
          .attr('opacity', 0.1)
          ;

      })
      .on('mouseleave', function (actual) {
        d3.selectAll('.value')
          .attr('opacity', 1)

        d3.select(this)
          .transition()
          .duration(300)
          .attr('opacity', 1)
          .attr('x', (a) => xScaleOfBar(actual.catagory))
          .attr('width', xScaleOfBar.bandwidth())

        chart.selectAll('#showzone').remove()

      });



    this.barGroups2 = this.chart2.selectAll(".textvalue2")
      .remove()
      .exit()
      .data(data)

    this.barGroups2
      .enter()
      .append('text')
      .attr('class', 'textvalue2')
      .attr('x', (d) => this.xScale(d.catagory) + this.xScale.bandwidth() / 2)
      .attr('y', (d) => this.yScale2(d.Value) - 15)
      .attr('text-anchor', 'middle')
      .style("font-size", "12")
      .attr("fill", "#000099")
      .text((d) => d.Value > 0 ? d.Value : '')


    this.svg2.select('.textH').remove()

    this.chart2.append('text')
      .attr('class', 'textH')
      .attr('x', width / 2)
      .attr('y', -8)
      .style("font-size", "16")
      .attr('text-anchor', 'middle')
      .style("font-weight", 'bold')
      .style("fill", "black")
      .text('CSI Payment(' + this.cCurrency + ')')


  }
  
  CreateChart3(gData) {

    //  console.log('gdata', gData)
    var margin = { top: 30, right: 20, bottom: 40, left: 40 },
      width = this.WIDTH - margin.left - margin.right,
      height = this.HEIGHT - margin.top - margin.bottom;

    let data: valueModel[];

    data = gData
    //let NoZero = data.filter(d => d.Value != 0)
    var yMinDomain = this.generateYminDomain(this.dataAll)
    var yMaxDomain = this.generateYmaxDomain(this.dataAll)

    this.yScale3.domain([yMinDomain, yMaxDomain + 2])
    this.xScale.domain(this.labelxScalemonth12.map(d => d))

    this.chart3
      .select(".y")
      .call(this.yAxis3)

    this.chart3
      .select(".xAxis")
      .attr('transform', `translate(0, ${height})`)
      .call(this.xAxis)
      .selectAll("text")


    //create range of grid line from yScale
    this.svg3.select('.grid').remove();//remove then create new because each data had scale of it.
    var makeYLines = () => d3.axisLeft(this.yScale3);
    this.chart3.append('g')
      .attr('class', 'grid')
      .call(makeYLines().ticks(5).tickSize(-width).tickFormat((d) => ""));

    let xScaleOfBar = this.xScale;
    let yScaleOfBar = this.yScale3;

    let chart = this.chart3;

    this.barGroups3 = this.chart3.selectAll(".bar3")
      .data(data)

    this.barGroups3
      .enter()
      .append("rect")
      .merge(this.barGroups3)
      .attr("class", "bar3")
      //.transition().duration(100)
      .attr('x', (d) => this.xScale(d.catagory))
      .attr('y', (d) => this.yScale3(d.Value > 0 ? d.Value : 0))
      .attr('height', (s) => height - this.yScale3(s.Value))
      .attr('width', this.xScale.bandwidth())
      .attr("fill", function (d) {
        return "#538ed5";
      }).on('mouseenter', function (actual, i) {
        d3.selectAll('.value')
          .attr('opacity', 0)

        d3.select(this)
          .transition()
          .duration(300)
          .attr('opacity', 0.6)
          .attr('x', (d) => xScaleOfBar(actual.catagory) - 5)
          .attr('width', xScaleOfBar.bandwidth() + 10)

        var y = yScaleOfBar(actual.Value);

        var line = chart.append('rect')
          .transition()
          .duration(500)
          .attr('id', 'showzone')
          .attr('x', 0)
          .attr('y', 0)
          .attr('width', width)
          .attr('height', y)
          .attr("fill", "blue")
          .attr('opacity', 0.1)
          ;

      })
      .on('mouseleave', function (actual) {
        d3.selectAll('.value')
          .attr('opacity', 1)

        d3.select(this)
          .transition()
          .duration(300)
          .attr('opacity', 1)
          .attr('x', (a) => xScaleOfBar(actual.catagory))
          .attr('width', xScaleOfBar.bandwidth())

        chart.selectAll('#showzone').remove()

      });



    this.barGroups3 = this.chart3.selectAll(".textvalue")
      .remove()
      .exit()
      .data(data)

    this.barGroups3
      .enter()
      .append('text')
      .attr('class', 'textvalue')
      .attr('x', (d) => this.xScale(d.catagory) + this.xScale.bandwidth() / 2)
      .attr('y', (d) => this.yScale3(d.Value) - 15)
      .attr('text-anchor', 'middle')
      .style("font-size", "12")
      .attr("fill", "#000099")
      .text((d) => d.Value > 0 ? d.Value : '')


    this.svg3.select('.textH').remove()

    this.chart3.append('text')
      .attr('class', 'textH')
      .attr('x', width / 2)
      .attr('y', -8)
      .style("font-size", "16")
      .attr('text-anchor', 'middle')
      .style("font-weight", 'bold')
      .style("fill", "black")
      .text('SSI & CSI Payment(' + this.cCurrency + ')')



  }

  //generateHearderList(data) {
  //    let newData = [];
  //    for (var key in data[0]) {
  //        if (key != 'FY' && key != 'catagory') {
  //            newData.push(key)
  //        }

  //    }
  //    return newData
  //}

  generateYmaxDomain(data: valueModel[]) {
    var yMaxDomain: number;
    yMaxDomain = d3.max(data, d => Number(d.Value))
    yMaxDomain = yMaxDomain + ((yMaxDomain * 10) / 100);

    return yMaxDomain
  }

  generateYminDomain(data: valueModel[]) {
    var yMinDomain: number = 0;
    //var yMinDomain = d3.min(data, d => Number(d.Value))
    //yMinDomain = yMinDomain - ((yMinDomain * 5) / 100)
    return yMinDomain
  }

  FYChange() {
    this.GetData()
  }
}


export interface ChartData {
  Values: Array<valueModel>;
  Legend: Array<legendModel>
}


export interface valueModel {
  catagory: string;
  Value: number;
  //Value2: number;
  //Value3: number;
}

export interface legendModel {
  legend: string;
  color: string;

}



