import { Component, OnInit, OnChanges, ElementRef, Input, ViewChild, ViewEncapsulation, SimpleChanges } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import * as D3 from 'd3';
//import { stack } from 'd3';

@Component({
    selector: 'simple-bar-chart-iiii',
    templateUrl: './simple-bar-chart-iiii.component.html',
    styleUrls: ['./simple-bar-chart-iiii.component.css']
})
export class SimpleBarChartIIIIComponent implements OnInit, OnChanges {

  constructor(
    private http: HttpService
  ) { }

  @Input() header: string = 'Demo';
  @Input("ID") ID: string;
  @Input() showLine: boolean = true;
  @Input() showLegen: boolean = true;
  @Input() Width : any;
  @Input() Height : any;
  @Input() Values: any = []
  @Input() color: any = [];
  @Input() Keys: any = ['HIGHER_EXPECTED', 'EXPECTED'];
  
  @Input() LegenList: any  = [{ legend: "aaaaaaaaaaaaaaa", color: '#248623' }, { legend: "bbbbbbbbbbbbbbbb", color: '#6395ec'}]
  @ViewChild('chart', { static: true }) chartContainer: ElementRef;

  private svg: any;
  private yScale: any;
  private yScale1: any;
  private xScale: any;
  private xAxis: any;
  private yAxis: any;
  private yAxis1: any;
  private chart: any;
  private barGroups: any;
  private margin = { top: 30, right: 20, bottom: 20, left: 35 }
  private width = 600 - this.margin.left - this.margin.right
  private height = 350 - this.margin.top - this.margin.bottom;
  private Data: any = [];
  private Delay: number = 0;
  private check: number = 0;

    private d3 : any = D3;

  ngOnInit() {
    //this.setUpChart(this.ID)
    // this.showLine = true;
    // this.showLegen = true
   
    if (this.Width) {
      this.width = this.Width - this.margin.left - this.margin.right
    }
  
    if (this.Height) {
      this.height = this.Height - this.margin.top - this.margin.bottom
    }

    if (this.Values[0] === undefined) {
    this.Values[0] = 0
    }
    if (this.Values[1] === undefined) {
    this.Values[1] = 0
    }
  }
 

  ngOnChanges(changes: SimpleChanges): void {
    this.Values[0] = this.Values[0] || 0
    this.Values[1] = this.Values[1] || 0

    this.Data = [{ 'catagory': this.Keys[0], 'Value': this.Values[0] , 'Color': this.color[0] },
      { 'catagory': this.Keys[1], 'Value': this.Values[1], 'Color': this.color[1] }]

    if (!changes["Values"].isFirstChange()) { 
      this.check++
      if (this.check > 1) {
        this.Delay = 3000;
      }
      this.createChart(this.Data)
    }

  }

  ngAfterViewInit() {
    this.setUpChart(this.ID)
  }



  setUpChart(id) {
    this.svg = this.d3.select("#" + id).append('svg')
        .attr("id", 'barchart' + id)
    .attr('width', this.width + this.margin.left + this.margin.right)
    .attr('height', this.height + this.margin.top + this.margin.bottom + 20)
    .style("background", "rgb(255, 255, 255");


    this.xScale = this.d3.scaleBand()
      .range([this.margin.left, this.width - this.margin.right])
      .padding(0.3)

    this.yScale = this.d3.scaleLinear()
      .rangeRound([this.height - this.margin.bottom, this.margin.top])

     

    this.xAxis = this.d3.axisBottom(this.xScale);

    this.yAxis = this.d3.axisLeft(this.yScale);

  
    this.chart = this.svg.append('g')
      .attr('transform', `translate(${this.margin.left / 2}, ${this.margin.top})`);

    this.chart.append('g')
      .attr("class", "x-axis")
      .attr("transform", `translate(0,${this.height - this.margin.bottom})`)
      .call(this.xAxis)


    this.chart.append('g')
      .attr("class", "y-axis")
      .attr("transform", `translate(${this.margin.left},0)`)
          .call(this.yAxis)
         

      this.createChart(this.Data)

  }

  createChart(data) {

    var yMinDomain = this.generateYminDomain(data)
    var yMaxDomain = this.generateYmaxDomain(data)
    this.yScale.domain([yMinDomain, yMaxDomain])
    this.xScale.domain(data.map((d) => d.catagory))
    //this.yScale1.domain([0,110])

    this.chart.selectAll(".y-axis").transition().duration(5)
    .call(this.d3.axisLeft(this.yScale).ticks(5))

    //this.chart.selectAll(".x-axis").transition().duration(5)
    //.call(this.d3.axisBottom(this.xScale).tickSizeOuter(0))
    



     this.chart
         .select(".x-axis")
       .call(this.xAxis)
       .selectAll("text")
       .style("text-anchor", "end")
       .attr("transform", (d) => "rotate(-20)");


    this.barGroups = this.chart.selectAll(".bar")
      //.remove()
      //.exit()
      .data(data)

   var xx = this.barGroups
      .enter()
      .append("rect")
      .merge(this.barGroups)
      .attr("class", "bar")
      //.transition()
      //.duration(this.Delay)
      .attr('x', (d) => this.xScale(d.catagory))
      .attr('y', (d) => this.yScale(d.Value))
      .attr('height', (s) => (this.height - 20) - this.yScale(s.Value))
      .attr('width', this.xScale.bandwidth())
      .attr("fill", (d) => d.Color)

    xx.transition()
      .duration(this.Delay)

    //xx.on("mouseover", (d) => this.handleMouseOver(d, 'Value'))
    //  .on("mouseout", (d) => this.handleMouseOut(d));


    this.barGroups = this.chart.selectAll(".textvalue")
      .remove()
      .exit()
      .data(data)

    this.barGroups
      .enter()
      .append('text')
      .attr('class', 'textvalue')
      .attr('x', (d) => this.xScale(d.catagory) + this.xScale.bandwidth() / 2)
      .attr('y', (d) => this.yScale(d.Value) - 5)
      .attr('text-anchor', 'middle')
      .attr("font-size", "8pt")
      .attr("fill", "#000000")
      .text((d) => d.Value > 0 ? d.Value : '')
      //.text((d) => d.Value)
     



    if(this.showLegen){
        this.CreateLegen(data)
    }

    this.svg.select('.textH').remove()
    this.chart.append('text')
      .attr('class', 'textH')
      .attr('x', this.width / 2)
      .attr('y', 0)
      .attr("font-size", "20px")
      .attr('text-anchor', 'middle')
      .style("font-weight", 'bold')
      .style('fill', '#969696')
      .text(this.header)


    //let xScale = this.xScale
    //let y = this.yScale
    //let chart = this.chart

  }


  //handleMouseOver = (d, Value) => {
  //  this.chart.append("text")
  //    .attr('class', 'textHover')
  //    .attr('x', (this.xScale(d.catagory) + this.xScale.bandwidth() / 2) - 10)
  //    .attr('y', this.yScale(d.Value) - 5)
  //    .attr("font-size", "7pt")
  //    .style("font-weight", 'bold')
  //    .style("fill", "#000000")
  //    .text(d[Value])
  //}

  //handleMouseOut = (d) => {
  //  this.chart.select('.textHover').remove()
  //}

  CreateLegen(data){
    let legendList = this.LegenList
    let height = this.height;
    let width = this.width;

    var nodeWidth = (d) => d.getBBox().width;
    //const cScale = this.d3.scaleOrdinal();

    const legend = this.svg.append('g')
      .attr('class', 'legend')
      .attr('transform', 'translate(0,0)');

    const lg = legend.selectAll('g')
      .data(legendList)
      .enter()
      .append('g')
      .attr('transform', (d, i) => `translate(${i * 100},${height + 15})`);

    lg.append('rect')
      .style('fill', d => d.color)
      .attr('x', 35)
      .attr('y', (this.height + this.margin.top + this.margin.bottom + this.margin.left))
      .attr('width', 50)
      .attr('height', 10);

    lg.append('text')
      .style('font-family', 'Georgia')
      .style('font-size', '8pt')
      .attr('x', 95)
      .attr('y', (this.height + this.margin.top + this.margin.bottom + this.margin.left + 10))
      .text(d => d.legend);

    let offset = 0;
    lg.attr('transform', function (d, i) {
      let x = offset;
      offset += nodeWidth(this) + 10;
      return `translate(${x},${-40})`;
    });

    legend.attr('transform', function () {
      return `translate(${(width - nodeWidth(this)) / 2},${0})`
    });
  }

  generateYmaxDomain(data: any) {

      var yMaxDomain: number = 0;
      //yMaxDomain = 100
    yMaxDomain = this.d3.max(data, d => Number(d.Value))

    if (yMaxDomain <= 0) {
      yMaxDomain = 0
    } else {
      yMaxDomain = yMaxDomain + ((yMaxDomain * 5) / 100);

    }

      


    return yMaxDomain
  }

  generateYminDomain(data: any) {
    var yMinDomain: number = 0;
    yMinDomain = this.d3.min(data, d => Number(d.Value))

    if (yMinDomain <= 0) {
      yMinDomain = 0
    } else {
      yMinDomain = yMinDomain - ((yMinDomain * 5) / 100)

    }
   
   
   
    return yMinDomain
  }



}





