import { Component, OnInit, OnChanges, Input, SimpleChanges } from "@angular/core";
import * as D3 from 'd3';


@Component({
    selector:'speed-meter',
    templateUrl: './speed-meter.component.html',
    styleUrls: ['./speed-meter.component.css']
})

export class SpeedMeterComponent implements OnInit,OnChanges{

    constructor(){

    }

    @Input() Value : number = 188;

    private d3 : any = D3;
    private group : any;
    private foreground : any;
    private radiusScaleB : any;
    private arc : any;

    ngOnInit(){

    }
    

    ngOnChanges(changes: SimpleChanges): void {


        if (!changes["Value"].isFirstChange()) {
          this.Update(this.Value)
        }
    
      }

   
  ngAfterViewInit() {
    this.SetUpchart()
    this.Update(this.Value);
  }


  SetUpchart(){

    const height = 300; 
    const width = 300; 
    const outerRadius = 110; 
    const innerRadius = 100; 

    const svg = this.d3.select('#' + 'fjknjfadkbvhaj').append('svg')
    .attr('width', width)
    .attr('height', height)
    .style('background', '#000000'); 

    this.group = svg.append('g')
    .attr('class', 'group')
    .attr('transform', `translate(${width * 0.5}, ${height * 0.5})`);

    const arc = this.d3.arc().innerRadius(innerRadius).outerRadius(outerRadius) 
    const keduListB = [0,1,2,3,4,5,6,7,8,9,10,11,12]; 
    const keduListS = [0,1,2,3,4,5,6,7,8,9,10,11]; 

    const radiusScaleB = this.d3.scaleLinear().domain([0, 12]).range([Math.PI * (- 2 / 3), Math.PI * 2 / 3]) 
    const radiusScaleS = this.d3.scaleLinear().domain([0, 11]).range([Math.PI * (- 11 / 18), Math.PI * ( 11 / 18 )]) 

    this.group.append('g').attr('class', 'arcs') // เส้นโค้ง
    this.group.append('g').attr('class', 'thick_line') //เส้นหนา
    this.group.append('g').attr('class', 'thin_line') //เส้นบาง
    this.group.append('g').attr('class', 'needle') //เข็ม
    this.group.append('g').attr('class', 'value') // value
    this.group.append('g').attr('class', 'show') //
    this.group.append('g').attr('class', 'button1') 

    this.group.select('.arcs').append('path')
    .attr('d', arc({startAngle: radiusScaleB(-0.3),endAngle: radiusScaleB(12.3)}))
    .attr('fill', '#0752b7')
    .attr('stroke', '#0752b7');

    this.group.select('.thick_line')
    .selectAll('line')
    .data(keduListB)
    .enter()
    .append('line')
    .attr('x1', d => 95 * Math.sin(radiusScaleB(d)))
    .attr('y1', d => 95 * Math.cos(radiusScaleB(d)) * -1)
    .attr('x2', d => 86 * Math.sin(radiusScaleB(d)))
    .attr('y2', d => 86 * Math.cos(radiusScaleB(d)) * -1)
    .attr('stroke', '#f2ffff')
    .attr('stroke-width', 5)

    this.group.select('.thin_line')
    .selectAll('line')
    .data(keduListS)
    .enter()
    .append('line')
    .attr('x1', d => 110 * Math.sin(radiusScaleS(d)))
    .attr('y1', d => 110 * Math.cos(radiusScaleS(d)) * -1)
    .attr('x2', d => 90 * Math.sin(radiusScaleS(d)))
    .attr('y2', d => 90 * Math.cos(radiusScaleS(d)) * -1)
    .attr('stroke', '#f2ffff')
    .attr('stroke-width', 1.5)

    this.group.select('.value')
    .selectAll('text')
    .data(keduListB)
    .enter()
    .append('text')
    .attr('x', d => 70 * Math.sin(radiusScaleB(d)))
    .attr('y', d => 70 * Math.cos(radiusScaleB(d)) * -1)
    .attr('text-anchor', 'middle')
    .attr('dy', '0.35em')
    .attr('fill', '#f2ffff')
    .attr('font-size','8pt')
    //.style('text-shadow', '0px 0px 2px #6AE3F8')
    .text(d => d * 20)

    this.group.select('.needle')
    .append('polygon')
    .attr('points', '-2,-20,2,-20,4,0,1,60,-1,60,-4,0')
    .attr('fill', '#e90708')
    .attr('transform', 'rotate(60)')

    this.group.select('.show')
    .attr('transform', 'translate(0, 100)')
    .append('rect')
    .attr('x', -60)
    .attr('y', 0)
    .attr('width', 120)
    .attr('height', 40)
    .attr('fill', 'none')
    .attr('stroke', '#6AE3F8')
    .attr('stroke-width', 3)
    this.group.select('.show')
    .append('text')
    .attr('text-anchor', 'middle')
    .attr('x', 0)
    .attr('y', 20)
    .attr('dy', '0.35em')
    .attr('fill', '#6AE3F8')
    .attr('stroke', '#6AE3F8')
    .attr('font-size', '20px')
    .text(`0 km/h`)


    //change()
 
  }


  
   Update = value => {

    let d3 : any = this.d3
    let texts = this.texts

    this.group.select('.needle')
        .transition()
        .duration(2000)
        .ease(this.d3.easeLinear)
        .attrTween('transform', function () {
            this._before = typeof this.getAttribute('transform') == 'string' ? this.getAttribute('transform').match(/\d+/g)[0] : 0;
            this._after = value;
            let interpolate = d3.interpolate(this._before, this._after);
            return function(t) {
                texts(interpolate(t))
                return `rotate(${interpolate(t)})`
            };
        })
    }


 texts = text => {
    this.group.select('.show').select('text')
    .attr('text-anchor', 'middle')
    .attr('x', 0)
    .attr('y', 20)
    .attr('dy', '0.35em')
    .attr('fill', '#6AE3F8')
    .attr('stroke', '#6AE3F8')
    .attr('font-size', '20px')
    .text(`${Math.floor(text)} km/h`)
}

}