import { Component, OnInit, OnChanges, Input, SimpleChanges, Output, EventEmitter } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import * as D3 from 'd3';



@Component({
  selector: 'stack-bar-horizontal-ii',
  templateUrl: './stack-bar-horizontal-ii.component.html',
  styleUrls: ['./stack-bar-horizontal-ii.component.css']
})
export class StackBarHorizontalIIComponent implements OnInit, OnChanges {

  constructor(
    private http: HttpService
  ) { }


  @Input() title: string = "Demo Chart";
  @Input("ID") ID: string;
  @Input() groupKey: Array<any> = ["PASS", "FAIL", "ALL"];
  @Input() LegenList: any = [{ legend: "PASSsssss", color: '#8eb4e3' }, { legend: "FAILlllll", color: '#92d14f' }, { legend: "ALL", color: "#ec3043" }]
  @Input() Data: any = [];
  @Input() Width: any;
  @Input() Heigth: any;
  @Input() Colors: any = ['#9FAAAE', '#92d14f', '#f38f10']
  @Input() ShowLegen: boolean = false;
  @Input() ShowTitle: boolean = false;
  @Input() highlight: boolean = false;
  @Input() Showhighlight: boolean = false;
  @Input() Compare: string;
  @Input() FilterType: string;
  @Output("CatagoryClick") CatagoryClick = new EventEmitter<any>()
  @Output("BarClick") BarClick = new EventEmitter<any>()



  private svg: any;
  private yScale: any;
  private xScale: any;
  private xAxis: any;
  private yAxis: any;
  private chart: any;
  private z: any;
  private barGroups: any;
  private data: any;
  private margin = { top: 10, right: 20, bottom: 10, left: 20 }
  private width = 600 - this.margin.left - this.margin.right
  private height = 400 - this.margin.top - this.margin.bottom;
  public layers: any;
  public delay: number = 0
  public delayText: number = 1000

  private d3: any = D3;
  private catagory: string;



  ngOnInit() {

    if (this.Width) {
      this.width = this.Width - this.margin.left - this.margin.right
    }
    if (this.Heigth) {
      this.height = this.Heigth - this.margin.top - this.margin.bottom
    }

    if (this.Data.length == 0) {

    }
  }

  ngOnChanges(changes: SimpleChanges): void {

    if (!changes["Data"].isFirstChange()) {
      //this.d3.select("#" + 'stack' + this.ID).remove();
      //this.SetUpChart(this.ID);
      this.delay = 3000;
      this.delayText = 0;
      this.CreateChart(this.Data);
    }
  }

  ngAfterViewInit() {
    this.SetUpChart(this.ID)
  }


  SetUpChart(id) {

    //var group = ["PASS","FAIL"];

    this.svg = this.d3.select("#" + id).append('svg')
      .attr("id", 'stack' + id)
      .attr('width', this.width + this.margin.left + this.margin.right)
      .attr('height', this.height + this.margin.top + this.margin.bottom + 20)
      .style("background", "rgb(255, 255, 255");


    this.yScale = this.d3.scaleBand()
      .range([this.height - this.margin.bottom, this.margin.top])
      .padding(0.5)


    this.xScale = this.d3.scaleLinear()
      .range([this.margin.left, (this.width - this.margin.right - this.margin.left)])

    this.xAxis = this.d3.axisTop(this.xScale);

    this.yAxis = this.d3.axisLeft(this.yScale).tickSizeOuter(0)
      .tickSizeInner(0)

    let translate_y = this.margin.top
    if (this.ShowLegen) {
      translate_y = this.margin.top + 30
    }

    this.chart = this.svg.append('g')
      .attr('transform', `translate(${this.margin.left + this.margin.right + this.margin.left}, ${translate_y})`);

    // this.chart.append('g')
    //   .attr("class", "x-axis")
    //   .attr("transform", `translate(0,${this.margin.top})`)
    //   .call(this.xAxis)


    this.chart.append('g')
      .attr("class", "y-axis")
      .attr("transform", `translate(${this.margin.left},0)`)
      .call(this.yAxis.tickSizeOuter(0))
      .attr("font-size", "7pt")


    this.z = this.d3.scaleOrdinal()
      .range(this.Colors, (d) => d)
      .domain(this.groupKey);

    //this.getData();

    this.CreateChart(this.Data)
  }

  CreateChart(gData) {

    var max = this.generateYmaxDomain(gData)
    this.yScale.domain(gData.map(d => {

      return d.catagory

    }));

    if (this.Data.length == 0) {

      this.chart.selectAll(".y-axis").transition().duration(5)
        .call(g => g.select(".domain").remove())
    } else {

      let check = this.Data.every(d => this.groupKey.every(x => d[x] == null))

      if (check) {
        this.chart.selectAll(".y-axis").transition().duration(5)
          .call(this.d3.axisLeft(this.yScale))
          .call(g => g.select(".domain").remove())
      } else {
        this.chart.selectAll(".y-axis").transition().duration(5)
          .call(this.d3.axisLeft(this.yScale))
      }


    }

    var ymax = this.generateYmaxDomain(this.Data)

    this.xScale.domain([0, this.d3.max(gData, d => this.d3.sum(this.groupKey, k => +d[k]) + 10)]).nice();

    var bargroup = this.chart.selectAll("g.layer")
      .data(this.d3.stack().keys(this.groupKey)(gData), d => d.key)

    bargroup.exit().remove()

    //------------ Modify to original 
    bargroup.enter().append("g")
      .classed("layer", true)
      .attr("fill", d => this.z(d.key));


    var bars = this.chart.selectAll("g.layer").selectAll("rect")
      .data(d => d, e => e.data.catagory);

    this.svg.selectAll(".y-axis .tick")
      .style("cursor", "pointer")
      .style("font-size", "8pt")
      .on("click", (d) => this.CatagoryClick.emit([d, this.ID]))


    //this.svg.selectAll('path').exit().remove()
    bars.exit().remove()

    bars.enter().append("rect")
      .merge(bars)
      .transition().duration(this.delay)
      .attr("x", d => this.xScale(d[0]))
      .attr("y", d => this.yScale(d.data.catagory))
      .attr("width", (d) => (this.xScale(d[1]) - this.xScale(d[0])))
      .attr("height", d => this.yScale.bandwidth())


    let d3: any = this.d3
    let CatagoryClick = this.CatagoryClick
    let ID = this.ID
    this.svg.selectAll("rect")
      .style("cursor", "pointer")
      .on("click", function (d) { CatagoryClick.emit([d.data.catagory, ID]) })


    this.svg.selectAll("rect")
      .attr('fill', d => {
        if (d.data.catagory == this.Compare) {
          return "#6cd350"
        }

        if (d.data.catagory == "Nationwide") {
          return "#e43b21"
        }
      })

    this.chart.selectAll(".textvalue")
      .remove().exit()

    bars = this.chart.selectAll("g.layer").selectAll("textvalue")
      .data(d => d, e => e.data.catagory);

    bars.enter().append("text")
      .attr('class', 'textvalue')
      .text((d) => (d[1] - d[0]) > 0 ? (d[1] - d[0]) : '')
      .transition().duration(this.delayText)
      .attr("x", (d) => (this.xScale(d[1]) + (this.xScale(d[0]) - this.xScale(d[1])) / 90) + 12)
      .attr('y', (d) => (this.yScale(d.data.catagory) + this.yScale.bandwidth() / 2) + 5)
      .attr('text-anchor', 'middle')
      .attr("font-size", "7pt")
      .style("fill", (d) => "#000000");



    let y_point = 0;

    if (this.ShowLegen) {
      y_point = -35
      this.CreateLegen()

    }

    if (this.ShowTitle) {

      let header = gData.find(x => x.catagory == this.Compare)

      this.svg.select('.textH').remove()
      this.chart.append('text')
        .attr('class', 'textH')
        .attr('x', (d) => {
          return this.width / 2
        })
        .attr('y', 5)
        .attr("font-size", "10pt")
        .attr('text-anchor', 'middle')
        .style("fill", "#000000")
        .text(() => {
          if (this.Compare == 'Outlet') {
            if (header == undefined) {
              return ''
            } else {
              return header.OUTLET_NAME_EN
            }
          } else {
            if (header == undefined) {
              return ''
            } else {
              return header.catagory

            }
          }
        })
    }
  }

  CreateLegen() {
    let legendList = this.LegenList
    let height = this.height;
    let width = this.width;

    var nodeWidth = (d) => d.getBBox().width;
    //const cScale = this.d3.scaleOrdinal();

    const legend = this.svg.append('g')
      .attr('class', 'legend')
      .attr('transform', 'translate(0,0)');

    const lg = legend.selectAll('g')
      .data(legendList)
      .enter()
      .append('g')
      .attr('transform', (d, i) => `translate(${i * 100},${height + 15})`);

    lg.append('rect')
      .style('fill', d => d.color)
      .attr('x', -100)
      .attr('y', 90)
      .attr('width', 50)
      .attr('height', 10);

    lg.append('text')
      //.style('font-family', 'Georgia')
      .style('font-size', '12px')
      .attr('x', -40)
      .attr('y', 100)
      .text(d => d.legend);

    let offset = 0;
    lg.attr('transform', function (d, i) {
      let x = offset;
      offset += nodeWidth(this) + 10;
      return `translate(${x},${-40})`;
    });

    legend.attr('transform', function () {
      return `translate(${(width - nodeWidth(this)) / 2},${0})`
    });
  }


  generateYmaxDomain(data: any) {

    var yMaxDomain: number = 0;
    yMaxDomain = this.d3.max(data, d => this.d3.sum(this.groupKey, k => +d[k]))
    //  //yMaxDomain = 100  
    yMaxDomain = yMaxDomain + ((yMaxDomain * 5) / 100);
    return yMaxDomain
  }

  generateYminDomain(data: any) {
    var yMinDomain: number = 0;
    //var yMinDomain = this.d3.min(data, d => Number(d.Value))
    //yMinDomain = yMinDomain - ((yMinDomain * 5) / 100)
    return yMinDomain
  }

  generateHearderList(data) {
    let newData = [];
    for (var key in data[0]) {
      if (key != 'FY' && key != 'catagory' && key != 'Value') {
        newData.push(key)
      }

    }
    return newData
  }

}


export interface ChartData {
  Values: Array<valueModel>;
  Legend: Array<legendModel>
}


export interface valueModel {
  catagory: string;
  PERCENT: number;
  //Value2: number;
  //Value3: number;
}

export interface legendModel {
  legend: string;
  color: string;

}



