import { Component, OnInit, OnChanges, Input, SimpleChanges, Output, EventEmitter } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import * as D3 from 'd3';


@Component({
  selector: 'stack-bar-horizontal-iii',
  templateUrl: './stack-bar-horizontal-iii.component.html',
  styleUrls: ['./stack-bar-horizontal-iii.component.css']
})
export class StackBarHorizontalIIIComponent implements OnInit, OnChanges {

  constructor(
    private http: HttpService
  ) { }


  @Input() title: string = "Demo Chart";
  @Input("ID") ID: string;
  @Input() groupKey: Array<any> = ["PASS", "FAIL", "ALL"];
  @Input() LegenList: any = [{ legend: "PASSsssss", color: '#8eb4e3' }, { legend: "FAILlllll", color: '#92d14f' }, { legend: "ALL", color: "#ec3043" }]
  @Input() Data: any = [];
  @Input() Width: any;
  @Input() Heigth: any;
  @Input() Colors: any = ['#8eb4e3', '#92d14f', '#ec3043']
  @Input() ShowLegen: boolean = false;
  @Input() ShowTitle: boolean = false;
  @Output("CatagoryClick") CatagoryClick = new EventEmitter<any>()



  private svg: any;
  private yScale: any;
  private xScale: any;
  private xAxis: any;
  private yAxis: any;
  private chart: any;
  private z: any;
  private barGroups: any;
  private data: any;
  private margin = { top: 15, right: 25, bottom: 10, left: 20 }
  private width = 600 - this.margin.left - this.margin.right
  private height = 400 - this.margin.top - this.margin.bottom;
  public layers: any;
  public delay: number = 0
  public delayText: number = 1000

  private d3: any = D3;




  ngOnInit() {

    if (this.Width) {
      this.width = this.Width - this.margin.left - this.margin.right
    }
    if (this.Heigth) {
      this.height = this.Heigth - this.margin.top - this.margin.bottom
    }

    if (this.Data.length == 0) {
      // this.Data = [{ 'catagory': 'Apr', 'PASS': 150, 'FAIL': 20, 'PERCENT': 75, 'ALL': 200 },
      // { 'catagory': 'May', 'PASS': 180, 'FAIL': 20, 'PERCENT': 90, 'ALL': 200 },
      // { 'catagory': 'Jun', 'PASS': 189, 'FAIL': 11, 'PERCENT': 94, 'ALL': 200 },
      // { 'catagory': 'Jul', 'PASS': 182, 'FAIL': 18, 'PERCENT': 91, 'ALL': 200 },
      // { 'catagory': 'Aug', 'PASS': 142, 'FAIL': 50, 'PERCENT': 83, 'ALL': 200 },
        //  {'catagory': 'Sep','PASS': 178,'FAIL': 22,'PERCENT': 89,'ALL': 200}, 
        //  {'catagory': 'Oct','PASS': 160,'FAIL': 40,'PERCENT': 80,'ALL': 200},
        //  {'catagory': 'Nov','PASS': 169,'FAIL': 31,'PERCENT': 84,'ALL': 200}, 
        //  {'catagory': 'Dec','PASS': 175,'FAIL': 25,'PERCENT': 87,'ALL': 200}, 
        //  {'catagory': 'Jan','PASS': 195,'FAIL': 5,'PERCENT': 97,'ALL': 200}, 
        //  {'catagory': 'Feb','PASS': 166,'FAIL': 34,'PERCENT': 83,'ALL': 200},
        //  {'catagory': 'Mar','PASS': 183,'FAIL': 17,'PERCENT': 91,'ALL': 200}
      //]
    }
  }

  ngOnChanges(changes: SimpleChanges): void {

    if (!changes["Data"].isFirstChange()) {
      //this.d3.select("#" + 'stack' + this.ID).remove();
      //this.SetUpChart(this.ID);
      this.delay = 3000;
      this.delayText = 0;
      this.CreateChart(this.Data);
    }

  }

  ngAfterViewInit() {
    this.SetUpChart(this.ID)
  }



  SetUpChart(id) {

    //var group = ["PASS","FAIL"];

    this.svg = this.d3.select("#" + id).append('svg')
      .attr("id", 'stack' + id)
      .attr('width', this.width + this.margin.left + this.margin.right)
      .attr('height', this.height + this.margin.top + this.margin.bottom + 20)
      .style("background", "rgb(243, 243, 243");


    this.yScale = this.d3.scaleBand()
      .range([this.height - this.margin.bottom, this.margin.top])
      .padding(0.7)
      

    this.xScale = this.d3.scaleLinear()
      .range([this.margin.left, (this.width / 3.25)])

    this.xAxis = this.d3.axisTop(this.xScale);

    this.yAxis = this.d3.axisLeft(this.yScale).tickSizeOuter(0)
    .tickSizeInner(0);

    let translate_y = this.margin.top
    if (this.ShowLegen) {
      translate_y = this.margin.top + 30
    }

    this.chart = this.svg.append('g')
      .attr('transform', `translate(${(this.width / 1.30)}, ${translate_y})`);

    // this.chart.append('g')
    //   .attr("class", "x-axis")
    //   .attr("transform", `translate(0,${this.margin.top})`)
    //   .call(this.xAxis)


    

    this.chart.append('g')
      .attr("class", "y-axis")
      .attr("transform", `translate(${this.margin.left},0)`)
          .call(this.yAxis.tickSizeOuter(0))
      .attr("font-size", "7.5pt")
      



    this.z = this.d3.scaleOrdinal()
      .range(this.Colors, (d) => d)
      .domain(this.groupKey);

    //this.getData();
    
    this.CreateChart(this.Data)
  }

  CreateChart(gData) {
    
    var max = this.generateYmaxDomain(gData)
    this.yScale.domain(gData.map(d => d.catagory));


   

    if(this.Data.length == 0){
    
      this.chart.selectAll(".y-axis").transition().duration(5)
      .call(this.d3.axisLeft(this.yScale))
      .call(g => g.select(".domain").remove())
    }else{
      
      let check = this.Data.every(d => this.groupKey.every(x => d[x] == null))

      if(check){
        this.chart.selectAll(".y-axis").transition().duration(5)
        .call(this.d3.axisLeft(this.yScale))
        .call(g => g.select(".domain").remove())
      }else{
        this.chart.selectAll(".y-axis").transition().duration(5)
        .call(this.d3.axisLeft(this.yScale))
        //.call(g => g.selectAll(".tick:not(:first-of-type) line").remove())
      }

      
    }
    




    var ymax = this.generateYmaxDomain(this.Data)

    this.xScale.domain([0, this.d3.max(gData, d => this.d3.sum(this.groupKey, k => +d[k]) + 10)]).nice();

    //this.xScale.domain([0,100]);

    // this.chart.selectAll(".x-axis").transition().duration(5)
    //   .call(this.d3.axisBottom(this.xScale).ticks(2))
    //   .call(g => g.selectAll(".domain").remove())
    //   .call(g => g.selectAll("line").remove())
    //   .call(g => g.selectAll("text").attr('y',-6))



    var bargroup = this.chart.selectAll("g.layer")
      .data(this.d3.stack().keys(this.groupKey)(gData), d => d.key)

    bargroup.exit().remove()

    bargroup.enter().append("g")
      .classed("layer", true)
      .attr("fill", d => this.z(d.key));

    var bars = this.chart.selectAll("g.layer").selectAll("rect")
      .data(d => d, e => e.data.catagory);

      this.svg.selectAll(".y-axis .tick")
      .style("cursor", "pointer")
      .on("click", (d) => this.CatagoryClick.emit([d, this.ID]))

      

    //this.svg.selectAll('path').exit().remove()
    bars.exit().remove()

    bars.enter().append("rect")
      .merge(bars)
      .transition().duration(this.delay)
      .attr("x", d => {
        // console.log(this.xScale(d[0]))
        return this.xScale(d[0])
      })
      .attr("y", d => this.yScale(d.data.catagory))
      .attr("width", (d) => (this.xScale(d[1]) - this.xScale(d[0])))
      //.attr("width",(d, i) =>  (this.xScale(d[1]) * 100))
      .attr("height", d => (this.yScale.bandwidth()))

      this.chart.selectAll(".textvalue")
      .remove().exit()

    bars = this.chart.selectAll("g.layer").selectAll("textvalue")
      .data(d => d, e => e.data.catagory);

   

    bars.enter().append("text")
      .attr('class', 'textvalue')
      .text((d) => (d[1] - d[0]) > 0 ? (d[1] - d[0]) : '')
      .transition().duration(this.delayText)
      .attr("x", (d) => ((this.xScale(d[1]) + (this.xScale(d[0]) - this.xScale(d[1])) / 2)) + 10)
      .attr('y', (d) => (this.yScale(d.data.catagory) + this.yScale.bandwidth() / 2) + 5)
      .attr('text-anchor', 'middle')
      .attr("font-size", "7pt")
      .style("fill", (d) => "#000000");

    // var text = this.svg.selectAll(".text")
    //   .data(gData, d => d.catagory);


    //text.exit().remove()

    // text.enter().append("text")
    //   .attr("class", "text")
    //   .attr("text-anchor", "middle")
    //   .merge(text)
    //   .transition().duration(5)
    //   .attr("y", d => (this.yScale(d.catagory) + this.yScale.bandwidth()) + this.margin.left + this.margin.right)
    //   .attr("x", d => this.xScale(d.total) + 40)
    //   .text(d => d.PERCENT + '' + '%')

    let y_point = 0;

    if (this.ShowLegen) {
      y_point = -35
      this.CreateLegen()

    }

    if (this.ShowTitle) {
      this.svg.select('.texttitle').remove()

      this.chart.append('text')
        .attr('class', 'texttitle')
        .attr('x', this.width / 2)
        .attr('y', y_point)
        .attr("font-size", "20px")
        .attr('text-anchor', 'middle')
        .text(this.title)
    }
    



  }


  CreateLegen() {
    let legendList = this.LegenList
    let height = this.height;
    let width = this.width;

    var nodeWidth = (d) => d.getBBox().width;
    //const cScale = this.d3.scaleOrdinal();

    const legend = this.svg.append('g')
      .attr('class', 'legend')
      .attr('transform', 'translate(0,0)');

    const lg = legend.selectAll('g')
      .data(legendList)
      .enter()
      .append('g')
      .attr('transform', (d, i) => `translate(${i * 100},${height + 15})`);

    lg.append('rect')
      .style('fill', d => d.color)
      .attr('x', -100)
      .attr('y', 90)
      .attr('width', 50)
      .attr('height', 10);

    lg.append('text')
      //.style('font-family', 'Georgia')
      .style('font-size', '12px')
      .attr('x', -40)
      .attr('y', 100)
      .text(d => d.legend);

    let offset = 0;
    lg.attr('transform', function (d, i) {
      let x = offset;
      offset += nodeWidth(this) + 10;
      return `translate(${x},${-40})`;
    });

    legend.attr('transform', function () {
      return `translate(${(width - nodeWidth(this)) / 2},${0})`
    });
  }



  generateYmaxDomain(data: any) {

    var yMaxDomain: number = 0;
    yMaxDomain = this.d3.max(data, d => this.d3.sum(this.groupKey, k => +d[k]))
    //  //yMaxDomain = 100  
    yMaxDomain = yMaxDomain + ((yMaxDomain * 5) / 100);
    return yMaxDomain
  }

  generateYminDomain(data: any) {
    var yMinDomain: number = 0;
    //var yMinDomain = this.d3.min(data, d => Number(d.Value))
    //yMinDomain = yMinDomain - ((yMinDomain * 5) / 100)
    return yMinDomain
  }

  generateHearderList(data) {
    let newData = [];
    for (var key in data[0]) {
      if (key != 'FY' && key != 'catagory' && key != 'Value') {
        newData.push(key)
      }

    }
    return newData
  }


}


export interface ChartData {
  Values: Array<valueModel>;
  Legend: Array<legendModel>
}


export interface valueModel {
  catagory: string;
  PERCENT: number;
  //Value2: number;
  //Value3: number;
}

export interface legendModel {
  legend: string;
  color: string;

}



