import { Component, OnInit, OnChanges, Input, SimpleChanges } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import * as D3 from 'd3';

@Component({
  selector: 'stack-bar-horizontal',
  templateUrl: './stack-bar-horizontal.component.html',
  styleUrls: ['./stack-bar-horizontal.component.css']
})
export class StackBarHorizontalComponent implements OnInit, OnChanges {

  constructor(
    private http: HttpService
  ) { }


  @Input() title: string = "Demo Chart";
  @Input("ID") ID: string;
  @Input() groupKey : Array<any> = ["PASS","FAIL"];
  @Input() LegenList: any = [{ legend: "PASSsssss", color: '#8eb4e3' }, { legend: "FAILlllll", color: '#92d14f'}]
  @Input() Data: any = [];
  @Input() Width: any;
  @Input() Heigth: any;

 

  private svg: any;
  private yScale: any;
  private xScale: any;
  private xAxis: any;
  private yAxis: any;
  private chart: any;
  private z: any;
  private barGroups: any;
  private data: any;
  private margin = { top: 30, right: 20, bottom: 20, left: 35 }
  private width = 600 - this.margin.left - this.margin.right
  private height = 400 - this.margin.top - this.margin.bottom;
  public layers: any;
  public delay: number = 0

  private d3 : any = D3;


  
    ngOnInit() {

        if (this.Width) {
            this.width = this.Width - this.margin.left - this.margin.right
        }
        if (this.Heigth) {
            this.height = this.Heigth - this.margin.top - this.margin.bottom
        }

    if(this.Data.length == 0){
      this.Data = [{'catagory': 'Apr','PASS': 150,'FAIL': 20,'PERCENT': 75,'total': 200}, 
                   {'catagory': 'May','PASS': 180,'FAIL': 20,'PERCENT': 90,'total': 200},
                   {'catagory': 'Jun','PASS': 189,'FAIL': 11,'PERCENT': 94,'total': 200}, 
                   {'catagory': 'Jul','PASS': 182,'FAIL': 18,'PERCENT': 91,'total': 200}, 
                   {'catagory': 'Aug','PASS': 167,'FAIL': 33,'PERCENT': 83,'total': 200}, 
                  //  {'catagory': 'Sep','PASS': 178,'FAIL': 22,'PERCENT': 89,'total': 200}, 
                  //  {'catagory': 'Oct','PASS': 160,'FAIL': 40,'PERCENT': 80,'total': 200},
                  //  {'catagory': 'Nov','PASS': 169,'FAIL': 31,'PERCENT': 84,'total': 200}, 
                  //  {'catagory': 'Dec','PASS': 175,'FAIL': 25,'PERCENT': 87,'total': 200}, 
                  //  {'catagory': 'Jan','PASS': 195,'FAIL': 5,'PERCENT': 97,'total': 200}, 
                  //  {'catagory': 'Feb','PASS': 166,'FAIL': 34,'PERCENT': 83,'total': 200},
                  //  {'catagory': 'Mar','PASS': 183,'FAIL': 17,'PERCENT': 91,'total': 200}
                  ]}
  }

  ngOnChanges(changes: SimpleChanges): void {

    if (!changes["Data"].isFirstChange()) {
      this.d3.select("#" + 'stack' + this.ID).remove();
      this.setUpChart(this.ID);
    }

  }

  ngAfterViewInit() {
    this.setUpChart(this.ID)
  }

  

  setUpChart(id) {

    //var group = ["PASS","FAIL"];

    this.svg = this.d3.select("#" + id).append('svg')
    .attr("id", 'stack' + id)
    .attr('width', this.width + this.margin.left + this.margin.right)
    .attr('height', this.height + this.margin.top + this.margin.bottom + 20)
    .style("background", "rgb(255, 255, 255");


    this.yScale = this.d3.scaleBand()
      .range([this.height - this.margin.bottom, this.margin.top])
      .padding(0.3)

    this.xScale = this.d3.scaleLinear()
      .range([this.margin.left, this.width - this.margin.right])

    this.xAxis = this.d3.axisBottom(this.xScale);

    this.yAxis = this.d3.axisLeft(this.yScale);

    this.chart = this.svg.append('g')
      .attr('transform', `translate(${this.margin.left / 2}, ${this.margin.top + 30})`);

    this.chart.append('g')
      .attr("class", "x-axis")
      .attr("transform", `translate(0,${this.height - this.margin.bottom})`)
      .call(this.xAxis)

    this.chart.append('g')
      .attr("class", "y-axis")
      .attr("transform", `translate(${this.margin.left},0)`)
      .call(this.yAxis)

 

    this.z = this.d3.scaleOrdinal()
      .range([this.LegenList[0].color, this.LegenList[1].color])
      .domain(this.groupKey);

    //this.getData();
    this.createChart(this.Data)
  }

  createChart(gData) {
      console.log(gData)
    var max = this.generateYmaxDomain(gData)
    this.yScale.domain(gData.map(d => d.catagory));

  
    this.chart.selectAll(".y-axis").transition().duration(5)
      .call(this.d3.axisLeft(this.yScale))


      var ymax = this.generateYmaxDomain(this.Data)

    this.xScale.domain([0, this.d3.max(gData, d => this.d3.sum(this.groupKey, k => +d[k]) + 10)]).nice();

    this.chart.selectAll(".x-axis").transition().duration(5)
      .call(this.d3.axisBottom(this.xScale).ticks(5))

    var bargroup = this.chart.selectAll("g.layer")
      .data(this.d3.stack().keys(this.groupKey)(gData), d => d.key)

    bargroup.exit().remove()

    bargroup.enter().append("g")
      .classed("layer", true)
      .attr("fill", d => this.z(d.key));

    var bars = this.chart.selectAll("g.layer").selectAll("rect")
    .data(d => d, e => e.data.catagory);

    //this.svg.selectAll('path').exit().remove()
    bars.exit().remove()

    bars.enter().append("rect")   
      .merge(bars)
      .transition().duration(this.delay)
      .attr("x", d =>  this.xScale(d[0]))
      .attr("y", d => this.yScale(d.data.catagory))
      .attr("width",(d) => (this.xScale(d[1])  - this.xScale(d[0])) )
      .attr("height", d => this.yScale.bandwidth())

     bars = this.chart.selectAll("g.layer").selectAll("textvalue")
      .data(d => d, e => e.data.catagory);

      bars.exit().remove()

    bars.enter().append("text")
    .attr('class', 'textvalue')
      .text((d) =>  (d[1] - d[0]) > 0 ? (d[1] - d[0]).toFixed(2) : '')
      .transition().duration(this.delay)
      .attr("x", (d) => (this.xScale(d[1]) + (this.xScale(d[0]) - this.xScale(d[1])) / 2)  )
      .attr('y', (d) => (this.yScale(d.data.catagory) + this.yScale.bandwidth() / 2) + 5)
      .attr('text-anchor', 'middle')
      .attr("font-size", "10pt")
      .style("fill", (d) => "#000000");

    // var text = this.svg.selectAll(".text")
    //   .data(gData, d => d.catagory);


    //text.exit().remove()

    // text.enter().append("text")
    //   .attr("class", "text")
    //   .attr("text-anchor", "middle")
    //   .merge(text)
    //   .transition().duration(5)
    //   .attr("y", d => (this.yScale(d.catagory) + this.yScale.bandwidth()) + this.margin.left + this.margin.right)
    //   .attr("x", d => this.xScale(d.total) + 40)
    //   .text(d => d.PERCENT + '' + '%')

    this.svg.select('.texttitle').remove()

    this.chart.append('text')
      .attr('class', 'texttitle')
      .attr('x', this.width / 2)
      .attr('y', -35)
      .attr("font-size", "20px")
      .attr('text-anchor', 'middle')
      .text(this.title)


    let legendList = this.LegenList
    let height = this.height;
    let width = this.width;

    var nodeWidth = (d) => d.getBBox().width;
    //const cScale = this.d3.scaleOrdinal();

    const legend = this.svg.append('g')
      .attr('class', 'legend')
      .attr('transform', 'translate(0,0)');

    const lg = legend.selectAll('g')
      .data(legendList)
      .enter()
      .append('g')
      .attr('transform', (d, i) => `translate(${i * 100},${height + 15})`);

    lg.append('rect')
      .style('fill', d => d.color)
      .attr('x', 40)
      .attr('y', 90)
      .attr('width', 40)
      .attr('height', 10);

    lg.append('text')
      //.style('font-family', 'Georgia')
      .style('font-size', '12px')
      .attr('x', 90)
      .attr('y', 100)
      .text(d => d.legend);

    let offset = 0;
    lg.attr('transform', function (d, i) {
      let x = offset;
      offset += nodeWidth(this) + 10;
      return `translate(${x},${-40})`;
    });

    legend.attr('transform', function () {
      return `translate(${(width - nodeWidth(this)) / 2},${0})`
    });

  }

  

  generateYmaxDomain(data: any) {

    var yMaxDomain: number = 0;
    yMaxDomain = this.d3.max(data, d => this.d3.sum(this.groupKey, k => +d[k]))
    //  //yMaxDomain = 100  
    yMaxDomain = yMaxDomain + ((yMaxDomain * 5) / 100);
    return yMaxDomain
  }

  generateYminDomain(data: any) {
    var yMinDomain: number = 0;
    //var yMinDomain = this.d3.min(data, d => Number(d.Value))
    //yMinDomain = yMinDomain - ((yMinDomain * 5) / 100)
    return yMinDomain
  }

  generateHearderList(data) {
    let newData = [];
    for (var key in data[0]) {
      if (key != 'FY' && key != 'catagory' && key != 'Value') {
        newData.push(key)
      }

    }
    return newData
  }


}


export interface ChartData {
  Values: Array<valueModel>;
  Legend: Array<legendModel>
}


export interface valueModel {
  catagory: string;
  PERCENT: number;
  //Value2: number;
  //Value3: number;
}

export interface legendModel {
  legend: string;
  color: string;

}



