import { Component, OnInit, OnChanges, ElementRef, Input, ViewChild, ViewEncapsulation, SimpleChanges } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import * as D3 from 'd3';

@Component({
  selector: 'stack-bar-line-i',
  templateUrl: './stack-bar-line-i.component.html',
  styleUrls: ['./stack-bar-line-i.component.css']
})
export class StackBarLineIComponent implements OnInit, OnChanges {

  constructor(
    private http: HttpService
  ) { }


  @Input() title: string = "Simple";
  @Input("ID") ID: string;
  @Input() groupKey: Array<any> = ["Value1", "Value2", "Value3", "Value4"];//["PASS","FAIL"]
  @Input() LegenList: any  = [{ legend: "PASSsssss", color: '#99cc00' }, { legend: "FAILlllll", color: '#c00000'}]
  @Input() Data : any =[];
  @Input() Width : any;
  @Input() Heigth : any;
 

  private svg: any;
  private yScale: any;
  private yScale1 :any;
  private xScale: any;
  private xAxis: any;
  private yAxis: any;
  private yAxis1 : any;
  private chart: any;
  private z: any;
  private barGroups: any;
  private data: any;
  private margin = { top: 30, right: 20, bottom: 20, left: 35 }
  private width = 600 - this.margin.left - this.margin.right
  private height = 400 - this.margin.top - this.margin.bottom;
  public layers: any;
  public delay: number = 0

  private d3 : any = D3;

  ngOnInit() {

    if (this.Width) {
      this.width = this.Width - this.margin.left - this.margin.right
    }
    if (this.Heigth) {
      this.height = this.Heigth - this.margin.top - this.margin.bottom
    }

    if(this.Data.length == 0){
      this.Data = [{ 'catagory': 'Apr', 'Value1': 150, 'Value2': 20, 'Value3': 150, 'Value4': 20,'PERCENT': 99}, 
                    { 'catagory': 'May', 'Value1': 178, 'Value2': 20, 'Value3': 150, 'Value4': 20,'PERCENT': 99},
                    { 'catagory': 'Jun', 'Value1': 188, 'Value2': 11, 'Value3': 150, 'Value4': 20,'PERCENT': 99}, 
                    { 'catagory': 'Jul', 'Value1': 180, 'Value2': 18, 'Value3': 150, 'Value4': 20,'PERCENT': 98}, 
                    { 'catagory': 'Aug', 'Value1': 160, 'Value2': 33, 'Value3': 150, 'Value4': 20,'PERCENT': 99}, 
                    { 'catagory': 'Sep', 'Value1': 178, 'Value2': 22, 'Value3': 150, 'Value4': 20,'PERCENT': 97}, 
                    { 'catagory': 'Oct', 'Value1': 160, 'Value2': 40, 'Value3': 150, 'Value4': 20,'PERCENT': 99},
                    { 'catagory': 'Nov', 'Value1': 169, 'Value2': 31, 'Value3': 150, 'Value4': 20,'PERCENT': 98}, 
                    { 'catagory': 'Dec', 'Value1': 175, 'Value2': 25, 'Value3': 150, 'Value4': 20,'PERCENT': 100}, 
                    { 'catagory': 'Jan', 'Value1': 195, 'Value2': 5, 'Value3': 150, 'Value4': 20,'PERCENT': 97}, 
                    { 'catagory': 'Feb', 'Value1': 166, 'Value2': 34, 'Value3': 150, 'Value4': 20,'PERCENT': 98},
                    { 'catagory': 'Mar', 'Value1': 183, 'Value2': 17, 'Value3': 150, 'Value4': 20,'PERCENT': 99}]}
  }

  ngOnChanges(changes: SimpleChanges): void {

    if (!changes["Data"].isFirstChange()) {
      //this.delay = 100
      //this.getData()
      this.d3.select("#" + 'stack' + this.ID).remove();
      this.setUpChart(this.ID)
    }

  }

  ngAfterViewInit() {
    this.setUpChart(this.ID)
  }



  setUpChart(id) {

    this.svg = this.d3.select("#" + id).append('svg')
    .attr("id", 'stack' + id)
    .attr('width', this.width + this.margin.left + this.margin.right)
    .attr('height', this.height + this.margin.top + this.margin.bottom + 60)
    .style("background", "rgb(255, 255, 255");


    this.xScale = this.d3.scaleBand()
      .range([this.margin.left, this.width - this.margin.right])
      .padding(0.3)

    this.yScale = this.d3.scaleLinear()
      .rangeRound([this.height - this.margin.bottom, this.margin.top])

      this.yScale1 = this.d3.scaleLinear()
      .range([this.height - this.margin.bottom, this.margin.top])
      //.domain([0, 120])

    this.xAxis = this.d3.axisBottom(this.xScale);

    this.yAxis = this.d3.axisLeft(this.yScale);

    this.yAxis1 = this.d3.axisRight(this.yScale1);

    this.chart = this.svg.append('g')
      .attr('transform', `translate(${this.margin.left / 2}, ${this.margin.top + 30})`);

    this.chart.append('g')
      .attr("class", "x-axis")
      .attr("transform", `translate(0,${this.height - this.margin.bottom})`)
      .call(this.xAxis)

    this.chart.append('g')
      .attr("class", "y-axis")
      .attr("transform", `translate(${this.margin.left},0)`)
      .call(this.yAxis)

      this.chart.append('g')
      .attr("class", "y-axis1")
      .attr("transform", `translate(${this.width - this.margin.right},0)`)
      .call(this.yAxis1);

    this.z = this.d3.scaleOrdinal()
      .range([this.LegenList[0].color, this.LegenList[1].color, this.LegenList[2].color, this.LegenList[3].color, this.LegenList[4].color])
      .domain(this.groupKey);

    //this.getData();
    this.createChart(this.Data)
  }

  createChart(gData) {
    console.log('gData', gData)
    var max = this.generateYmaxDomain(gData)
    this.yScale.domain([0, this.d3.max(gData, d => this.d3.sum(this.groupKey, k => +d[k]))]).nice();

  
    this.chart.selectAll(".y-axis").transition().duration(5)
      .call(this.d3.axisLeft(this.yScale).ticks(5))

    this.xScale.domain(gData.map(d => d.catagory));

    this.chart.selectAll(".x-axis").transition().duration(5)
      .call(this.d3.axisBottom(this.xScale).tickSizeOuter(0))


    this.yScale1.domain([0,110]);

    this.chart.selectAll(".y-axis1").transition().duration(5)
      .call(this.d3.axisRight(this.yScale1).ticks(5))

    var bargroup = this.chart.selectAll("g.layer")
      .data(this.d3.stack().keys(this.groupKey)(gData), d => d.key)

    bargroup.exit().remove();

    bargroup.enter().insert("g", ".y-axis").append("g")
			.classed("layer", true)
			.attr("fill", d => this.z(d.key));

    // bargroup.enter().append("g")
    //   .classed("layer", true)
    //   .attr("fill", d => this.z(d.key));

    var bars = this.chart.selectAll("g.layer").selectAll("rect")
    .data(d => d, e => e.data.catagory);

    //bars.exit().remove();

  var xx =  bars.enter().append("rect")   
      .merge(bars)
      //.transition().duration(this.delay)
      .attr("x", d => this.xScale(d.data.catagory))
      .attr("y", d => this.yScale(d[1]))
      .attr("width", this.xScale.bandwidth())
      .attr("height", d => {
       return (this.yScale(d[0])  - this.yScale(d[1]))
      })
      

    xx.transition().duration(this.delay)

    xx.on("mouseover", (d) => this.handleMouseOver(d, 'Value1'))
      .on("mouseout", (d) => this.handleMouseOut(d));

    bars.enter().append("text")
       // .text((d) => (d[1] - d[0]) > 0 ? (d[1] - d[0]) : '') //Show Text
      .transition().duration(this.delay)
      .attr("y", (d) => (this.yScale(d[1]) + (this.yScale(d[0]) - this.yScale(d[1])) / 2) +10 )
      .attr('x', (d) => (this.xScale(d.data.catagory) + this.xScale.bandwidth() / 2))
      .attr('text-anchor', 'middle')
      .attr("font-size", "8pt")
      .style("font-weight", 'bold')
      .style("fill", (d) => "#000000");



      var line = this.d3.line()
      .x((d, i) => this.xScale(d.catagory) + this.xScale.bandwidth() / 2)
      .y((d) => this.yScale1(d.PERCENT))
      //.curve(this.this.d3.curveMonotoneX);



    this.chart.append("path")
      .attr("fill", "none")
      .attr("stroke-width", 1.5)
      .attr("stroke", "#8eb4e3")   
      .attr("d", line(gData)); 

      this.chart.selectAll("dot")
      .data(gData)
      .enter()
      .append("circle")
      .style("fill","#8eb4e3")
      .attr("r", 4)
      .attr("cx",(d) => (this.xScale(d.catagory) + this.xScale.bandwidth() / 2))
      .attr("cy", (d) => this.yScale1(d.PERCENT))
      .on("mouseover", (d) => this.handleMouseOver2(d, 'd.PERCENT'))
      .on("mouseout", (d) => this.handleMouseOut2(d));


    var text = this.svg.selectAll(".text")
      .data(gData, d => d.catagory);


    text.exit().remove();

    let y = this.yScale
    let g = this.groupKey;

    text.enter().append("text")
      .attr("class", "text")
      .attr("text-anchor", "middle")
      .attr("font-size", "8pt")
      .merge(text)
      .transition().duration(5)
      .attr("x", d => (this.xScale(d.catagory) + this.xScale.bandwidth() / 2) + 20)
      .attr("y", d => (this.yScale1(d.PERCENT) + this.margin.top + 15))
      //.text(d => d.PERCENT > 0 ? d.PERCENT + '%' : '')

    this.svg.select('.texttitle').remove()

    this.chart.append('text')
      .attr('class', 'texttitle')
      .attr('x', this.width / 2)
      .attr('y', -15)
      .attr("font-size", "20px")
      .attr('text-anchor', 'middle')
      .style("font-weight", 'bold')
      .style("fill", "#595959")
      .text(this.title)


    let legendList = this.LegenList
    let height = this.height;
    let width = this.width;

    var nodeWidth = (d) => d.getBBox().width;
    //const cScale = this.d3.scaleOrdinal();

    const legend = this.svg.append('g')
      .attr('class', 'legend')
      .attr('transform', 'translate(0,0)');

    const lg = legend.selectAll('g')
      .data(legendList)
      .enter()
      .append('g')
      .attr('transform', (d, i) => `translate(${i * 100},${height + 15})`);

    lg.append('rect')
      .style('fill', d => d.color)
      .attr('x', -5)
      .attr('y', 60)
      .attr('width', 30)
      .attr('height', 10);

    lg.append('text')
      //.style('font-family', 'Georgia')
      .style('font-size', '10px')
      .attr('x', 28)
      .attr('y', 70)
      .text(d => d.legend);

    let offset = 0;
    lg.attr('transform', function (d, i) {
      let x = offset;
      offset += nodeWidth(this) + 10;
      //return `translate(${x},${-40})`;
      return `translate(${x},${height+10})`;
    });

    legend.attr('transform', function () {
      return `translate(${(width - nodeWidth(this)) / 2},${0})`
    });

    let xScale = this.xScale
    let yScale = this.yScale
    let chart = this.chart

  }

  handleMouseOver = (d, Value) => {
    this.chart.append("text")
      .attr('class', 'textHover')
      .attr("y", (this.yScale(d[1]) + (this.yScale(d[0]) - this.yScale(d[1])) / 2))
      .attr('x', (this.xScale(d.data.catagory) + this.xScale.bandwidth() / 2)-5)
      .attr("font-size", "7pt")
      .style("font-weight", 'bold')
      .style("fill", "#000000")
      .text(d[1] - d[0])
  }

  handleMouseOut = (d) => {
    this.chart.select('.textHover').remove()
  }

  handleMouseOver2 = (d, Value) => {
    this.chart.append("text")
      .attr('class', 'textHover')
      .attr("x", (this.xScale(d.catagory) + this.xScale.bandwidth() / 2)+4)
      .attr("y", (this.yScale1(d.PERCENT)))
      .attr("font-size", "7pt")
      .style("font-weight", 'bold')
      .style("fill", "#000000")
      .text(d.PERCENT + '%')
  }

  handleMouseOut2 = (d) => {
    this.chart.select('.textHover').remove()
  }

  


  generateYmaxDomain(data: valueModel[]) {

    var yMaxDomain: number;

    //if (this.TypeChart == 'Value') {
    //  //yMaxDomain = 100
    //  yMaxDomain = this.d3.max(data, d => Number(d.Value))
    //  //yMaxDomain = yMaxDomain + ((yMaxDomain * 5) / 100);
    //} else {

    //  yMaxDomain = 100;
    //}

     yMaxDomain = this.d3.max(data, d => Number(d.PERCENT))
     //yMaxDomain = yMaxDomain + ((yMaxDomain * 5) / 100);

    return yMaxDomain
  }

  generateYminDomain(data: valueModel[]) {
    var yMinDomain: number = 0;
    //var yMinDomain = this.d3.min(data, d => Number(d.Value))
    //yMinDomain = yMinDomain - ((yMinDomain * 5) / 100)
    return yMinDomain
  }

  generateHearderList(data) {
    let newData = [];
    for (var key in data[0]) {
      if (key != 'FY' && key != 'catagory' && key != 'Value') {
        newData.push(key)
      }

    }
    return newData
  }


}


export interface ChartData {
  Values: Array<valueModel>;
  Legend: Array<legendModel>
}


export interface valueModel {
  catagory: string;
  PERCENT: number;
  //Value2: number;
  //Value3: number;
}

export interface legendModel {
  legend: string;
  color: string;

}



