import { Component, OnInit, OnChanges, ElementRef, Input, ViewChild, ViewEncapsulation, SimpleChanges } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import * as D3 from 'd3';

@Component({
  selector: 'stack-bar-line',
  templateUrl: './stack-bar-line.component.html',
  styleUrls: ['./stack-bar-line.component.css']
})
export class StackBarLineComponent implements OnInit, OnChanges {

  constructor(
    private http: HttpService
  ) { }


  @Input() title: string = "Simple";
  @Input("ID") ID: string;
  @Input() groupKey: Array<any> = ["PASS", "FAIL"];
  @Input() LegenList: any = [{ legend: "PASSsssss", color: '#99cc00' }, { legend: "FAILlllll", color: '#c00000' }]
  @Input() Data: any = [];
  @Input() Width: any;
  @Input() Heigth: any;
  @Input() showPercent: string='';

  private svg: any;
  private yScale: any;
  private yScale1: any;
  private xScale: any;
  private xAxis: any;
  private yAxis: any;
  private yAxis1: any;
  private chart: any;
  private z: any;
  private barGroups: any;
  private data: any;
  private margin = { top: 30, right: 20, bottom: 20, left: 35 }
  private width = 600 - this.margin.left - this.margin.right
  private height = 400 - this.margin.top - this.margin.bottom;
  public layers: any;
  public delay: number = 0

  private d3: any = D3;

  ngOnInit() {

    if (this.Width) {
      this.width = this.Width - this.margin.left - this.margin.right
    }
    if (this.Heigth) {
      this.height = this.Heigth - this.margin.top - this.margin.bottom
    }

    if (this.Data.length == 0) {
      this.Data = [{ 'catagory': 'Apr', 'PASS': 150, 'FAIL': 20, 'PERCENTAGE': 99 },
      { 'catagory': 'May', 'PASS': 178, 'FAIL': 20, 'PERCENTAGE': 99 },
      { 'catagory': 'Jun', 'PASS': 188, 'FAIL': 11, 'PERCENTAGE': 99 },
      { 'catagory': 'Jul', 'PASS': 180, 'FAIL': 18, 'PERCENTAGE': 98 },
      { 'catagory': 'Aug', 'PASS': 160, 'FAIL': 33, 'PERCENTAGE': 99 },
      { 'catagory': 'Sep', 'PASS': 178, 'FAIL': 22, 'PERCENTAGE': 97 },
      { 'catagory': 'Oct', 'PASS': 160, 'FAIL': 40, 'PERCENTAGE': 99 },
      { 'catagory': 'Nov', 'PASS': 169, 'FAIL': 31, 'PERCENTAGE': 98 },
      { 'catagory': 'Dec', 'PASS': 175, 'FAIL': 25, 'PERCENTAGE': 100 },
      { 'catagory': 'Jan', 'PASS': 195, 'FAIL': 5, 'PERCENTAGE': 97 },
      { 'catagory': 'Feb', 'PASS': 166, 'FAIL': 34, 'PERCENTAGE': 98 },
      { 'catagory': 'Mar', 'PASS': 183, 'FAIL': 17, 'PERCENTAGE': 99 }
      ]
    }
  }

  ngOnChanges(changes: SimpleChanges): void {

    if (!changes["Data"].isFirstChange()) {
      //this.delay = 100
      //this.getData()
      this.d3.select("#" + 'stack' + this.ID).remove();
      this.setUpChart(this.ID)
    }

  }

  ngAfterViewInit() {
    this.setUpChart(this.ID)
  }



  setUpChart(id) {

    this.svg = this.d3.select("#" + id).append('svg')
      .attr("id", 'stack' + id)
      .attr('width', this.width + this.margin.left + this.margin.right)
      .attr('height', this.height + this.margin.top + this.margin.bottom + 20)
      .style("background", "rgb(255, 255, 255");


    this.xScale = this.d3.scaleBand()
      .range([this.margin.left, this.width - this.margin.right])
      .padding(0.3)

    this.yScale = this.d3.scaleLinear()
      .rangeRound([this.height - this.margin.bottom, this.margin.top])

    this.yScale1 = this.d3.scaleLinear()
      .range([this.height - this.margin.bottom, this.margin.top])
    //.domain([0, 120])

    this.xAxis = this.d3.axisBottom(this.xScale);

    this.yAxis = this.d3.axisLeft(this.yScale);

    this.yAxis1 = this.d3.axisRight(this.yScale1);

    this.chart = this.svg.append('g')
      .attr('transform', `translate(${this.margin.left / 2}, ${this.margin.top + 30})`);

    this.chart.append('g')
      .attr("class", "x-axis")
      .attr("transform", `translate(0,${this.height - this.margin.bottom})`)
      .call(this.xAxis)

    this.chart.append('g')
      .attr("class", "y-axis")
      .attr("transform", `translate(${this.margin.left},0)`)
      .call(this.yAxis)

    this.chart.append('g')
      .attr("class", "y-axis1")
      .attr("transform", `translate(${this.width - this.margin.right},0)`)
      .call(this.yAxis1);

    this.z = this.d3.scaleOrdinal()
      .range([this.LegenList[0].color, this.LegenList[1].color])
      .domain(this.groupKey);

    //this.getData();
    this.createChart(this.Data)
  }

  createChart(gData) {

    var max = this.generateYmaxDomain(gData)
    this.yScale.domain([0, this.d3.max(gData, d => this.d3.sum(this.groupKey, k => +d[k]) + 10)]).nice();


    this.chart.selectAll(".y-axis").transition().duration(5)
      .call(this.d3.axisLeft(this.yScale).ticks(5))

    this.xScale.domain(gData.map(d => d.catagory));

    this.chart.selectAll(".x-axis").transition().duration(5)
      .call(this.d3.axisBottom(this.xScale).tickSizeOuter(0))


    this.yScale1.domain([0, 110]);

    this.chart.selectAll(".y-axis1").transition().duration(5)
      .call(this.d3.axisRight(this.yScale1).ticks(5))

    var bargroup = this.chart.selectAll("g.layer")
      .data(this.d3.stack().keys(this.groupKey)(gData), d => d.key)

    bargroup.exit().remove();

    bargroup.enter().insert("g", ".y-axis").append("g")
      .classed("layer", true)
      .attr("fill", d => this.z(d.key));

    var bars = this.chart.selectAll("g.layer").selectAll("rect")
      .data(d => d, e => e.data.catagory);

    bars.exit().remove();

    bars.enter().append("rect")
      .merge(bars)
      .transition().duration(this.delay)
      .attr("x", d => this.xScale(d.data.catagory))
      .attr("y", d => this.yScale(d[1]))
      .attr("width", this.xScale.bandwidth())
      .attr("height", d => {
        return (this.yScale(d[0]) - this.yScale(d[1]))
      })


    bars.enter().append("text")
      .text((d) => {    
        return (d[1] - d[0]) > 0 ? this.d3.format(".2f")(d[1] - d[0]) + ' ' + this.showPercent : ''
      })
      .transition().duration(this.delay)
      .attr("y", (d) => (this.yScale(d[1]) + (this.yScale(d[0]) - this.yScale(d[1])) / 2) + 10)
      .attr('x', (d) => (this.xScale(d.data.catagory) + this.xScale.bandwidth() / 2))
      .attr('text-anchor', 'middle')
      .attr("font-size", "8pt")
      .style("font-weight", 'bold')
      .style("fill", (d) => "#000000");



    var line = this.d3.line()
      .x((d, i) => this.xScale(d.catagory) + this.xScale.bandwidth() / 2)
      .y((d) => this.yScale1(d.PERCENTAGE))

    this.chart.append("path")
      .attr("fill", "none")
      .attr("stroke-width", 1.5)
      .attr("stroke", this.LegenList[2].color)
      .attr("d", line(gData));

    this.chart.selectAll("dot")
      .data(gData)
      .enter()
      .append("circle")
      .style("fill", this.LegenList[2].color)
      .attr("r", 4)
      .attr("cx", (d) => (this.xScale(d.catagory) + this.xScale.bandwidth() / 2))
      .attr("cy", (d) => this.yScale1(d.PERCENTAGE))


    var text = this.svg.selectAll(".text")
      .data(gData, d => d.catagory);


    text.exit().remove();

    text.enter().append("text")
      .attr("class", "text")
      .attr("text-anchor", "middle")
      .attr("font-size", "8pt")
      .merge(text)
      .transition().duration(5)
      .attr("x", d => (this.xScale(d.catagory) + this.xScale.bandwidth() / 2) + 20)
      .attr("y", d => (this.yScale1(d.PERCENTAGE) + this.margin.top + 15))
      .text(d => d.PERCENTAGE > 0 ? d.PERCENTAGE + '%' : '')

    this.svg.select('.texttitle').remove()

    this.chart.append('text')
      .attr('class', 'texttitle')
      .attr('x', this.width / 2)
      .attr('y', -35)
      .attr("font-size", "20px")
      .attr('text-anchor', 'middle')
      .style("font-weight", 'bold')
      .style("fill", "#595959")
      .text(this.title)


    let legendList = this.LegenList
    let height = this.height;
    let width = this.width;

    var nodeWidth = (d) => d.getBBox().width;
    //const cScale = this.d3.scaleOrdinal();

    const legend = this.svg.append('g')
      .attr('class', 'legend')
      .attr('transform', 'translate(0,0)');

    const lg = legend.selectAll('g')
      .data(legendList)
      .enter()
      .append('g')
      .attr('transform', (d, i) => `translate(${i * 100},${height + 15})`);

    lg.append('rect')
      .style('fill', d => d.color)
      .attr('x', -5)
      .attr('y', 90)
      .attr('width', 30)
      .attr('height', 10);

    lg.append('text')
      .style('font-size', '10px')
      .attr('x', 28)
      .attr('y', 100)
      .text(d => d.legend);

    let offset = 0;
    lg.attr('transform', function (d, i) {
      let x = offset;
      offset += nodeWidth(this) + 10;
      return `translate(${x},${-40})`;
    });

    legend.attr('transform', function () {
      return `translate(${(width - nodeWidth(this)) / 2},${0})`
    });

  }




  generateYmaxDomain(data: valueModel[]) {

    var yMaxDomain: number;

    yMaxDomain = this.d3.max(data, d => Number(d.PERCENTAGE))

    return yMaxDomain
  }

  generateYminDomain(data: valueModel[]) {
    var yMinDomain: number = 0;
    //var yMinDomain = this.d3.min(data, d => Number(d.Value))
    //yMinDomain = yMinDomain - ((yMinDomain * 5) / 100)
    return yMinDomain
  }

  generateHearderList(data) {
    let newData = [];
    for (var key in data[0]) {
      if (key != 'FY' && key != 'catagory' && key != 'Value') {
        newData.push(key)
      }

    }
    return newData
  }


}


export interface ChartData {
  Values: Array<valueModel>;
  Legend: Array<legendModel>
}


export interface valueModel {
  catagory: string;
  PERCENTAGE: number;
  //Value2: number;
  //Value3: number;
}

export interface legendModel {
  legend: string;
  color: string;

}



