import { Component, ContentChild, TemplateRef, ViewChild, Inject, Input } from '@angular/core';
import { MatDialog, MatDialogRef, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { Overlay } from '@angular/cdk/overlay';

@Component({
    selector: 'dialog-template-caller',
    template: '<span></span>'
})
export class DialogTemplateCallerComponent {

    @Input() iconName: string = "edit";
    @Input() title: string = "Dialog";
    @Input() width: string = "75%";
    @Input() minWidth: string = "auto";
    @Input() maxWidth: string = "auto";
    @Input() height: string = "100%";
    @Input() minHeight: string = "auto";
    @Input() maxHeight: string = "auto";
    @ContentChild(TemplateRef, { static: false }) templateRef;
    private dialogRef: any;
    
    constructor(
        private dialog: MatDialog,
        private overlay: Overlay
    ) { }

    OpenDialog() {
        this.dialogRef = this.dialog.open(DialogTemplateCallerDialog, {
            autoFocus: false,
            width: this.width,
            height: this.height,
            minHeight: this.minHeight,
            maxHeight: this.maxHeight,
            minWidth: this.minWidth,
            maxWidth: this.maxWidth,
            data: {
                title: this.title,
                templateRef: this.templateRef
            }
        })

        this.dialogRef.afterClosed().subscribe(result => { })
    }

    close() {
        this.dialogRef.close();
    }

}

@Component({
    selector: 'dialog-caller-dialog',
    templateUrl: './dialog-template-caller.component.html',
    styleUrls: ['./dialog-template-caller.component.css']
})
export class DialogTemplateCallerDialog {
    public templateRef: TemplateRef<any>;
    public title: string;

    constructor(
        public dialogRef: MatDialogRef<DialogTemplateCallerDialog>,
        @Inject(MAT_DIALOG_DATA) public dialogData: any
    ) { }

    ngOnInit() {
        this.title = this.dialogData.title;
        this.templateRef = this.dialogData.templateRef;
    }

    OnCloseClick() {
        this.dialogRef.close();
    }
}