import { Injectable, Optional } from '@angular/core';
import { CanActivate, Router, CanLoad, ActivatedRouteSnapshot, RouterStateSnapshot, Route, UrlSegment } from '@angular/router';
import { JwtHelperService } from '@auth0/angular-jwt';
import { AuthService } from './auth-service.service';
import { HttpService } from './http.service';
import { map, catchError } from 'rxjs/operators';
import { of } from 'rxjs';

@Injectable()
export class AuthGuard implements CanActivate {
    private auth: AuthService;

    constructor(
        private jwtHelper: JwtHelperService,
        private router: Router,
        @Optional() private http: HttpService
    ) {
        this.auth = new AuthService(this.jwtHelper, this.router, this.http);
    }

    canActivate(next: ActivatedRouteSnapshot, state: RouterStateSnapshot) {
        return this.auth.checkToken().pipe(
            map(e => {
                if (e) {
                    return true;
                } else {
                    this.auth.clearToken();
                    this.router.navigate(['/login'])
                    return false;
                }
            }),
            catchError(() => {
                this.auth.clearToken();
                this.router.navigate(['/login'])
                return of(false);
            })
        )
    }
}

@Injectable()
export class AuthGuardWithRefresh implements CanActivate {
    private auth: AuthService;

    constructor(
        private jwtHelper: JwtHelperService,
        private router: Router,
        @Optional() private http: HttpService
    ) {
        this.auth = new AuthService(this.jwtHelper, this.router, this.http);
    }

    canActivate(next: ActivatedRouteSnapshot, state: RouterStateSnapshot) {
        return this.auth.refreshToken().pipe(
            map(e => {
                if (e) {
                    return true;
                } else {
                    this.auth.clearToken();
                    this.router.navigate(['/login'])
                    return false;
                }
            }),
            catchError(() => {
                console.log("err");
                
                this.auth.clearToken();
                this.router.navigate(['/login'])
                return of(false);
            })
        )
    }
}

@Injectable()
export class AuthLoggedIn implements CanActivate {
    private auth: AuthService;

    constructor(
        private jwtHelper: JwtHelperService,
        private router: Router,
        @Optional() private http: HttpService
    ) {
        this.auth = new AuthService(this.jwtHelper, this.router, this.http);
    }

    canActivate(): boolean {
        if (this.auth.isAuthenticated()) {
            this.router.navigate(['/portal'])
            return false;
        }
        return true;
    }
}

@Injectable()
export class AuthLoad implements CanLoad {
    private auth: AuthService;

    constructor(
        private jwtHelper: JwtHelperService,
        private router: Router,
        @Optional() private http: HttpService
    ) {
        this.auth = new AuthService(this.jwtHelper, this.router, this.http);
    }

    canLoad(route: Route, segments: UrlSegment[]) {
        return this.auth.checkToken().pipe(
            map(e => {
                if (e) {
                    return true;
                } else {
                    this.auth.clearToken();
                    this.router.navigate(['/login'])
                    return false;
                }
            }),
            catchError(() => {
                this.auth.clearToken();
                this.router.navigate(['/login'])
                return of(false);
            })
        )
    }
}