import { Injectable, Inject, Optional } from '@angular/core';
import { HttpService } from './http.service';
import { Router, NavigationExtras } from '@angular/router';
import { AuthDecode } from './auth-decode.service';
import { FormRole } from './role-resolver.service';
import { AuthService } from './auth-service.service';
import moment, { Moment } from 'moment';
// import { RoleService } from './role.service';

@Injectable()
export class Globals {
  constructor(
    @Optional() private http: HttpService,
    @Optional() private router: Router,
    @Optional() private authDecode: AuthDecode,
    @Optional() private auth: AuthService
  ) { }

  public DateFormat: string = "dd-MMM-yyyy"
  public DateTimeFormat: string = "dd-MMM-yyyy hh:mm:ss a"
  private RoleDetail: FormRole;

  findUnique(value, index, self, key) {
    return self.findIndex(x => x[key] === value[key]) === index;
  }

  GoToUrl(url) {
    window.open(url, "_blank");
  }

  public CompareDate(date1: Date, date2: Date): boolean {
    date1 = date1 ? new Date(date1) : null;
    date2 = date2 ? new Date(date2) : null;

    date1 ? date1.setHours(0, 0, 0, 0) : null;
    date2 ? date2.setHours(0, 0, 0, 0) : null;

    return date1 === date2;
  }

  CurrentDate() {

    let d = new Date();
    let Day = d.getDate().toString();
    let Month = d.getMonth() + 1;
    let NewMonth = Month.toString().length < 2 ? '0' + Month.toString() : Month.toString()
    let Year = d.getFullYear().toString();
    let hours = d.getHours();
    let minute = d.getMinutes();
    let milli = d.getMilliseconds();

    return Year + NewMonth + Day + '_' + hours + ':' + minute + ':' + milli;
  }

  GoToReport(reportName: string, parameters: any, newTab: boolean = true) {
    let navigationExtras: NavigationExtras = {
      queryParams: parameters
    }

    // EXCELOPENXML
    // pdf

    if (newTab) {

      let url: string = this.router.createUrlTree(["report", reportName], navigationExtras).toString();

      window.open(url, '_blank');

    } else {


      this.router.navigate(["report", reportName], navigationExtras)
    }
  }


  ConvertDateToFormatSTD(pDate: Moment) {
    if (moment(pDate).isValid()) {
      return moment(pDate).format('YYYY-MM-DD');
    }
    return null;
  }


  GoToExport(reportName: string, parameters: any, newTab: boolean = true, exportTo: string = '') {
    let navigationExtras: NavigationExtras = {
      queryParams: parameters
    }

    // EXCELOPENXML
    // pdf

    if (newTab) {
      if (exportTo != '') {
        let url: string = this.router.createUrlTree(["report", exportTo, reportName], navigationExtras).toString();
        window.open(url, '_blank');
      }
    } else {
      this.router.navigate(["report", reportName], navigationExtras)
    }
  }

  GetTokenDetail(claim: string) {
    let tokenDetail
    
    tokenDetail = this.authDecode.getDetail(false);
    
    return tokenDetail[claim];
  }

  SetRoleDetail(role: FormRole) {
    this.RoleDetail = role;
  }

  GetRoleDetail() {
    return this.RoleDetail;
  }

  escapeCharacter(data: string): string {
    return data.replace("%", "[percnt]");
  }


  ClearObject(Object?) {

    for (var key in Object) {
      if (Object.hasOwnProperty(key)) {
        if (Array.isArray(Object[key])) {
          Object[key] = [];
        }
        else if (Number.isInteger(Object[key])) {
          Object[key] = 0;
        }
        else if ((Object[key] instanceof Date)) {
          Object[key] = new Date();
        }
        else {
          Object[key] = null;
        }
      }
    }
    return Object;
  }

}
