import { HttpClient, HttpHeaders, HttpErrorResponse, HttpResponse, HttpParams } from '@angular/common/http';
import { Injectable, Optional, Inject } from "@angular/core";
import { Observable, throwError, of } from 'rxjs';
import { catchError, retry, map, tap, finalize } from 'rxjs/operators';
import { environment } from './../../environments/environment';
import { LoaderService } from './loader.service';

@Injectable()
export class HttpService {
    private origin: string = environment.apiLocation;
    // private origin: string = window.location.protocol + "//" + window.location.hostname + ":8090/api/";

    constructor(
        private http: HttpClient,
        @Inject(LoaderService) private loaderService: LoaderService
    ) { }


    get(path: string, method: string): Observable<any>;
    get(path: string, method: string, id: string): Observable<any>;
    get(path: string, method: string, id: string, id2: string): Observable<any>;
    get(path: string, method: string, id: string, id2: string, id3: string): Observable<any>;
    get(path: string, method: string, id: string, id2: string, id3: string, id4: string): Observable<any>;
    get(path: string, method: string, id: string, id2: string, id3: string, id4: string, id5: string): Observable<any>;
    get(path: string, method: string, id: string, id2: string, id3: string, id4: string, id5: string, id6: string): Observable<any>;
    get(path: string, method: string, id: string, id2: string, id3: string, id4: string, id5: string, id6: string, id7: string): Observable<any>;
    get(path: string, method: string, id: string, id2: string, id3: string, id4: string, id5: string, id6: string, id7: string, id8: string): Observable<any>;
    get(path: string, method: string, id: string, id2: string, id3: string, id4: string, id5: string, id6: string, id7: string, id8: string, id9: string): Observable<any>;
    get(path: string, method: string, id: string, id2: string, id3: string, id4: string, id5: string, id6: string, id7: string, id8: string, id9: string, id10: string): Observable<any>;
    get(path: string, method: string, id?: string, id2?: string, id3?: string, id4?: string, id5?: string, id6?: string, id7?: string, id8?: string, id9?: string, id10?: string): Observable<any> {

        const httpOptions = {}

        let pathStr: string;

        if (!id) {
            pathStr = this.origin + path + '/' + method;
        } else {
            pathStr = this.origin + path + '/' + method + '/' + id;
            if (id2) pathStr = pathStr.concat('/' + id2);
            if (id3) pathStr = pathStr.concat('/' + id3);
            if (id4) pathStr = pathStr.concat('/' + id4);
            if (id5) pathStr = pathStr.concat('/' + id5);
            if (id6) pathStr = pathStr.concat('/' + id6);
            if (id7) pathStr = pathStr.concat('/' + id7);
            if (id8) pathStr = pathStr.concat('/' + id8);
            if (id9) pathStr = pathStr.concat('/' + id9);
            if (id10) pathStr = pathStr.concat('/' + id10);
        }

        this.showLoader();
        return this.http.get(pathStr, httpOptions)
            .pipe(
                catchError(this.handleError),
                finalize(() => { this.hideLoader() })
            )
    };

    getParams(path: string, method: string | string[], params: any): Observable<any> {
        let _method = (method instanceof Object) ? method.join('/') : method;
        let pathStr: string = this.origin + path + '/' + _method;

        this.showLoader();
        return this.http.get(pathStr, { params: params })
            .pipe(
                catchError(this.handleError),
                finalize(() => { this.hideLoader() })
            )
    }

    post(path: string, method: string | string[], data: any): Observable<any> {
        const httpOptions = {
            headers: new HttpHeaders({
                'Content-Type': 'application/json'
            })
        }

        let _method = (method instanceof Object) ? method.join('/') : method;

        this.showLoader();
        return this.http
            .post(this.origin + path + '/' + _method, data, httpOptions)
            .pipe(
                catchError(this.handleError),
                finalize(() => { this.hideLoader() })
            )
    }

    patch(path: string, method: string, id: string | number, data: any): Observable<any> {
        const httpOptions = {
            headers: new HttpHeaders({
            })
        }

        this.showLoader();
        return this.http
            .patch(this.origin + path + '/' + method + '/' + id.toString(), data, httpOptions)
            .pipe(
                catchError(this.handleError),
                finalize(() => { this.hideLoader() })
            )
    }

    postFile(path: string, method: string, data: any): Observable<any> {
        const httpOptions = {
            headers: new HttpHeaders({
                "enctype": "multipart/form-data"
            })
        }

        this.showLoader();
        return this.http
            .post(this.origin + path + '/' + method, data, httpOptions)
            .pipe(
                catchError(this.handleError),
                finalize(() => { this.hideLoader() })
            )
    }

    postLogin(username: string, password: string, company: string): Observable<any> {
        const httpOptions = {
            headers: new HttpHeaders({
                'Content-Type': 'application/json'
            })
        }

        let data = { username, password, company }

        return this.http
            .post(this.origin + "System/Login", data, httpOptions)
            .pipe(
                catchError(this.handleError)
            )
    }

    postChangePwd(username: string, oldpassword: string, newpassword: string, confirmpassword: string): Observable<any> {
        const httpOptions = {
            headers: new HttpHeaders({
                'Content-Type': 'application/json'
            })
        }

        let data = { username, oldpassword, newpassword, confirmpassword }

        return this.http
            .post(this.origin + "System/ChangePwd", data, httpOptions)
            .pipe(
                catchError(this.handleError)
            )
    }

    GetFormRole(formId: string) {
        const httpOptions = {
            headers: new HttpHeaders({
                'Content-Type': 'application/json'
            })
        }

        return this.http.get<any>(this.origin + "System/GetFormUserRole/" + formId)
    }




    
    DownloadMultipleFilesAsZip(path: string, method: string, data: any) {

        var headers = {
            "Authorization": "Bearer " + this.getToken(),
            'Content-Type': 'application/json',
        }

        this.showLoader();
        return this.http.post(this.origin + path + '/' + method, data, { responseType: 'arraybuffer', headers: headers })
            .pipe(
                catchError(this.handleError),
                finalize(() => { this.hideLoader() })
            )
    }

    writeContents(content, model) {
        const a = document.createElement('a');
        const file = new Blob([content], { type: content.type });
        a.href = URL.createObjectURL(file);
        a.download = model.FILE_NAME;
        a.click();
    }

    RefreshToken(token: string, refreshToken: string) {
        const httpOptions = {
            headers: new HttpHeaders({
                'Content-Type': 'application/json'
            })
        }

        let data = { token, refreshToken }

        return this.http
            .post(this.origin + "token/Refresh", data, httpOptions)
    }

    CheckToken(token: string, refreshToken: string) {
        const httpOptions = {
            headers: new HttpHeaders({
                'Content-Type': 'application/json'
            })
        }

        let data = { token, refreshToken }

        return this.http
            .post(this.origin + "token/Check", data, httpOptions)
    }

    GetSystemDate() {
        this.showLoader();
        return this.http.get(this.origin + "System/GetCurrentDate")
            .pipe(
                map((result: any) => {
                    if (result && result.currentDate) {
                        return result.currentDate;
                    }
                }),
                catchError(this.handleError),
                finalize(() => { this.hideLoader() })
            );
    }

    /**
     * Auto complete
     */

    autoCompleteGet(path: string, method: string): Observable<any>;
    autoCompleteGet(path: string, method: string, id: string): Observable<any>;
    autoCompleteGet(path: string, method: string, id: string, id2: string): Observable<any>;

    autoCompleteGet(path: string, method: string, id?: string, id2?: string): Observable<any> {

        const httpOptions = {}

        let pathStr: string;

        if (!id) {
            pathStr = this.origin + path + '/' + method;
        } else {
            pathStr = this.origin + path + '/' + method + '/' + id;
            if (id2) pathStr = pathStr.concat('/' + id2);
        }

        return this.http.get(pathStr, httpOptions)
            .pipe(
                catchError(this.handleError)
            )
    };

    autoCompletePost(path: string, method: string | string[], data: any): Observable<any> {
        const httpOptions = {
            headers: new HttpHeaders({
                'Content-Type': 'application/json'
            })
        }

        let _method = (method instanceof Object) ? method.join('/') : method;

        return this.http
            .post(this.origin + path + '/' + _method, data, httpOptions)
            .pipe(
                catchError(this.handleError)
            )
    }
    /**
     * End Auto complete
     */

    private handleError(error: HttpErrorResponse) {
        if (error.error instanceof ErrorEvent) {
            // A client-side or network error occurred. Handle it accordingly.
            console.error('An error occurred:', error.error.message);
        } else {
            // The backend returned an unsuccessful response code.
            // The response body may contain clues as to what went wrong,
            console.error(
                `Backend returned code ${error.status}, ` +
                `body was: ${error.error}`);
        }
        // return an observable with a user-facing error message
        return throwError(error);
    };

    private getToken() {
        return localStorage.getItem("jwtToken");
    }

    private showLoader(): void {
        this.loaderService.show();
    }

    private hideLoader(): void {
        this.loaderService.hide();
    }
    
      postGetReport(path: string, method: string, data: any): Observable<any> {
    const httpOptions = {
      headers: new HttpHeaders({
        'Content-Type': 'application/json'
      }),
      responseType: 'blob' as 'json'
    }

    return this.http
      .post('/api/' + path + '/' + method, data, httpOptions)
      .pipe(
        catchError(this.handleError)
      );
  }
  
   Export(path: string, method: string, model: any) {

    var headers = {
        "Authorization": "Bearer " + this.getToken(),
        'Content-Type': 'application/json',
    }

    return this.http.post(this.origin + path + '/' + method, model, { responseType: 'blob', headers: headers })
        .pipe(
            catchError(this.handleError)
        )

}
UploadFilePath(formData: any, datamonth: any): Observable<any> {

    //const formData = new FormData();
    //formData.append('file', data.get('profile').value);

    const httpOptions = {
      headers: new HttpHeaders({
        "Authorization": "Bearer " + this.getToken(),
        "enctype": "multipart/form-data"
      })
    }

    return this.http
      .post(this.origin + "System/UploadFilePath" + '/' + datamonth, formData, httpOptions)
      .pipe(
        catchError(this.handleError)
      )
  }

  UploadFileData(formData: any, datamonth: any): Observable<any> {

    const httpOptions = {
      headers: new HttpHeaders({
        "Authorization": "Bearer " + this.getToken(),
        "enctype": "multipart/form-data"
      })
    }

    return this.http
      .post(this.origin + "System/UploadFileData" + '/' + datamonth, formData, httpOptions)
      .pipe(
        catchError(this.handleError)
      )
  }

  Download(model: any) {

    var headers = {
      "Authorization": "Bearer " + this.getToken(),
      'Content-Type': 'application/json',
    }

    this.http.post(this.origin + "System/Download", model, { responseType: 'blob', headers: headers })
      .subscribe(res => {
        this.writeContents(res, model);
      })

  }

}
