import { NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { HttpClientModule } from '@angular/common/http';
import { ReactiveFormsModule, FormsModule } from '@angular/forms';
import { RouterModule } from '@angular/router';
import { MatSidenavModule } from '@angular/material/sidenav';
import { MatToolbarModule } from '@angular/material/toolbar';
import { SharedComponentModule } from '../shared/shared-component.module';
import { AuthGuard } from 'src/app/core/auth-guard.service';
import { MatIconModule } from '@angular/material/icon';
import { DhAndRptComponent } from '../../pages/dh-and-rpt/dh-and-rpt.component';
import { CsiTargetComponent } from '../../pages/dh-and-rpt/CSI-TARGET/CSI-TARGET.component';
import { SsiTargetComponent } from '../../pages/dh-and-rpt/SSI-TARGET/SSI-TARGET.component';
import { SsiCsiNationwideComponent } from '../../pages/dh-and-rpt/SSI-CSI-Nationwide/SSI-CSI-Nationwide.component';
import { SsiCsiQualifiersComponent } from '../../pages/dh-and-rpt/SSI-CSI-Qualifiers-Outlet-Pass/SSI-CSI-Qualifiers.component';
import { SsiCsiQualifiersNoOutletPassComponent } from '../../pages/dh-and-rpt/SSI-CSI-Qualifiers-No-Outlet-Pass/SSI-CSI-Qualifiers-No-Outlet-Pass.component';
import { SsiCsiSelfAssesmentComponent } from '../../pages/dh-and-rpt/SSI-CSI-Self-Assessment/SSI-CSI-Self-Assessment.component';
import { SsiCsiQuickVocComponent } from '../../pages/dh-and-rpt/SSI-CSI-Quick-VOC/SSI-CSI-Quick-VOC.component';
import { SsiCsiManpowerComponent } from '../../pages/dh-and-rpt/SSI-CSI-Manpower/SSI-CSI-Manpower.component';
import { SsiCsiAvgDrComponent } from '../../pages/dh-and-rpt/SSI-CSI-AVG-NPS-DR/SSI-CSI-AVG-NPS-DR.component';
import { SsiCsiAvgSop1Component } from '../../pages/dh-and-rpt/SSI-CSI-AVG-SOP1/SSI-CSI-AVG-SOP1.component';
import { SsiCsiAvgSop2Component } from '../../pages/dh-and-rpt/SSI-CSI-AVG-SOP2/SSI-CSI-AVG-SOP2.component';
import { SsiCsiAvgSop3Component } from '../../pages/dh-and-rpt/SSI-CSI-AVG-SOP3/SSI-CSI-AVG-SOP3.component';
import { SsiCsiAvgSop4Component } from '../../pages/dh-and-rpt/SSI-CSI-AVG-SOP4/SSI-CSI-AVG-SOP4.component';
import { SsiCsiPaymentPageComponent } from '../../pages/dh-and-rpt/SSI-CSI-PAYMENT/SSI-CSI-PAYMENT.component';
import { RPTComponent } from 'src/app/pages/dh-and-rpt/RPT/RPT.component';
import { AllComponent } from '../../pages/dh-and-rpt/ALL/ALL.component';



@NgModule({
  declarations: [
    DhAndRptComponent,
    CsiTargetComponent,
    SsiTargetComponent,
    SsiCsiNationwideComponent,
    SsiCsiQualifiersComponent,
    SsiCsiQualifiersNoOutletPassComponent,
    SsiCsiSelfAssesmentComponent,
    SsiCsiQuickVocComponent,
    SsiCsiManpowerComponent,
    SsiCsiAvgDrComponent,
    SsiCsiAvgSop1Component,
    SsiCsiAvgSop2Component,
    SsiCsiAvgSop3Component,
    SsiCsiAvgSop4Component,
    SsiCsiPaymentPageComponent,
        RPTComponent,
        AllComponent
  ],
  imports: [
    CommonModule,
    HttpClientModule,
    FormsModule,
    ReactiveFormsModule,
    SharedComponentModule,
    FormsModule,
    MatSidenavModule,
    MatToolbarModule,
    MatIconModule,

    RouterModule.forChild([
      {
        path: '',
        component: DhAndRptComponent,
        children: [
          {
            path: 'CSI_TARGET',
            component: CsiTargetComponent,
          },
          {
            path: 'SSI_TARGET',
            component: SsiTargetComponent,
          },
          {
            path: 'SSI_CSI_Nationwide',
            component: SsiCsiNationwideComponent,
          },
          {
            path: 'SSI_CSI_Qualifiers',
            component: SsiCsiQualifiersComponent
          },
          {
            path: 'SSI_CSI_Qualifiers_No_Outlet_Pass',
            component : SsiCsiQualifiersNoOutletPassComponent
          },
          {
            path: 'SSI_CSI_SELF_ASSESMENT',
            component: SsiCsiSelfAssesmentComponent
          },
          {
            path: 'SSI_CSI_Quick_VOC',
            component : SsiCsiQuickVocComponent
          },
          {
            path: 'SSI_CSI_Manpower',
            component : SsiCsiManpowerComponent
          },
          {
            path: 'SSI_CSI_AVG_DR',
            component : SsiCsiAvgDrComponent
          },
          {
            path: 'SSI_CSI_SOP1',
            component: SsiCsiAvgSop1Component
          },
          {
            path: 'SSI_CSI_SOP2',
            component: SsiCsiAvgSop2Component
          },
          {
            path: 'SSI_CSI_SOP3',
            component: SsiCsiAvgSop3Component
          },
          {
            path: 'SSI_CSI_SOP4',
            component: SsiCsiAvgSop4Component
          },
          {
            path: 'SSI_CSI_PAYMENT',
            component: SsiCsiPaymentPageComponent
          },
          {
            path: 'report',
            component: RPTComponent
            },
            {
                path: 'ALL',
                component : AllComponent
            }

        ],
        canActivate: [AuthGuard]
      }
    ])
  ],
    exports: [],
  providers: [AuthGuard],
})
export class DhAndRptModule { }
