﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Net;
using System.ServiceModel;
using System.Threading.Tasks;
// using Apollo.aControl;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Newtonsoft.Json.Linq;
using RSExeService;

namespace SYS.Controllers
{
  [Route("api/[controller]")]
  public class ReportController : Controller
  {

    public IConfiguration config { get; set; }
    public ReportController(IConfiguration configuration)
    {
      // odb = new DBManager(configuration);
      config = configuration;
    }

    // private DBManager odb;
   // private String gSQL = "";

    [HttpPost("GetReport/{pType}/{pReportName}")]
    public IActionResult GetReport([FromRoute] string pType, [FromRoute] string pReportName, [FromBody] ParameterValue[] pParameters)
    {
      var render = RenderReportAsync(config["reportEndpoint"]
              , config["reportEndpointUsername"]
              , config["reportEndpointUserPassword"]
              , config["reportLocation"]
              , pReportName
              , pType
              , pParameters);

      if (render.Result != null)
      {
        return File(render.Result, "application/octet-stream");
      }
      else
      {
        return StatusCode(500);
      }

    }

    private static async Task<byte[]> RenderReportAsync(string endpoint, string endpointUsername, string endpointPassword, string reportLocation, string reportPath, string type, ParameterValue[] parameters)
    {
      reportPath = String.Concat(reportLocation, reportPath);

      BasicHttpBinding rsBinding = new BasicHttpBinding();
      rsBinding.Security.Mode = BasicHttpSecurityMode.TransportCredentialOnly;
      rsBinding.Security.Transport.ClientCredentialType = HttpClientCredentialType.Ntlm;
      rsBinding.MaxReceivedMessageSize = Int32.MaxValue;
      rsBinding.MaxBufferSize = Int32.MaxValue;

      // Add Endpoint address
      EndpointAddress rsEndpointAddress = new EndpointAddress(endpoint);

      // Instantiate the Soap client
      ReportExecutionServiceSoapClient rsExec = new ReportExecutionServiceSoapClient(rsBinding, rsEndpointAddress);
      // Create a network credential object with the appropriate username and password used
      // to access the SSRS web service
      NetworkCredential clientCredentials = new NetworkCredential(endpointUsername, endpointPassword);

      // Here, for testing purposes, I use impersonation for a local account on the SSRS server
      // rsExec.ClientCredentials.Windows.AllowedImpersonationLevel = System.Security.Principal.TokenImpersonationLevel.Impersonation;
      rsExec.ClientCredentials.Windows.ClientCredential = clientCredentials;

      string historyID = null;

      TrustedUserHeader trustedUserHeader = new TrustedUserHeader();
      ExecutionHeader execHeader = new ExecutionHeader();

      // Here we call the async LoadReport() method using the "await" keyword, which means any code below this method
      // will not execute until the result from the LoadReportAsync task is returned
      LoadReportResponse taskLoadReport = await rsExec.LoadReportAsync(trustedUserHeader, reportPath, historyID);
      // By the time the LoadReportAsync task is returned successfully, its "executionInfo" property
      // would have already been populated. Now the remaining code in this main thread will resume executing
      if (taskLoadReport.executionInfo != null)
      {
        execHeader.ExecutionID = taskLoadReport.executionInfo.ExecutionID;
      }

      string deviceInfo = null;
      string format = type; //EXCELOPENXML, MHTML

      await rsExec.SetExecutionParametersAsync(execHeader, trustedUserHeader, parameters, "en-us");

      // Now, similar to the above task, we will call the RenderAsync() method and await its result
      RenderRequest renderReq = new RenderRequest(execHeader, trustedUserHeader, format, deviceInfo);
      RenderResponse taskRender = await rsExec.RenderAsync(renderReq);
      // When the result is returned and if it is successfull, the rendered report's byte[] should be available

      return taskRender.Result;
    }
  }
}